/*
 * $Id: beacon.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <iostream>
#include <cstring>
#include <string>
#include <limits>

#include "saratoga.h"
#include "screen.h"
#include "sarflags.h"
#include "frame.h"
#include "beacon.h"

using namespace std;

namespace saratoga 
{

beacon::beacon( enum f_descriptor d,	// Descriptor Size
	enum f_streams s,	// Are we a streams
	enum f_txwilling t,	// Can we transmit
	enum f_rxwilling r,	// Can we receive
	string eid)		// EID of the peer as a string
{
	uint32_t	tmp_32;
	sarwin::screen	msg;

	/*
	 * These are the valid flags types applicable
	 * to a beacon, set them 
	 */
	Fversion	version(F_VERSION_1);
	Fframetype	frametype(F_FRAMETYPE_BEACON); // Of course we are a BEACON
	Fdescriptor	descriptor(d);
	Fbundle		bundle(F_BUNDLE_NO); // We dont handle bundles
	Fstreams		streams(s);
	Ftxwilling	txwilling(t);
	Frxwilling	rxwilling(r);
	Fudptype	udptype(F_UDPONLY); // We only handle UDP at the moment
	Ffreespace	freespace(F_FREESPACE_NO); // No freespace to be advertised
	Ffreespaced	freespaced(F_FREESPACED_16); // Set it to 16 bit for laughs

	/*
	 * Turn on/off the bits appropriate in the
	 * header flags field so our flags are set correctly
	 */	
	_badframe = false;
	_flags = 0;
	version.set(&_flags);
	frametype.set(&_flags);
	descriptor.set(&_flags);
	bundle.set(&_flags);
	streams.set(&_flags);
	txwilling.set(&_flags);
	rxwilling.set(&_flags);
	udptype.set(&_flags);
	freespace.set(&_flags);
	freespaced.set(&_flags);

	// Copy the beacon info to the frame payload
	_freespace = 0;
	_eid = eid;

	_paylen = sizeof(flag_t) + _eid.length() + 1;
	_payload = new char[_paylen];

	char *bptr = _payload;
	tmp_32 = htonl((uint32_t) _flags);
	memcpy(bptr, &tmp_32, sizeof(flag_t));
	bptr += sizeof(flag_t);

	strcpy(bptr, eid.c_str());
}

/*
 * Given an eid and file system info assemble a beacon from the given flags
 */
beacon::beacon( enum f_descriptor d,	// Descriptor Size
	enum f_streams s,		// Are we a streams
	enum f_txwilling t,	// Can we transmit
	enum f_rxwilling r,	// Can we receive
	fsinfo *fspace,	// How much freespace is available we work out the descriptor
	string eid)		// EID of the peer string binary or not
{
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif
	sarwin::screen	msg;
	/*
	 * These are the valid flags types applicable
	 * to a beacon, set them to defaults
	 */
	Fversion	version(F_VERSION_1);
	Fframetype	frametype(F_FRAMETYPE_BEACON); // Of course we are a BEACON
	Fdescriptor	descriptor(d);
	Fbundle		bundle(F_BUNDLE_NO); // We dont handle bundles
	Fstreams		streams(s);
	Ftxwilling	txwilling(t);
	Frxwilling	rxwilling(r);
	Fudptype	udptype(F_UDPONLY); // We only handle UDP at the moment
	Ffreespace	freespace(F_FREESPACE_YES);
	Ffreespaced freespaced(fspace->freespaced());

	/*
	 * Turn on/off the bits appropriate in the
	 * header flags field so our flags are set correctly
	 */	
	_badframe = false;
	_flags = 0;
	version.set(&_flags);
	frametype.set(&_flags);
	descriptor.set(&_flags);
	bundle.set(&_flags);
	streams.set(&_flags);
	txwilling.set(&_flags);
	rxwilling.set(&_flags);
	udptype.set(&_flags);
	freespace.set(&_flags);
	freespaced.set(&_flags);
	_freespace = (uint64_t) fspace->freespace();
	_eid = eid;

	_paylen = sizeof(flag_t);
	_paylen += freespaced.length();
	_paylen += _eid.length() + 1;
	_payload = new char[_paylen];

	// Copy flags
	char *bptr = _payload;
	tmp_32 = htonl((uint32_t) _flags);
	memcpy(bptr, &tmp_32, sizeof(flag_t));
	bptr += sizeof(flag_t);

	switch(freespaced.get())
	{
	case F_FREESPACED_16:
		tmp_16 = htons((uint16_t) _freespace);
		memcpy(bptr, &tmp_16, sizeof(uint16_t));
		bptr += sizeof(uint16_t);
		break;
	case F_FREESPACED_32:
		tmp_32 = htonl((uint32_t) _freespace);
		memcpy(bptr, &tmp_32, sizeof(uint32_t));
		bptr += sizeof(uint32_t);
		break;
	case F_FREESPACED_64:
		tmp_64 = htonll((uint64_t) _freespace);
		memcpy(bptr, &tmp_64, sizeof(uint64_t));
		bptr += sizeof(uint64_t);
		break;
	case F_FREESPACED_128:
#ifdef UINT128_T
		tmp_128 = htonlll((uint128_t) _freespace);
		memcpy(bptr, &tmp_128, sizeof(uint128_t));
		bptr += sizeof(uint128_t);
#else
		msg.error("beacon(frame): Descriptor 128 Bit Size Not Supported\n");
#endif
	}
	strcpy(bptr, eid.c_str());
}

/*
 * We have an input frame contents, get it into a beacon
 */
beacon::beacon(char	*payload,
		size_t	paylen)
{
	extern string printsarflags(flag_t);
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif
	sarwin::screen	msg;

	_badframe = false;
	_paylen = paylen;
	_payload = new char[_paylen];
	memcpy(_payload, payload, _paylen);
	
	// Copy flags
	memcpy(&tmp_32, payload, sizeof(flag_t));
	_flags = (flag_t) ntohl(tmp_32);
	payload += sizeof(flag_t);
	paylen -= sizeof(flag_t);
	
	Fversion	version(_flags);
	Fframetype	frametype(_flags);
	// Check to see we have a beacon
	if (version.get() != F_VERSION_1)
	{
		msg.error("Beacon: Bad Saratoga Version");
		msg.error(printsarflags(_flags));
		_badframe = true;
		return;
	}
	if (frametype.get() != F_FRAMETYPE_BEACON)
	{
		msg.error("Beacon: Not a BEACON frame");
		msg.error(printsarflags(_flags));
		_badframe = true;
		return;
	}

	Ffreespace	freespace(_flags);
	Ffreespaced	freespaced(_flags);

	// If they advertise freespace then get it	
	if (freespace.get() == F_FREESPACE_YES)
	{
		switch(freespaced.get())
		{
		case F_FREESPACED_16:
			memcpy(&tmp_16, payload, sizeof(uint16_t));
			_freespace = (offset_t) ntohs(tmp_16);
			payload += sizeof(uint16_t);
			paylen -= sizeof(uint16_t);
			break;
		case F_FREESPACED_32:
			memcpy(&tmp_32, payload, sizeof(uint32_t));
			_freespace = (offset_t) ntohl(tmp_32);
			payload += sizeof(uint32_t);
			paylen -= sizeof(uint32_t);
			break;
		case F_FREESPACED_64:
			memcpy(&tmp_64, payload, sizeof(uint64_t));
			_freespace = (offset_t) ntohll(tmp_64);
			payload += sizeof(uint64_t);
			paylen -= sizeof(uint64_t);
			break;
		case F_FREESPACED_128:
#ifdef UINT128_T
			memcpy(&tmp_128, payload, sizeof(uint128_t));
			_freespace = (offset_t) ntohlll(tmp_128);
			payload += sizeof(uint128_t);
			paylen -= sizeof(uint128_t);
#else
			msg.error("beacon(frame): Descriptor 128 Bit Size Not Supported\n");
#endif
		}
	}
	// And now the EID
	_eid.append(payload, paylen);
}

// Print out the current flag bits
string
beacon::printflags()
{
	string s("");

	// cout << "FLAGS=" << f << endl;
	s += "\n    BEACON FLAGS         1                   2                   3\n";
	s += "    |0 1 2 3 4 5 6 7|8 9 0 1 2 3 4 5|6 7 8 9 0 1 2 3|4 5 6 7 8 9 0 1|\n";
	s += "    |";
	for (int i = 0; i < 32; i++)
	{
		flag_t tmpf = _flags;
		tmpf <<= i;
		tmpf >>= 31;
		if (tmpf == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	return s;
}

/*
 * Print out the beacon flags and eid
 */
string
beacon::print()
{
	char	tmp[128];
	string s;
	/*
	 * These are the valid flags types applicable
	 * to a beacon
	 */
	if (_badframe)
	{
		s = "Bad Frame: Not a BEACON";
		return(s);
	}
	Fversion	version(_flags);
	Fframetype	frametype(_flags);
	Fdescriptor	descriptor(_flags);
	Fbundle		bundle(_flags);
	Fstreams		streams(_flags);
	Ftxwilling	txwilling(_flags);
	Frxwilling	rxwilling(_flags);
	Fudptype	udptype(_flags);
	Ffreespace	freespace(_flags);
	Ffreespaced	freespaced(_flags);

	s = frametype.print();
	s += this->printflags();
	s += "    ";
	s += version.print();
	s += "\n    ";
	s += descriptor.print();
	s += "\n    ";
	s += bundle.print();
	s += "\n    ";
	s += streams.print();
	s += "\n    ";
	s += txwilling.print();
	s += "\n    ";
	s += rxwilling.print();
	s += "\n    ";
	s += udptype.print();
	s += "\n    ";
	if (freespace.get() ==  F_FREESPACE_YES)
	{
		s += "Freespace (";
		s += freespaced.print();
		sprintf(tmp, "): %" PRIu64 " kB", (uint64_t) _freespace);
		s += tmp;
	}
	else
		s += freespace.print();
	s += "\n    EID: ";

	// Handle the eid it is not necessarily an ascii string so if not print
	// it out in hex
	bool dohex = 0;
	for (size_t i = 0; i < _eid.length(); i++)
	{
		if (!isprint(_eid[i]) && _eid[i] != '\0')
		{	
			dohex = 1;
			break;
		}
	}
	for (size_t i = 0; i < _eid.length(); i++)
	{
		if (dohex)
		{
			// Format the hex string up so we have 16 chars per line
			// and make sure everyone know it is in hex
			if (i == 0)
				s += "0x:";
			if (i % 16 == 0 && i != 0)
				s += "\n           ";
			char	h[4];
			// Last hex char dones not have a :
			if ( i != _eid.length() - 1)
				sprintf(h,"%02X:", (int) _eid[i]);
			else
				sprintf(h,"%02X", (int) _eid[i]);
			s += h;
		}
		else
		{
			// We have an ascii printable string
			s += _eid[i];
		}
	}
	return(s);
}

// These are the flags we can play with in beacons
enum f_txwilling
beacon::txwilling(enum f_txwilling t)
{
	Ftxwilling	txwilling(t);
	txwilling.set(&_flags);
	return txwilling.get();
}

enum f_rxwilling
beacon::rxwilling(enum f_rxwilling r)
{
	Frxwilling	rxwilling(r);
	rxwilling.set(&_flags);
	return rxwilling.get();
}

// What am I set to ?
enum f_txwilling
beacon::txwilling()
{
	Ftxwilling	txwilling(_flags);
	return txwilling.get();
}

enum f_rxwilling
beacon::rxwilling()
{
	Frxwilling	rxwilling(_flags);
	return rxwilling.get();
}


}; // Namespace saratoga

/*
 * $Log: beacon.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.18  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.17  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.16  2013/06/06 23:47:04  chas
 * Sorted out calculations of the available freespace so beacon id now TOTALLY DONE. Tidied up the allocator accordingly so now you only have to pass a pointer to the freespace and it sets the freespace descriptor up accordingly. So you can create beacons without freesoace or with in the allocator.
 *
 * Revision 1.15  2013/06/05 07:20:09  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.14  2013/06/05 01:44:19  chas
 * Sorted out frame's
 *
 * Revision 1.13  2013/06/01 07:32:40  chas
 * Some issues with frame virtualisation. Perhaps this was a bad idea pulling the flags into the frame class... Ill think it over with a couple of scotches (single malt of course).
 *
 * Revision 1.12  2013/06/01 00:16:54  chas
 * Fixed bug with frame conversion
 *
 * Revision 1.11  2013/04/03 05:18:51  chas
 * Major changes yet again to flags. The set function now only needs
 * nnn.set(&_flag) rather than _flag = nnn.set(_flag)
 * much neater and shorter and less eror prone.
 * Still issues with dirent timestamps. Have to have more intelligence
 * in the timestamp constructor's so the right one is chosen.
 *
 * Revision 1.10  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.9  2013/03/28 08:22:36  chas
 * Found an error in flags. Some wrong bit shifting for data flags
 * Progressing along with data class. And making printf's a bit more
 * portable with PRIuXX's for _session and _offset types
 *
 * Revision 1.8  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.7  2013/03/24 22:40:32  chas
 * Added flag changes to beacon. Will need to add more for other flags
 * and add the appropriate functions to other frame types as well.
 * I'll set up this in frame as well as virtual function calls.
 * This will ensure that only the appropriate flags are set for the approriate frame type.
 *
 * Revision 1.6  2013/03/24 22:00:26  chas
 * Added data.h and split out beacon.h from frame.h
 *
 * Revision 1.5  2013/03/23 12:01:00  chas
 * Finally sorted out polymorhic "frame" class and beacon all done
 * so now I can move onto similar for other frame types
 *
 * Revision 1.4  2013/03/15 12:39:01  chas
 * Showing Frankie some code
 *
 * Revision 1.3  2013/03/15 12:17:34  chas
 * Moving along with the polymorphism. Beacon nearly done
 *
 * Revision 1.2  2013/03/15 04:47:06  chas
 * Moving frames to polymophism so we have a single base class of frame and
 * then polymorph frame types of beacon and data. As we move forward
 * I will add the other saratoga frame types. Lots more work to do here yet
 * as I havn't got the base class sorted yet.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 * Revision 1.5  2011-02-22 10:19:14  chas
 * Removed nast #defines and replaced with static const for all of the
 * flag bit handling classes
 *
 * Revision 1.4  2011-01-18 10:15:43  chas
 * Refining logging and error printing classes
 *
 * Revision 1.3  2011-01-05 05:28:32  chas
 * Changes to make flags classes available to all modules
 * and starting to get beacon class implemented.
 * The flags.h defines all of the flag classes and flag.c
 * contains the functions to manipulate them. The set functions
 * set a flag based upon its enumarated toye or if you pass
 * it a uint32_t header then it sets the bits in that flag.
 * Get works similarly.
 *
 * Revision 1.2  2011-01-03 23:05:19  chas
 * Changes License to official 3 Clause BSD License with Charles Smith as Author and
 * Vallona Networks as Organisation.
 * This will be used as the official License in all codeset and binaries of this
 * implementation.
 *
 * Revision 1.1.1.1  2010-12-30 12:44:56  chas
 * Open source development of Saratoga in C++ under a 2 clause BSD License.
 *
 * Date of commencement December 30 2010 - During the XMAS Holiday break
 *
 * flags.cpp uses enumerated types & classes to handle all flags used in 
 * the saratoga 32 bit header and the 16 bit directory entry headers.
 *
 */


