/*
 * $Id: checksum.cpp,v 1.3 2015/01/10 08:25:45 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <unistd.h>
#include <stdio.h>
#include <inttypes.h>
#include <string>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <cctype>
#include "checksum.h"

extern "C" 
{

extern unsigned long Crc32_ComputeBuf(unsigned long, char *, size_t);
extern void MD5Init(MD5_CTX *);
extern void MD5Update(MD5_CTX *, unsigned char *, ssize_t);
extern void MD5Final(unsigned char *, MD5_CTX *);
extern void SHA1Reset(SHA1Context *);
extern int SHA1Result(SHA1Context *);
extern void SHA1Input(SHA1Context *, unsigned char *, unsigned);
}

namespace checksums
{

// Remove white space, delimeter cahracters and convert to upper case
// e.g. ab:cd:ef -> ABCDEF
string
strip(string str)
{
   str.erase(std::remove(str.begin(), str.end(), ' '), str.end());
   str.erase(std::remove(str.begin(), str.end(), '\t'), str.end());
   str.erase(std::remove(str.begin(), str.end(), ':'), str.end());
   str.erase(std::remove(str.begin(), str.end(), '-'), str.end());
   str.erase(std::remove(str.begin(), str.end(), '.'), str.end());
   for( size_t i=0; i < str.length();i++)
   {
      str[i] = toupper(str[i]);
   }
   return str;
}

// ************************************************************************************
// No Checksum
// ************************************************************************************

string
none::print()
{
	char	tmp[128];
	string s = "No CSUM";
	if (_fname != "")
	{
		sprintf(tmp, " (%s)",
			_fname.c_str());
		s += tmp;
	}
	return(s);
}

// ************************************************************************************
// CRC32 Checksum
// ************************************************************************************

// Return the string value of a crc32 checksum
string
crc32::strval()
{
	char	tmp[128];

	string s("");
	sprintf(tmp, "%08" PRIX32"",
		(uint32_t) _crc32);
	s += tmp;
	cout << "crc32::strval() " << s << endl;
	return(s);
}

string
crc32::print()
{
	char	tmp[128];

	string s = "CRC32";
	if (_fname != "")
	{
		sprintf(tmp, "(%s) ",
			_fname.c_str());
		s += tmp;
	}
	sprintf(tmp, "%08" PRIX32"",
		(uint32_t) _crc32);
	s += tmp;
	cout << "crc32::print() " << s << endl;
	// s += this->strval();
	return(s);
}

crc32::crc32(string fname)
{
	const size_t	buflen = 1024;
	char	buf[buflen];
	// char	*buf = new char [buflen];
	unsigned long	csum = 0;
	sarwin::screen	msg;

	// checksum variables
	_fname = fname;
	_csumtype = CRC32;

	ifstream	fs(fname.c_str(), ios::binary);
	if (!fs.is_open())
	{
		msg.error("crc32: Can't open file: %s", fname.c_str());
		_crc32 = 0;
		_fname = "";
		return;
	}
	/* accumulate crc32 from file */
	while(fs.read(buf, buflen))
	{
		if (fs.fail())
		{
			msg.error("Error reading crc32 checksum file:%s", 
				fname.c_str());
			_crc32 = 0;
			return;
		}
		csum = Crc32_ComputeBuf(csum, buf, buflen);
	}
	// Handle the last buffer < buflen
	if (fs.gcount() > 0)
		csum = Crc32_ComputeBuf(csum, buf, fs.gcount());
	fs.close();
	_crc32 = csum;
	cout << "crc32::crc32(" << fname << ") " << _crc32 << endl;
	return;
}

crc32::crc32(uint32_t i)
{
	_fname = "";
	_csumtype = CRC32;
	_crc32 = i;
	cout << "crc32::crc32(" << i << ") " << _crc32 << endl;
}

// ************************************************************************************
// MD5 Checksum
// ************************************************************************************

uint32_t *
md5::value()
{
		static uint32_t	v[4];

		for (size_t i = 0; i < _md5_size; i++)
		{
			v[i] = (uint32_t) (_md5[i * 4 + 0] << 24);
			v[i] |= (uint32_t) (_md5[i * 4 + 1] << 16);
			v[i] |= (uint32_t) (_md5[i * 4 + 2] << 8);
			v[i] |= (uint32_t) (_md5[i * 4 + 3]);
		}
		return v;
}

// Return the string value of a md5 checksum
string
md5::strval()
{
	char	tmp[128];

	string s("");
	for (size_t i = 0; i < _md5_size * 4; i++)
	{
		sprintf(tmp, "%02" PRIX8 "", (uint8_t) _md5[i]);
		s += tmp;
		if (i == 3 || i == 7 || i == 11)
			s += " ";
	}
	return(s);
}

string
md5::print()
{
	char	tmp[128]; 

	string s("MD5 ");
	if (_fname != "")
	{
		sprintf(tmp, "(%s) ",
			_fname.c_str());
		s += tmp;
	}
	for (size_t i = 0; i < _md5_size * 4; i++)
	{
		sprintf(tmp, "%02" PRIX8 "", (uint8_t) _md5[i]);
		s += tmp;
		if (i == 3 || i == 7 || i == 11)
			s += " ";
	}
//	s += this->strval();
	return(s);
}

md5::md5(uint32_t i[])
{
	_fname = "";
	_csumtype = MD5;
	for (size_t x = 0; x < _md5_size; x++)
	{
		_md5[x * 4 + 0] = (i[x] >> 24);
		_md5[x * 4 + 1] = (i[x] >> 16) & 0xFF;
		_md5[x * 4 + 2] = (i[x] >> 8) & 0xFF;
		_md5[x * 4 + 3] = i[x] & 0xFF;
	}
}

md5::md5(string fname)
{
	const size_t	buflen = 1024;
	char		buf[buflen];
	// char		*buf = new char [buflen];
	sarwin::screen	msg;
	MD5_CTX		context;

	// checksum variables
	_fname = fname;
	_csumtype = MD5;

	ifstream	fs(fname.c_str(), ios::binary);
	if (!fs.is_open())
	{
		msg.error("MD5: Can't open file: %s", fname.c_str());
		for (size_t i = 0; i < _md5_size * 4; i++)
			_md5[i] = 0;
		return;
	}
	MD5Init(&context);
	/** accumulate MD5 from file **/
	while(fs.read(buf, buflen))
	{
		if (fs.fail())
		{
			msg.error("Error reading MD5 checksum file:%s", 
				fname.c_str());
			for (size_t i = 0; i < _md5_size * 4; i++)
				_md5[i] = 0;
			fs.close();
			return;
		}
		MD5Update(&context, (unsigned char *) &buf[0], buflen);
	}
	// Handle the last buffer < buflen
	if (fs.gcount() > 0)
		MD5Update(&context, (unsigned char *) &buf[0], fs.gcount());
	MD5Final(_md5,  &context);
	fs.close();
	return;
}

// ************************************************************************************
// SHA1 Checksum
// ************************************************************************************

// Return the string value of a sha1 checksum
string
sha1::strval()
{
	char	tmp[128];

	string s("");
	for (size_t i = 0; i < _sha1_size; i++)
	{
		sprintf(tmp, "%08" PRIX32 "", (uint32_t) _sha1[i]);
		s += tmp;
		if (i < _sha1_size - 1)
			s += " ";
	}
	return(s);
}

string
sha1::print()
{
	char	tmp[128]; 

	string s("SHA1 ");
	if (_fname != "")
	{
		sprintf(tmp, "(%s) ",
			_fname.c_str());
		s += tmp;
	}
	for (size_t i = 0; i < _sha1_size; i++)
	{
		sprintf(tmp, "%08" PRIX32 "", (uint32_t) _sha1[i]);
		s += tmp;
		if (i < _sha1_size - 1)
			s += " ";
	}
	// s += this->strval();
	return(s);
}

sha1::sha1(uint32_t i[])
{
	_csumtype = SHA1;
	_fname = "";
	for (size_t c = 0; c < _sha1_size; c++)
		_sha1[c] = i[c];
}

sha1::sha1(string fname)
{
	sarwin::screen	msg;
	SHA1Context	sha;
	const size_t	buflen = 1024;
	char		buf[buflen];

	// checksum vlariables
	_fname = fname;
	_csumtype = SHA1;

	ifstream	fs(fname.c_str(), ios::binary);
	if (!fs.is_open())
	{
		msg.error("SHA1: Can't open file: %s", fname.c_str());
		return;
	}
	SHA1Reset(&sha);
	/** accumulate SHA1 from file **/
	while(fs.read(buf, buflen))
	{
		if (fs.fail())
		{
			msg.error("Error reading SHA1 checksum file:%s", 
				fname.c_str());
			return;
		}
		for (unsigned int i = 0; i < buflen; i++)
			SHA1Input(&sha, (unsigned char *) &buf[i], 1);
	}
	// Handle the last buffer < buflen
	if (fs.gcount() > 0)
	{
		for (unsigned int i = 0; i < fs.gcount(); i++)
			SHA1Input(&sha, (unsigned char *) &buf[i], 1);
	}
	if (!SHA1Result(&sha))
	{
		msg.error("Could not compute SHA1 message digest for %s\n", 
			fname.c_str());
		return;
	}
	for (unsigned int i = 0; i < _sha1_size; i++)
		_sha1[i] = (uint32_t) sha.Message_Digest[i];
	fs.close();
	return;
}

}; // Namespace checksums

/*
 * $Log: checksum.cpp,v $
 * Revision 1.3  2015/01/10 08:25:45  charlesetsmith
 * Changed to support curses for debug and error output.
 * Also flags for 64 bit support
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.5  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.4  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.3  2013/09/27 06:08:25  chas
 * Problem with CRC32 pointers fixed
 *
 * Revision 1.2  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 * Revision 1.1.1.1  2013/03/09 07:28:18  chas
 * Checksum Library and standalone programs
 * crc32, md5 and sha1
 *
 */

