/*
 * $Id: checksum.h,v 1.4 2015/01/10 09:00:51 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _CHECKSUM_H
#define _CHECKSUM_H

#include <unistd.h>
#include <stdio.h>
#include <inttypes.h>
#include <string>
#include <cctype>
#include "../screen.h"
#include "md5.h"
#include "sha1.h"

#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS	// So we can d PRI_u64 in printf
#endif

using namespace std;
using namespace saratoga;

namespace checksums
{

typedef unsigned char uchar_t;

// Remove white space and punctuation from checksum strings
extern string	strip(string);

// Superclass for checksum so we can just have one of these pointing
// to the actual type of checksum. Makes adding new one easy later on
enum f_csumtype {
	NONE,
	CRC32,
	MD5,
	SHA1 
};

class checksum
{
protected:
	string	_fname;
	enum f_csumtype	_csumtype;
public:
	virtual ~checksum() { };
	
	string	fname() { return _fname; };

	string	fname(string s) { _fname = s; return _fname; };

	enum f_csumtype csumtye() { return _csumtype; }; 

	// Array of 32 bit integers
	virtual uint32_t*	value() { return this->value(); };

	// Number of octets in value
	virtual size_t	length() { return this->length(); };

	virtual string strval() { return this->strval(); };

	virtual string print() { return this->print(); };
};

// No checksum to calculate
class none : public checksum
{
protected:
	const static size_t	_none_size = 0; // # uint32 in none
public:
	none(string fname) { _fname = fname; _csumtype = NONE; };
	none() { _csumtype = NONE; };

	~none() { _fname = ""; _csumtype = NONE; };
	string print();
	size_t	length() { return _none_size; };
	uint32_t*	value() { return NULL; };
};

// Handle CRC32's
class crc32 : public checksum
{
protected:
	const static size_t 	_crc32_size = 1; // # uint32 in CRC32
private:
	uint32_t		_crc32;
	string			strval();
public:

	crc32(string);
	crc32(uint32_t);
	uint32_t*	value() { return &_crc32; };

	// Equals Compare with another crc32
	bool	operator==(crc32 c1) {
		crc32	tmp(*this);
		return (tmp._crc32 == c1._crc32);
	}

	// Equals Compare with a string hlding a crc32 value
	bool	operator==(string s) {
		crc32	tmp(*this);
		string	s1 = tmp.strval();
		string s2 = s;
		s1 = strip(s1);
		s2 = strip(s2);
		return (s1 == s2);
	}

	// Not equals Compare with another crc32
	bool	operator!=(crc32 c1) {
		crc32	tmp(*this);
		return (tmp == c1) ? 0 : 1;
	}

	// Not equals Compare with a string holding a crc32 value
	bool	operator!=(string s) {
		crc32	tmp(*this);
		string	s1 = tmp.strval();
		string s2 = s;
		s1 = strip(s1);
		s2 = strip(s2);
		return (s1 != s2);
	}

	~crc32() { 
		_crc32 = 0; 
		_fname = ""; 
		_csumtype = NONE;
	}

	// Length of checksum in uint32_t
	size_t	length() { return _crc32_size; };

	string print();
};

// Handle MD5's
class md5 : public checksum
{
protected:
	const static size_t	_md5_size = 4; // # uint_32 in MD5
private:
	uchar_t			_md5[_md5_size * 4];
	string			strval();
public:
	md5(string);
	// md5(uchar_t []);
	md5(uint32_t []);

	// Compare ==
	bool	operator==(md5 m1) {
		md5	tmp(*this);
		for (size_t i = 0; i < _md5_size; i++)
			if (tmp._md5[i] != m1._md5[i])
				return 0;
		return 1;
	}

	bool	operator==(string s) {
		md5	tmp(*this);
		string	s1 = tmp.strval();
		string s2 = s;
		s1 = strip(s1);
		s2 = strip(s2);
		return (s1 == s2);
	}

	// Compare !=
	bool	operator!=(md5 m1) {
		md5	tmp(*this);
		return (tmp == m1) ? 0 : 1;
	}

	bool	operator!=(string s) {
		md5	tmp(*this);
		string	s1 = tmp.strval();
		string s2 = s;
		s1 = strip(s1);
		s2 = strip(s2);
		return (s1 != s2);
	}

	~md5() { 
		_fname = "";
		for (size_t i = 0; i < _md5_size * 4; i++)
			_md5[i] = (uchar_t) 0;
		_csumtype = NONE;
	}

	// Length in uint32_t
	size_t	length() { return _md5_size; };

	uint32_t*	value();

	string print();
};

// Handle SHA1's
class sha1 : public checksum
{
protected:
	const static size_t	_sha1_size = 5; // # uint32 in SHA1
private:
	uint32_t		_sha1[_sha1_size];
	string			strval();
public:
	sha1(string);
	sha1(uint32_t []);

	// Compare == with another sha1 value
	bool	operator==(sha1 s1) {
		sha1	tmp(*this);
		for (size_t i = 0; i < _sha1_size; i++)
			if (tmp._sha1[i] != s1._sha1[i])
				return 0;
		return 1;
	}

	bool	operator==(string s) {
		sha1	tmp(*this);
		string	s1 = tmp.strval();
		string s2 = s;
		s1 = strip(s1);
		s2 = strip(s2);
		return (s1 == s2);
	}

	// Compare != with another sha1 value
	bool	operator!=(sha1 s1) {
		sha1	tmp(*this);
		return (tmp == s1) ? 0 : 1;
	}

	bool	operator!=(string s) {
		sha1	tmp(*this);
		string	s1 = tmp.strval();
		string s2 = s;
		s1 = strip(s1);
		s2 = strip(s2);
		return (s1 != s2);
	}

	~sha1() { 
		for (size_t i = 0; i < _sha1_size; i++) 
			_sha1[i] = 0; 
		_fname = "";
		_csumtype = NONE;
	}

	// length in uint32_t
	size_t	length() { return _sha1_size; };

	uint32_t*	value() { return &_sha1[0]; };

	string print();
};

}; // Namespace checksums

#endif // _CHECKSUM_H

/*
 * $Log: checksum.h,v $
 * Revision 1.4  2015/01/10 09:00:51  charlesetsmith
 * Added support for sprinf wide format integers
 *
 * Revision 1.3  2015/01/10 08:25:45  charlesetsmith
 * Changed to support curses for debug and error output.
 * Also flags for 64 bit support
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.6  2013/10/04 02:04:19  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.5  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.4  2013/09/27 06:08:49  chas
 * Problem with CRC32 pointers fixed
 *
 * Revision 1.3  2013/06/05 10:09:47  chas
 * Progress on metadata. Some additional work on checksums added constructor that takes in uint32_t's for each type to store them. Now I am assuming in saratoga that the way we transmit checksums is in a series of ntohl (uint32_t) values and that when we receive them we do a htonl to get the values back. Is 1 uint for crc32, 4 for md5 and 5 for sha1. I'll talk to LLoyd and see what we need to add into the internet draft.
 *
 * Revision 1.2  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 * Revision 1.1.1.1  2013/03/09 07:28:18  chas
 * Checksum Library and standalone programs
 * crc32, md5 and sha1
 *
 */

