/*
 * $Id: cli.cpp,v 1.3 2015/01/10 09:16:24 charlesetsmith Exp $
 */

// Command Line Interpretor for saratoga

// #include <iostream>
#include <cstring>
#include <string>
#include <vector>
#include <sys/types.h>
#include "screen.h"
#include "ip.h"
#include "cli.h"
#include "sarflags.h"
#include "timestamp.h"
#include "fileio.h"
#include "beacon.h"
#include "globals.h"

using namespace std;

// A command has a name, a usage a help and a function to call

namespace saratoga
{

// The command line interface flag classes
// these flags and variables are used to set the protocol flags
// default values and running state
// We have a class for each major cli keyword
// The handler is cmd::cmd_keyword
// The cli flags are cli_keyword

// Make sure we have an unsigned int string for stoi conversions
bool
isuint(string s)
{
	for (size_t i = 0; i < s.length(); i++)
		if (s[i] < '0' || s[i] > '9')
			return(false);
	return(true);
}

// ********************************************************************************
// * Command line handlers
// ********************************************************************************

bool
cmd::cmd_help()
{
	sarwin::screen	msg;
	cmds c;

	// We only expect a single argument
	if (_args.size() != 1)
	{
		msg.std("ERROR: help - Invalid number of arguments");
		return(false);
	}
	// Print out the command list
	msg.std(c.print());
	return(true);
}

bool
cmd::cmd_exit()
{
	sarwin::screen	msg;

	std::string::size_type sz; // needed for stoi

	if (_args.size() > 2)
	{
		msg.std("ERROR: exit - Invalid number of arguments");
		return(false);
	}

	if (_args.size() == 1)
	{
		msg.std("Goodbye");
		c_exit.flag(0);
		return(true);
	}
	if (_args.size() == 2)
	{
		int x = std::stoi(_args[1], &sz);
		if (x == 0)
		{
			// Exit gracefully
			c_exit.flag(0);
			return(true);
		}
		if (x == 1)
		{
			// Exit with extreme prejudice
			c_exit.flag(1);
			return(true);
		}
		msg.std("ERROR: exit - Invalid argument");
		c_exit.flag(-1);
		return(false);
	}
	// We never get here
	return(false);
}

// Set cli flags for beacons
// return whether to run the beacons or not
bool
cmd::cmd_beacon()
{
	sarwin::screen	msg;
	cmds c;
	string s;

	std::string::size_type sz; // needed for stoi
	string outstr;
	// Show current beacon flag settings
	if (_args.size() == 1)
	{
		// Send a single beacon
		if (!c_beacon.canrun())
		{
			msg.std("No destination for beacons set");
			return(true);
		}
		s = "Sending " + c_beacon.print() + " beacons";
		msg.std(s);
		return(true);
	}
	// SHow usage
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("beacon"));
		return(true);
	}
	// Turn all flags for beacons off
	if (_args.size() == 2 && _args[1] == "off")
	{
		// Just clear beacon flags
		c_beacon.v4flag(false);
		c_beacon.v6flag(false);
		c_beacon.secs(0);
		c_beacon.zap();
		msg.std("Beacons off");
		return(true);
	}

	size_t startpos = 1;
	bool flagset = true;
	if (_args[1] == "off")
	{
		flagset = false;
		startpos = 2;
	}
	for (size_t i = startpos; i < _args.size(); i++)
	{
		// We send v4 broadcast beacon
		if (_args[i] == "v4") 
		{
			c_beacon.v4flag(flagset);
			continue;
		}
		// We send v6 broadcast beacon
		if (_args[i] == "v6")
		{
			c_beacon.v6flag(flagset);
			continue;
		}
		// Send the beacon every x secs
		if (isuint(_args[i]))
		{
			if (flagset == false)
				c_beacon.secs(0);
			else
				c_beacon.secs(std::stoi(_args[i], &sz));
			continue;
		}
		// We send unicast beacon to ip address's
		sarnet::ip isip(_args[i]);
		if (isip.isv4() || isip.isv6())
		{
			if (flagset == false)
				c_beacon.iprm(_args[i]);
			else
				c_beacon.ipadd(_args[i]);
			continue;
		}
	}
	s = "Beacon Flags: " + c_beacon.print();
	msg.std(s);
	return(true);
}

bool
cmd::cmd_bundles()
{
	sarwin::screen	msg;
	cmds c;

	// We only expect a single argument
	if (_args.size() == 1)
	{
		msg.std(c_bundles.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "on")
	{
		msg.std("Bundles not supported yet");
		c_bundles.flag(false);
		msg.std(c_bundles.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_bundles.flag(false);
		msg.std(c_bundles.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("bundles"));
		return(true);
	}
	msg.std(c.usage("bundles"));
	return(false);
}

bool
cmd::cmd_checksum()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 1)
	{
		msg.std(c_checksum.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("checksum"));
		return(true);
	}
	if (_args.size() == 2)
	{
		if  (_args[1] == "off" || _args[1] == "none")
			c_checksum.flag(F_CSUM_NONE);
		else if (_args[1] == "crc32")
			c_checksum.flag(F_CSUM_CRC32);
		else if (_args[1] == "md5")
			c_checksum.flag(F_CSUM_MD5);
		else if (_args[1] == "sha1")
			c_checksum.flag(F_CSUM_SHA1);
		else
		{
			msg.std("Invalid Checksum Type");
			return(false);
		}
	}
	msg.std(c_checksum.print());
	return(true);
}

bool
cmd::cmd_debug()
{
	sarwin::screen	msg;
	string	s;
	cmds c;

	// We only expect a single argument
	if (_args.size() == 1)
	{
		msg.std(c_debug.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("debug"));
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_debug.level(0);
		msg.std(c_debug.print());
		return(true);
	}
	if (_args.size() == 2)
	{
		if (_args[1] == "0" || 
			_args[1] == "1" || 
			_args[1] == "2" ||
			_args[1] == "3" ||
			_args[1] == "4" ||
			_args[1] == "5" ||
			_args[1] == "6" ||
			_args[1] == "7" ||
			_args[1] == "8" ||
			_args[1] == "9")
			c_debug.level(_args[1][0] - '0');
		else
		{
			msg.error("Invalid Debug Level must be 0-9");
			msg.std(c.usage("debug"));
			return(true);
		}
	}
	msg.std(c_debug.print());
	return(true);
}

bool
cmd::cmd_descriptor()
{
	sarwin::screen	msg;
	cmds c;

	extern enum f_descriptor maxdescriptor(); // Maximim descriptor for largest file size
	if (_args.size() == 1)
	{
		msg.std(c_descriptor.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("descriptor"));
		return(true);
	}
	if (_args.size() == 2)
	{
		if  (_args[1] == "off")
			c_descriptor.flag(maxdescriptor()); // Reset to largest file size
		else if (_args[1] == "16")
			c_descriptor.flag(F_DESCRIPTOR_16);
		else if (_args[1] == "32")
			c_descriptor.flag(F_DESCRIPTOR_32);
		else if (_args[1] == "64")
			c_descriptor.flag(F_DESCRIPTOR_64);
		else if (_args[1] == "128")
		{
			msg.std("128bit  Descriptors not supported");
			c_descriptor.flag(F_DESCRIPTOR_64);
		}
		else
		{
			msg.std("Invalid Descriptor Type");
			return(false);
		}
	}
	msg.std(c_descriptor.print());
	return(true);
}

bool
cmd::cmd_eid()
{
	sarwin::screen	msg;
	string	s;
	cmds c;

	// We only expect a single argument
	if (_args.size() == 1)
	{
		s = "EID: " + c_eid.print();
		msg.std(s);
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("eid"));
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_eid.eid("");
		return(true);
	}
	if (_args.size() == 2)
	{
		c_eid.eid(_args[1]);
	}
	s = "EID: " + c_eid.print();
	msg.std(s);
	return(true);
}

bool
cmd::cmd_freespace()
{
	sarwin::screen	msg;
	cmds c;

	// We only expect a single argument
	if (_args.size() == 1)
	{
		msg.std(c_freespace.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("freespace"));
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "on")
	{
		c_freespace.flag(F_FREESPACE_YES);
		msg.std(c_freespace.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_freespace.flag(F_FREESPACE_NO);
		msg.std(c_freespace.print());
		return(true);
	}
	msg.std(c.usage("freespace"));
	return(false);
}

bool
cmd::cmd_get()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("get"));
		return(true);
	}
	if (_args.size() != 3)
	{
		msg.std(c.usage("get"));
		return(true);
	}
	sarnet::ip isip(_args[1]);
	if (isip.isv4() || isip.isv6())
	{
		c_get.peer(isip);
		c_get.fname(_args[2]);
		return(true);
	}
	c_get.zap();
	c_get.fname("");
	msg.std(c.usage("get"));
	return(true);
}

bool
cmd::cmd_ls()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 1 || (_args.size() == 2 && _args[1] == "?"))
	{
		msg.std(c.usage("ls"));
		return(true);
	}
	if (_args.size() == 2)
	{
		c_ls.dname(".");
	}
	if (_args.size() == 3)
	{
		c_ls.dname(_args[2]);
	}
	sarnet::ip isip(_args[1]);
	if (isip.isv4() || isip.isv6())
	{
		c_ls.peer(isip);
		return(true);
	}
	c_ls.zap();
	c_ls.dname("");
	msg.std(c.usage("ls"));
	return(true);
}

bool
cmd::cmd_peers()
{
	sarwin::screen	msg;
	string	s;
	cmds c;

	std::string::size_type sz; // needed for stoi
	string outstr;
	// Show current peer flag settings
	if (_args.size() == 1)
	{
		string ps = c_peers.print();
		if (ps == "")
			msg.std("No Peers");
		else
		{
			s = "Current Peers: " + ps;
			msg.std(s);
		}
		return(true);
	}
	// Show usage
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("peers"));
		return(true);
	}
	// Remove all peers
	if (_args.size() == 2 && _args[1] == "off")
	{
		// Just clear all the current peers
		c_peers.zap();
		return(true);
	}

	size_t startpos = 1;
	bool flagset = true;
	if (_args[1] == "off")
	{
		flagset = false;
		startpos = 2;
	}
	for (size_t i = startpos; i < _args.size(); i++)
	{
		// We clear peers we have listed
		sarnet::ip isip(_args[i]);
		if (isip.isv4() || isip.isv6())
		{
			if (flagset == false)
			{
				s = "Removing Peer: <" + _args[i] + ">";
				msg.std(s);
				c_peers.remove(_args[i]);
			}
			else
				c_peers.add(_args[i]);
			continue;
		}
		s = "peers - Invalid IP: " + isip.print();
		msg.std(s);
	}
	s = "Current Peers: " + c_peers.print();
	msg.std(s);
	return(true);
}

bool
cmd::cmd_put()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("put"));
		return(true);
	}
	if (_args.size() != 3)
	{
		msg.std(c.usage("put"));
		return(true);
	}
	sarnet::ip isip(_args[1]);
	if (isip.isv4() || isip.isv6())
	{
		c_put.peer(isip);
		c_put.fname(_args[2]);
		return(true);
	}
	c_put.zap();
	c_put.fname("");
	msg.std(c.usage("put"));
	return(true);
}

bool
cmd::cmd_rm()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("rm"));
		return(true);
	}
	if (_args.size() != 3)
	{
		msg.std(c.usage("rm"));
		return(true);
	}
	sarnet::ip isip(_args[1]);
	if (isip.isv4() || isip.isv6())
	{
		c_rm.peer(isip);
		c_rm.fname(_args[2]);
		return(true);
	}
	msg.std(c.usage("rm"));
	return(true);
}

bool
cmd::cmd_rmdir()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("rmdir"));
		return(true);
	}
	if (_args.size() != 3)
	{
		msg.std(c.usage("rmdir"));
		return(true);
	}
	sarnet::ip isip(_args[1]);
	if (isip.isv4() || isip.isv6())
	{
		c_rmdir.peer(isip);
		c_rmdir.dname(_args[2]);
		return(true);
	}
	msg.std(c.usage("rmdir"));
	return(true);
}

bool
cmd::cmd_rx()
{
	sarwin::screen	msg;
	cmds c;

	// We only expect a single argument
	if (_args.size() == 1)
	{
		msg.std(c_rx.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("rx"));
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "on")
	{
		c_rx.flag(F_RXWILLING_YES);
		msg.std(c_rx.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_rx.flag(F_RXWILLING_NO);
		msg.std(c_rx.print());
		return(true);
	}
	msg.std(c.usage("rx"));
	return(false);
}

bool
cmd::cmd_streams()
{
	sarwin::screen	msg;
	string	s;
	cmds c;

	// We only expect a single argument
	if (_args.size() == 1)
	{
		msg.std(c_streams.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("streams"));
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "on")
	{
		c_streams.flag(F_STREAMS_NO);
		s = "Streams not supported " + c_streams.print();
		msg.std(s);
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_streams.flag(F_STREAMS_NO);
		msg.std(c_streams.print());
		return(true);
	}
	msg.std(c.usage("streams"));
	return(false);
}

bool
cmd::cmd_timestamp()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 1)
	{
		msg.std(c_timestamp.print());
		return(true);
	}
	if (_args[1] == "?" || _args.size() != 2)
	{
		msg.std(c.usage("timestamp"));
		return(true);
	}
	if (_args[1] == "off") {
		c_timestamp.flag(F_TIMESTAMP_NO);
		// UDEF is not supported
		c_timestamp.ttype(T_TSTAMP_UDEF); }
	else if (_args[1] == "32") {
		c_timestamp.flag(F_TIMESTAMP_YES);
		c_timestamp.ttype(T_TSTAMP_32); }
	else if (_args[1] == "64") {
		c_timestamp.flag(F_TIMESTAMP_YES);
		c_timestamp.ttype(T_TSTAMP_64); }
	else if (_args[1] == "32_32") {
		c_timestamp.flag(F_TIMESTAMP_YES);
		c_timestamp.ttype(T_TSTAMP_32_32); }
	else if (_args[1] == "64_32") {
		c_timestamp.flag(F_TIMESTAMP_YES);
		c_timestamp.ttype(T_TSTAMP_64_32); }
	else if (_args[1] == "32_y2k") {
		c_timestamp.flag(F_TIMESTAMP_YES);
		c_timestamp.ttype(T_TSTAMP_32_Y2K); }
	else {
		msg.std(c.usage("timestamp"));
		return(true); }
	msg.std(c_timestamp.print());
	return(true);
}

bool
cmd::cmd_tx()
{
	sarwin::screen	msg;
	cmds c;

	if (_args.size() == 1)
	{
		msg.std(c_tx.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "?")
	{
		msg.std(c.usage("tx"));
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "on")
	{
		c_tx.flag(F_TXWILLING_YES);
		msg.std(c_tx.print());
		return(true);
	}
	if (_args.size() == 2 && _args[1] == "off")
	{
		c_tx.flag(F_TXWILLING_NO);
		msg.std(c_tx.print());
		return(true);
	}
	msg.std(c.usage("tx"));
	return(false);
}

// *******************************************************************************

string
cmds::usage(string c)
{
	for (int i = 0; i < _ncmds; i++)
		if (_clist[i] == c)
			return _clist[i].usage();
	return "";
}

string
cmds::cmdstr(string c)
{
	for (int i = 0; i < _ncmds; i++)
		if (_clist[i] == c)
			return _clist[i].name();
	return "";
}

void
cmd::setargs(vector<string> argv)
{
	for (size_t i = 0; i < argv.size(); i++)
		_args.push_back(argv[i]);
}

void
cmd::delargs()
{
	if (_args.size() > 0)		
		_args.erase(_args.begin(), _args.end());
}

bool
cmds::runcmd(vector<string> argv)
{
	for (int i = 0; i < _ncmds; i++)
		if (_clist[i] == argv[0])
		{
			_clist[i].delargs(); // delete old
			_clist[i].setargs(argv); // add new
			return(_clist[i].run()); // run it
		}
	return(false);
}

string
cmds::print()
{
	string s = "";
	for (int i = 0; i < _ncmds; i++)
	{
		s += _clist[i].name();
	//	s += ":\n\t";
		s += ":\t";
		s += _clist[i].usage();
		s += " - \t";
		s += _clist[i].help();
		s += '\n';
	}
	return s;
}

/*
 *********************************************************************************
 */

// The command line interface flag classes
// these flags and variables are used to set the protocol flags
// default values and running state
// We have a class for each major cli keyword
// The handler is cmd::cmd_keyword
// The cli flags are cli_keyword

// Given a wakup period see if the
// beacon timer has expired and if so return true
bool
cli_beacon::timerexpired()
{
	timestamp	now; // The current time
	timestamp	*then = this->timer();
	timestamp	diff = now - *then;

	// No timer so always flase
	if (_secs == 0)
		return(false);
	// If our timer has expired then reset it to now
	if (diff.secs() >= _secs)
	{
		this->settimer(now);
		return(true);
	}
	// Nope timer has not expired yet
	return(false);
}

void
cli_beacon::execute()
{
	sarwin::screen	msg;
	string	s;

	string 	eidstr;

	frame*	f;
	fsinfo *fs = new fsinfo(".");

	if (c_eid.eid() == "")
	{
		eidstr = v4out->straddr();
		eidstr += " ";
		eidstr += v6out->straddr();
	}
	else
		eidstr = c_eid.eid();
	if (c_freespace.flag() == F_FREESPACE_YES)
	{
		f = new beacon(
			c_descriptor.flag(),
			c_streams.flag(),
			c_tx.flag(),
			c_rx.flag(),
			fs,
			eidstr);
	}
	else
		f = new beacon(
			c_descriptor.flag(),
			c_streams.flag(),
			c_tx.flag(),
			c_rx.flag(),
			eidstr);
	msg.info("Sending Beacon");
	msg.info(f->print());
	if (this->v4flag())
	{
//		if (f->tx(v4mcastout) < 0)
//		 	msg.error("Cant queue a multicast v4 beacon");
		if (f->tx(v4out) < 0)
		 	msg.error("Cant queue a unicast v4 beacon");
	}
	if (this->v6flag())
	{
//		if (f->tx(v6mcastout) < 0)
//		 	msg.error("Cant queue a v6 beacon");
		if (f->tx(v6out) < 0)
		 	msg.error("Cant queue a unicast v6 beacon");
	}
	for (std::list<sarnet::ip>::iterator i = this->begin(); i != this->end(); i++)
	{
		sarnet::udp	*sock;
		s = "Sending beacon to " + i->print();
		msg.info(s);
		// WORK ON THIS match 
		if ((sock = sarpeers.match(*i)) != NULL)
			f->tx(sock);
	}
	delete fs;
	delete f;
}

string
cli_beacon::print()
{
	string outstr = "";
	if (this->v4flag())
		outstr += "v4 ";
	if (this->v6flag())
		outstr += "v6 ";
	for (std::list<sarnet::ip>::iterator i = begin(); i != end(); i++)
	{
		outstr += i->print();
		outstr += " ";
	}
	if (this->secs() > 0 && this->canrun())
	{
		char tmp[64];
		sprintf(tmp, "every %d secs", (int) this->secs());
		outstr += tmp;
	}
	if (outstr.length() > 0)
		return(outstr);
	else
		return("beacons off");
}

string
cli_bundles::print()
{
	if (this->flag() == true)
		return("bundles on");
	else
		return("bundles off");
}

string
cli_checksum::print()
{
	Fcsum	f(this->flag());

	return(f.print());
}

string
cli_debug::print()
{
	char	tmp[16];

	sprintf(tmp, "Debug Level %d", this->level());
	string s = tmp;
	return(s);
}

string
cli_descriptor::print()
{
	Fdescriptor	f(this->flag());

	return(f.print());
}

string
cli_eid::print()
{
	if (this->eid() == "")
		return("EID Not Set");
	return(this->eid());
}

string
cli_freespace::print()
{
	Ffreespace	f(this->flag());

	return(f.print());
}

string
cli_peers::print()
{
	string outstr = "";

	for (std::list<sarnet::ip *>::iterator i = begin(); i != end(); i++)
	{
		outstr += (*i)->print();
		outstr += " ";
	}
	return outstr;
}

string
cli_rx::print()
{
	Frxwilling	f(this->flag());

	return(f.print());
}

string
cli_streams::print()
{
	Fstreams	f(this->flag());

	return(f.print());
}

string
cli_tx::print()
{
	Ftxwilling	f(this->flag());

	return(f.print());
}

string
cli_timestamp::print()
{
	Ttimestamp		f(this->ttype());

	if (this->flag() == F_TIMESTAMP_NO)
		return("timestamps disabled");
	return(f.print());
}

}; // namespace saratoga

/*
 * $Log: cli.cpp,v $
 * Revision 1.3  2015/01/10 09:16:24  charlesetsmith
 * Problem with CVS repository - Pointed to the worng one.
 *
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.4  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.3  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.2  2013/04/21 12:45:18  chas
 * MOre work on cli decided on arrays of strings. This will change to arrays of structs
 * when I get the command function pointers added.
 *
 * Revision 1.1  2013/04/17 03:02:18  chas
 * Beginning the CLI user interface. Have a vector holding the
 * valid cli options.
 *
 */

