/*
 * $Id: data.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */


#include <iostream>
#include <cstring>
#include <string>

#include "saratoga.h"
#include "sarflags.h"
#include "timestamp.h"
#include "frame.h"
#include "data.h"

using namespace std;

namespace saratoga
{

// Create a data frame from scratch
data::data( enum f_descriptor des,
	enum f_transfer tfr,
	enum f_reqstatus stat,
	enum f_eod eodf,
	enum f_tstamp ts,
	session_t	session,
	offset_t 	offset,
	char 		*dbuf, 
	size_t 		dblen)
{
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif

	sarwin::screen	msg;

	Fversion	version(F_VERSION_1);
	Fframetype	frametype(F_FRAMETYPE_DATA);
	Fdescriptor	descriptor(des);
	Ftransfer	transfer(tfr);
	Freqstatus	reqstatus(stat);
	Feod		eod(eodf);
	Ftstamp		tstamp(ts);
	
	// Work out how big our frame has to be allocate it
	size_t fsize = sizeof(flag_t) + sizeof(session_t); // Flag + Session
	fsize += descriptor.length(); // Offset
	if (tstamp.get() == F_TIMESTAMP_YES) // Optional timestamp
	{
		_timestamp = new timestamp; // Create the timestamp
		fsize += _timestamp->length();
	}
	fsize += dblen;
	_payload = new char[fsize];
	_paylen = fsize;
	char	*dbufp = _payload;

	// Set and copy flags
	_badframe = false;
	_flags = 0;
	version.set(&_flags);
	frametype.set(&_flags);
	descriptor.set(&_flags);
	transfer.set(&_flags);
	tstamp.set(&_flags);
	reqstatus.set(&_flags);
	eod.set(&_flags);
	memcpy(dbufp, &_flags, sizeof(flag_t));
	dbufp += sizeof(flag_t);

	// Set and copy session
	_session = session;
	tmp_32 = htonl(session);
	memcpy(dbufp, &tmp_32, sizeof(session_t));
	dbufp += sizeof(session_t);

	// Set and copy offset
	_offset = offset;
	switch(descriptor.get())
	{
	case F_DESCRIPTOR_16:
		tmp_16 = htons(_offset);
		memcpy(dbufp, &tmp_16, sizeof(uint16_t));
		break;
	case F_DESCRIPTOR_32:
		tmp_32 = htonl(_offset);
		memcpy(dbufp, &tmp_32, sizeof(uint32_t));
		break;
	case F_DESCRIPTOR_64:
		tmp_64 = htonll(_offset);
		memcpy(dbufp, &tmp_64, sizeof(uint64_t));
		break;
	case F_DESCRIPTOR_128:
	#ifdef UINT128_T
			tmp_128 = htonlll(_offset);
			memcpy(dbufp, &tmp_128, sizeof(uint128_t));
	#else
			msg.error("data(payload, len): Descriptor 128 Bit Size Not Supported\n");
			break;
	#endif
	default:
			msg.error("data(payload, len): Invalid Descriptor Size");
			exit(1);
		break;
	}
	dbufp += descriptor.length();

	// Set and copy the timestamp
	// If we have a timestamp then lets make it by default ZULU time
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		char *tbuf = _timestamp->hton();
		memcpy(dbufp, tbuf, _timestamp->length());
		dbufp += _timestamp->length();
	}
	else
		_timestamp = NULL;

	// And finally the actual data buffer
	memcpy(dbufp, dbuf, dblen);
	_dbuf = dbufp; // No need to copy it it's in the frame just point to it
	_dbuflen = dblen;
}

/*
 * Given a buffer and length, assemble the data
 * this is what we do when re receive a saratoga frame
 * so we do the ntoh's
 */
data::data(char *payload,
	size_t paylen)
{
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif
	sarwin::screen	msg;

	_badframe = false;
	_paylen = paylen;
	_payload = new char[_paylen];
	memcpy(_payload, payload, paylen);

	// Get the frame info
	// flags and payload

	// Assemble the flags info
	memcpy(&tmp_32, payload, sizeof(flag_t));
	_flags = ntohl(tmp_32);
	payload += sizeof(flag_t);
	paylen -= sizeof(flag_t);
	Fversion	version(_flags);
	Fframetype	frametype(_flags);

	if (version.get() != F_VERSION_1)
	{
		msg.error("Data: Bad Saratoga Version");
		_badframe = true;
		return;
	}
	if (frametype.get() != F_FRAMETYPE_DATA)
	{
		msg.error("Data: Not a DATA frame");
		_badframe = true;
		return;
	}
	Fdescriptor descriptor(_flags);
	Ftstamp tstamp(_flags);
	
	// Assemble the data info

	// Session ID
	memcpy(&tmp_32, payload, sizeof(uint32_t));
	_session = (session_t) ntohl(tmp_32);
	payload += sizeof(session_t);
	paylen -= sizeof(session_t);

	// Offset into file/stream
	switch(descriptor.get())
	{
	case F_DESCRIPTOR_16:
		memcpy(&tmp_16, payload, sizeof(uint16_t));
		_offset = (offset_t) ntohs(tmp_16);
		break;
	case F_DESCRIPTOR_32:
		memcpy(&tmp_32, payload, sizeof(uint32_t));
		_offset = (offset_t) ntohl(tmp_32);
		break;
	case F_DESCRIPTOR_64:
		memcpy(&tmp_64, payload, sizeof(uint64_t));
		_offset = (offset_t) ntohll(tmp_64);
		break;
	case F_DESCRIPTOR_128:
#ifdef UINT128_T
		memcpy(&tmp_128, payload, sizeof(uint128_t));
		_offset = (offset_t) ntohlll(tmp_128);
#else
		msg.error("data(frame): Descriptor 128 Bit Size Not Supported\n");
#endif
		break;
	}
	payload += descriptor.length();
	paylen -= descriptor.length();

	// Optional timestamp on data frame
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		_timestamp = new timestamp(payload);
		payload += _timestamp->length();
		paylen -= _timestamp->length();
	}

	_dbuflen = paylen;
	// We dont need to copy dbuf we just need to point to it in the payload
	_dbuf = payload;
}

// Print out the current flag bits
string
data::printflags()
{
	string s;

	// cout << "FLAGS=" << f << endl;
	s = "\n    DATA FLAGS           1                   2                   3\n";
	s += "    |0 1 2 3 4 5 6 7|8 9 0 1 2 3 4 5|6 7 8 9 0 1 2 3|4 5 6 7 8 9 0 1|\n";
	s += "    |";
	for (int i = 0; i < 32; i++)
	{
		flag_t tmpf = _flags;
		tmpf <<= i;
		tmpf >>= 31;
		if (tmpf == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	return s;
}

/*
 * Print out the data flags buffer length and
 */
string
data::print()
{
	char	tmp[128];
	string	s;
	/*
	 * These are the valid flags types applicable
	 * to a data
	 */
	if (_badframe)
	{
		s = "Bad Frame: Not a DATA";
		return(s);
	}
	Fversion	version(_flags);
	Fframetype	frametype(_flags);
	Fdescriptor	descriptor(_flags);
	Ftransfer	transfer(_flags);
	Ftstamp		tstamp(_flags);
	Freqstatus	reqstatus(_flags);
	Feod		eod(_flags);

	s = frametype.print();
	s += this->printflags();
	s += "    ";
	s += version.print();
	s += "\n    ";
	s += descriptor.print();
	s += "\n    ";
	s += transfer.print();
	s += "\n    ";
	s += tstamp.print();
	s += "\n    ";
	s += reqstatus.print();
	s += "\n    ";
	s += eod.print();
	s += "\n    ";

	sprintf(tmp, 
		"Session: %" PRIu32 "", (uint32_t) _session);
	s += tmp;
	s += "\n    ";
	sprintf(tmp, 
		"Offset: %" PRIu64 "", (uint64_t) _offset);
	s += tmp;
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		s += "\n    ";
		timestamp *t = new timestamp;
		s += t->print();
		delete t;
	}
	s += "\n    ";
	sprintf(tmp, "Buffer Length: %lu", _dbuflen);
	s += tmp;
	return(s);
}

// Set various flags applicable to data frames
enum f_transfer
data::transfer(enum f_transfer t)
{
	Ftransfer	transfer(t);
	transfer.set(&_flags);
	return transfer.get();
}

enum f_reqstatus
data::reqstatus(enum f_reqstatus t)
{
	Freqstatus	reqstatus(t);
	reqstatus.set(&_flags);
	return reqstatus.get();
}

enum f_eod
data::eod(enum f_eod t)
{
	Feod	eod(t);
	eod.set(&_flags);
	return eod.get();
}

enum f_tstamp
data::tstamp(enum f_tstamp t)
{
	Ftstamp	tstamp(t);
	tstamp.set(&_flags);
	return tstamp.get();
}

// Get various flags applicable to data frames
enum f_transfer
data::transfer() {
	Ftransfer t(_flags);
	return t.get();
}

enum f_reqstatus
data::reqstatus() {
	Freqstatus s(_flags);
	return s.get();
}

enum f_eod
data::eod() {
	Feod e(_flags);
	return e.get();
}

enum f_tstamp
data::tstamp() {
	Ftstamp t(_flags);
	return t.get();
}

} // Namespace saratoga

/*
 * $Log: data.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.14  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.13  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.12  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.11  2013/06/05 07:20:09  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.10  2013/06/05 01:44:20  chas
 * Sorted out frame's
 *
 * Revision 1.9  2013/06/01 07:32:40  chas
 * Some issues with frame virtualisation. Perhaps this was a bad idea pulling the flags into the frame class... Ill think it over with a couple of scotches (single malt of course).
 *
 * Revision 1.8  2013/06/01 06:30:09  chas
 * Removed _flags from each individual frame type (BEACON, DATA, METADATA...) and ut the _flags in a friend frame class.
 *
 * Revision 1.7  2013/04/03 05:18:51  chas
 * Major changes yet again to flags. The set function now only needs
 * nnn.set(&_flag) rather than _flag = nnn.set(_flag)
 * much neater and shorter and less eror prone.
 * Still issues with dirent timestamps. Have to have more intelligence
 * in the timestamp constructor's so the right one is chosen.
 *
 * Revision 1.6  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.5  2013/03/28 08:56:35  chas
 * Moe bugs in the flags for data. Im shifting the worng number of bits.
 * Think this will be a generic problem to sort as we go through the flags
 * for each frame type.
 *
 * Revision 1.3  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.2  2013/03/15 04:48:23  chas
 * Moving frames to polymophism so we have a single base class of frame and
 * then polymorph frame types of beacon and data. As we move forward
 * I will add the other saratoga frame types. Lots more work to do here yet
 * as I havn't got the base class sorted yet.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 *
 */

