/*
 * $Id: dirent.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/statvfs.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits>

#include <iostream>
#include <cstring>
#include <string>

#include "saratoga.h"
#include "screen.h"
#include "sarflags.h"
#include "timestamp.h"
#include "dirent.h"

extern "C"
{
extern int stat(const char *path, struct stat *buf);
extern int statvfs(const char *path, struct statvfs *fidata);
}

namespace saratoga 
{

// Given a local filename get its details and create
// a dirent from it IT WILL BE PLAFORM SPECIFIC
// you Windows lot put your coding hats on :)
dirent::dirent(string fname)
{
	sarwin::screen	msg;
	struct stat	st;
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128
	uint128_t	tmp_128;
#endif

	_fname = fname;
	char cfname[1024]; // Max Filename len is 1023 + NULL
	strncpy(cfname, fname.c_str(), 1024);
	cfname[1023] = '\0';
	
	Dsod sod; // Start of Directory Marker
	Dprop prop(D_PROP_FILE); // We are a file
	Dforb forb(D_FORB_FILE); // We are not a bundle

	// Make sure I can read and stat the file
	if (access(cfname, R_OK) == -1 || stat(cfname, &st) != 0)
	{
		/*
		 * I can't stat it
		 * errno set to: EACCESS, EBADF, EFAULT, ELOOP
		 * ENAMETOOLONG, ENOENT, ENOMEM, ENOTDIR, EOVERFLOW
		 */
		msg.error("Can't access or stat file: %s", cfname);
		_mtime = new timestamp(T_TSTAMP_32_Y2K, 0);
		_ctime = new timestamp(T_TSTAMP_32_Y2K, 0);
		_filesize = 0;
		prop.set(D_PROP_SPECIALFILE);
		_valid = false; // Always as it does not exist!
	}
	else
	{
		// We have stat'd it so lets get the correct info from the stat
		_filesize = (offset_t) st.st_size;
		_mtime = new timestamp(T_TSTAMP_32_Y2K, st.st_mtime);
		_ctime = new timestamp(T_TSTAMP_32_Y2K, st.st_ctime);
	
		/*
		 * These are the valid flags types applicable
		 * to a dirent set them
		 */
		switch(st.st_mode & S_IFMT)
		{
		case S_IFREG:	// regular file
			prop.set(D_PROP_FILE);
			_valid = true;
			break;
		case S_IFDIR:	// directoy
			prop.set(D_PROP_DIR);
			_valid = true;
			break;
		case S_IFCHR:	// character device
		case S_IFBLK:	// block device
		case S_IFIFO:	// FIFO named pipe
		case S_IFLNK:	// symbolic link
		case S_IFSOCK:	// socket
			prop.set(D_PROP_SPECIALFILE);
			_valid = false; // For the moment!!!!!
			break;
		default:
			prop.set(D_PROP_SPECIALFILE);
			_valid = false;
		}
	}
	// The appropriate descriptor
	enum d_descriptor	desc;
	if (_filesize < (offset_t) numeric_limits<uint16_t>::max())
		desc = D_DESCRIPTOR_16;
	else if (_filesize < (offset_t) numeric_limits<uint32_t>::max())
		desc = D_DESCRIPTOR_32;
	else if (_filesize < (offset_t) numeric_limits<uint64_t>::max())
		desc = D_DESCRIPTOR_64;
	else
	#ifdef UINT128_T
		desc = D_DESCRIPTOR_128;
	#else
		desc = D_DESCRIPTOR_64;
	#endif
	Ddescriptor	descriptor(desc);

	/*
	 * Turn on/off the bits appropriate in the
	 * header flags field so our flags are set correctly
	 */

	_dflags = 0; // 0x8000;	// High order bit of directory flags always set
	sod.set(&_dflags);
	descriptor.set(&_dflags);
	prop.set(&_dflags);
	forb.set(&_dflags);

	// Copy the directory info to the payload
	_dpaylen = sizeof(dflag_t) +
		descriptor.length() +
		sizeof(dtime_t) + sizeof(dtime_t) +
		fname.length() + 1;

	// Allocate the payload and copy it all
	_dpayload = new char[_dpaylen];
	char *payload = _dpayload;
	// Flags
	memcpy(payload, &_dflags, sizeof(dflag_t));
	payload += sizeof(dflag_t);

	// Filesize
	switch(descriptor.get())
	{
	case D_DESCRIPTOR_16:
		tmp_16 = htons((uint16_t) _filesize);
		memcpy(payload, &tmp_16, sizeof(uint16_t));
		payload += sizeof(uint16_t);
		break;
	case D_DESCRIPTOR_32:
		tmp_32 = htonl((uint32_t) _filesize);
		memcpy(payload, &tmp_32, sizeof(uint32_t));
		payload += sizeof(uint32_t);
		break;
	case D_DESCRIPTOR_64:
		tmp_64 = htonll((uint64_t) _filesize);
		memcpy(payload, &tmp_64, sizeof(uint64_t));
		payload += sizeof(uint64_t);
		break;
	case D_DESCRIPTOR_128:
#ifdef UINT128_T
		tmp_128 = htonlll((uint128_t) _filesize);
		memcpy(payload, &tmp_128, sizeof(uint128_t));
		payload += sizeof(uint128_t);
#else
		msg.error("dirent: Descriptor 128 Bit Size Not Supported\n");
		break;
#endif
	default:
		msg.error("dirent: Undefined Descriptor\n");
		break;
	}

	// MTime
	dtime_t		dtmp;
	dtmp = htonl(_mtime->dirtime());
	memcpy(payload, &dtmp, sizeof(dtime_t));
	payload += sizeof(dtime_t);
	// CTime
	dtmp = _mtime->dirtime();
	dtmp = htonl(_ctime->dirtime());
	memcpy(payload, &dtmp, sizeof(dtime_t));
	payload += sizeof(dtime_t);

	// The File Name
	memcpy(payload, _fname.c_str(), _fname.length());
	payload += _fname.length();
	*payload = '\0'; // Make sure its always NULL terminated
	this->print();
}

// An input buffer convert to dirent
dirent::dirent(char	*payload,
		size_t	paylen)
{
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif
	sarwin::screen	msg;

	/* Copy the full dirent payload */
	_dpaylen = paylen;
	_dpayload = (char *) new char[_dpaylen];
	memcpy(_dpayload, payload, paylen);

	// Copy the data fields and convert them
	// as required to host byte order

	// flags	
	memcpy(&_dflags, payload, sizeof(dflag_t));
	payload += sizeof(dflag_t);
	paylen -= sizeof(dflag_t);

	Dsod		sod(_dflags);
	Ddescriptor	descriptor(_dflags);
	Dprop		prop(_dflags);

	if (sod.get() != D_SOD_YES)
	{
		msg.error("Invalid SOD Flag Not Set");
		exit(1);
	}	
	switch(descriptor.get())
	{
	case D_DESCRIPTOR_16:
		memcpy(&tmp_16, payload, sizeof(uint16_t));
		_filesize = (offset_t) ntohs(tmp_16);
		payload += sizeof(uint16_t);
		paylen -= sizeof(uint16_t);
		break;
	case D_DESCRIPTOR_32:
		memcpy(&tmp_32, payload, sizeof(uint32_t));
		_filesize = (offset_t) ntohl(tmp_32);
		payload += sizeof(uint32_t);
		paylen -= sizeof(uint32_t);
		break;
	case D_DESCRIPTOR_64:
		memcpy(&tmp_64, payload, sizeof(uint64_t));
		_filesize = (offset_t) ntohll(tmp_64);
		payload += sizeof(uint64_t);
		paylen -= sizeof(uint64_t);
		break;
	case D_DESCRIPTOR_128:
#ifdef UINT128_T
		memcpy(&tmp_128, payload, sizeof(uint128_t));
		_filesize = (offset_t) ntohlll(tmp_128);
		payload += sizeof(uint128_t);
		paylen -= sizeof(uint128_t);
#else
		msg.error("dirent: Descriptor 128 Bit Size Not Supported\n");
		payload += 16;
		paylen -= 16;
		break;
#endif
	default:
		msg.error("dirent: Undefined Descriptor\n");
		break;
	}
	// Mtime
	memcpy(&tmp_32, payload, sizeof(dtime_t));
	tmp_32 = ntohl(tmp_32);
	tmp_32 -= Y2KCONST; // We have to subtract it so that we don't double add it when we create new timestamp
	_mtime = new timestamp(T_TSTAMP_32_Y2K, tmp_32);
	payload += sizeof(dtime_t);
	paylen -= sizeof(dtime_t);
	
	// Ctime
	memcpy(&tmp_32, payload, sizeof(dtime_t));
	tmp_32 = ntohl(tmp_32);
	tmp_32 -= Y2KCONST; // We have to subtract is so we don't double add it when we create new timestamp
	_ctime = new timestamp(T_TSTAMP_32_Y2K, tmp_32);
	payload += sizeof(dtime_t);
	paylen -= sizeof(dtime_t);
	// Rest of payload is filename
	_fname.assign(payload, paylen);
	if (prop.get() == D_PROP_SPECIALFILE)
		_valid = false;
	else
		_valid = true;
}

size_t
Ddescriptor::length()
{
	sarwin::screen	msg;

	switch(Ddescriptor::get())
	{
	case D_DESCRIPTOR_16:
		return sizeof(uint16_t);
	case D_DESCRIPTOR_32:
		return sizeof(uint32_t);
	case D_DESCRIPTOR_64:
		return sizeof(uint64_t);
	case D_DESCRIPTOR_128:
#ifdef UINT128_T
		return sizeof(uint128_t);
#else
		msg.error("Descriptor size of 128 bit illegal");
#endif
	break;
	}
msg.error("Descriptor size illegal");
	return D_DESCRIPTOR_16;
}

/*
 * Print out the bits set in the dirent flags
 */
string
dirent::printdflags()
{
	string s("");
	
	s += "\n        DIRENTFLAGS          1\n";
	s += "        |0 1 2 3 4 5 6 7|8 9 0 1 2 3 4 5|\n";
	s += "        |";
	for (int i = 0; i <  16; i++)
	{
		dflag_t f = _dflags;
		f <<= i;
		f >>= 15;
		if (f == 1) 
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	return s;
}

/*
 * Print out the dirent flags and info
 */
string
dirent::print()
{
	char	tmp[256];
	Dsod sod(_dflags);
	Dprop prop(_dflags);
	Ddescriptor des(_dflags);
	Dforb forb(_dflags); // File or bundle

	enum d_prop dp = prop.get();

	string s("Directory Entry Information: ");
	s += _fname;
	s += this->printdflags();
	s += "        ";
	s += sod.print();
	s += "\n        ";
	if (_valid == true)
	{
		s += "Entry accessible";
	}
	else
	{
		s += "Entry not accessible";
	}
	s += "\n        ";
	s += prop.print();
	if (dp == D_PROP_FILE)
	{
		s += "\n        ";
		sprintf(tmp, "Filesize=%" PRIu64 "", (uint64_t) _filesize);
		s += tmp;
		s += "\n        ";

		s += des.print();
		s += "\n        ";

		s += forb.print();
	}
	if (dp == D_PROP_FILE || dp == D_PROP_DIR)
	{
		s += "\n        CTime: ";
		s += (_ctime->print());
		s += "\n        MTime: ";
		s += _mtime->print();
	}
	return(s);
}

/*
 * ******************************************************************************************************
 */

/*
 *  Allocator for file system space information. Get the info and then
 *  work out what descriptor to use to show the free space and put
 *  into the beacon frame.
 */
fsinfo::fsinfo(string fname)
{
	struct statvfs fsdata;
	char path[256];
	sarwin::screen	msg;

	// Copy fname from string to null terminated char*
	if (fname.length() < 256 )
	{
		size_t len = fname.copy(path,fname.length(),0);
		path[len]='\0';
	}
	else
	{
		msg.perror(errno, "Can't fsinfo: file name too long");
		_fname = fname;
		_blocksize = 0;
		_totblocks = 0;
		_freeblocks = 0;
		_free1kblocks = 0;
		_freespaced = F_FREESPACED_16;
		return;
	}


	// Get the file system info for the file name
	// and complain if we can't
	if ((statvfs(path, &fsdata)) < 0)
	{
		msg.perror(errno, "Can't fsinfo(%s): statvfs", path);
		_fname = fname;
		_blocksize = 0;
		_totblocks = 0;
		_freeblocks = 0;
		_freespaced = F_FREESPACED_16;
		return;
	}
	// Put it all into our class
	_fname = fname;
	_blocksize = (offset_t) fsdata.f_bsize;
	_totblocks = (offset_t) fsdata.f_blocks;
	_freeblocks = (offset_t) fsdata.f_bfree;
	_free1kblocks = (offset_t) ((double) _blocksize * ((double) _freeblocks / 1024.0));
	if (_free1kblocks < (offset_t) numeric_limits<uint16_t>::max())
		_freespaced = F_FREESPACED_16;
	else if (_free1kblocks < (offset_t) numeric_limits<uint32_t>::max())
		_freespaced = F_FREESPACED_32;
	else if (_free1kblocks < (offset_t) numeric_limits<uint64_t>::max())
		_freespaced = F_FREESPACED_64;
	else
	#ifdef UINT128_T
		_freespaced = F_FREESPACED_128;
	#else
		_freespaced = F_FREESPACED_64;
	#endif
	return;
}

string
fsinfo::print()
{
	char	tmp[128];

		string s("File System Information for: ");
		s += _fname;
		s += "\n    ";
		sprintf(tmp,
			"Blocksize: %ld", _blocksize);
		s += tmp;
		s += "\n    ";
		sprintf(tmp,
			"Total Blocks: %ld", _totblocks);
		s += tmp;
		s += "\n    ";
		sprintf(tmp, "Free Blocks: %ld", _freeblocks);
		s += tmp;
		s += "\n    ";
		sprintf(tmp, "Free Space 1kB Blocks: %ld\n", this->freespace());
		s += tmp;
		return(s);
}

}; // Namespace saratoga

/*
 * $Log: dirent.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.12  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.11  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.10  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.9  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.8  2013/09/27 06:13:40  chas
 * dflag_t was incorrect set to correct uint16_t now
 *
 * Revision 1.7  2013/06/08 07:31:50  chas
 * Direcory entry as Gordon Ramsey would say DONE!! So it can read directory entries and get the file type and info to be ready to go into a metadata or read from to get the info.
 *
 * Revision 1.6  2013/06/07 11:49:14  chas
 * Lots if work done on dirent. Still more work to do. It currently core dumps when you put in an invalid file name. Converting a payload back into a direct isn;t working yet either.
 *
 * Revision 1.5  2013/06/06 23:47:04  chas
 * Sorted out calculations of the available freespace so beacon id now TOTALLY DONE. Tidied up the allocator accordingly so now you only have to pass a pointer to the freespace and it sets the freespace descriptor up accordingly. So you can create beacons without freesoace or with in the allocator.
 *
 * Revision 1.4  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.3  2013/04/03 05:18:51  chas
 * Major changes yet again to flags. The set function now only needs
 * nnn.set(&_flag) rather than _flag = nnn.set(_flag)
 * much neater and shorter and less eror prone.
 * Still issues with dirent timestamps. Have to have more intelligence
 * in the timestamp constructor's so the right one is chosen.
 *
 * Revision 1.2  2013/04/01 09:57:55  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.1  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 */


