/*
 * $Id: dirent.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _DIRENT_H
#define _DIRENT_H

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <iostream>
#include <cstring>
#include <string>
using namespace std;

#include "saratoga.h"
#include "screen.h"
#include "timestamp.h"
#include "dirflags.h"

namespace saratoga 
{

/*
 * DIRECTORY ENTRY
 */
class dirent
{
private:
	char		*_dpayload;	// A dirent payload
	size_t		_dpaylen;	// This is how long the payload is
	dflag_t		_dflags;
	offset_t	_filesize;
	timestamp	*_mtime;
	timestamp	*_ctime;
	string		_fname;
	bool		_valid; // Can I access or stat this dirent ie does it exist.
public:
	// Local file on the system look its details up
	// and create a directory entry for it
	dirent(string);

	// We have been given a directory entry payload
	// so decode it and set up the values
	dirent(char *, size_t);
	
	~dirent() { if (_dpayload) delete _dpayload;
		if (_mtime) delete _mtime;
		if (_ctime) delete _ctime;
		_fname = "";
		_filesize = 0;
		_dpaylen = 0;
		_dflags = 0;
		_valid = false;
	};
 
	// Get various flags applicable to dirent
	enum d_prop	prop() { Dprop d(_dflags); return(d.prop()); };
	enum d_sod	sod() {Dsod d(_dflags); return(d.sod()); };
	enum d_descriptor	descriptor() {Ddescriptor d(_dflags); return(d.descriptor()); };; 

	// Set various flags applicable to dirent
	enum d_prop	prop(d_prop);
	enum d_sod	sod(d_sod);
	enum d_descriptor	descriptor(d_descriptor);
	
	char	*payload() { return(_dpayload); };
	size_t	paylen() { return _dpaylen; };
	dflag_t 	flags() { return _dflags; };
	timestamp	*mtime() { return _mtime; };
	timestamp	*ctime() { return _ctime; };
	string		fname() { return _fname; };
	bool		valid() { return _valid; };

	// enum d_descriptor	descriptor() { Ddescriptor d(_dflags); return d.get(); };

	/* Print the details of the dirent */
	string	printdflags();
	string	print();
};

// Handles calculating the available free space where the named file is located
// Now this returns different values than df does and I dont know why!!
// This is VERY PLATFORM DEPENDANT. I am a linux guy, you windows lot can write your own :)

class fsinfo
{
private:
		string		_fname;		// File name relative to file system
		offset_t	_blocksize; // Block Size used
		offset_t	_totblocks; // Total number of blocks in file system
		offset_t	_freeblocks; // How many of those blocks are free
		offset_t	_free1kblocks; // How many 1kB blocks there are
		enum f_freespaced	_freespaced; // What freespace descriptor we use
public:
		fsinfo(string);

		offset_t	blocksize() { return _blocksize; };
		offset_t	totalblocks() { return _totblocks; };
		offset_t	freeblocks() { return _freeblocks; };

		// Number 1kB blocks  on file system
		offset_t freespace() { return _free1kblocks; };
		// What freespace descriptor needed to hold the _free1kblocks integer
		// Now there MAY be a problem here once file system sizes exceed offset_t
		// given offset_t is a signed integer and we use unsigned integers in saratoga
		// offsets
		enum f_freespaced	freespaced() { return _freespaced; };

		string	print();

};

} // Namesapce saratoga

#endif // _DIRENT_H

/*
 * $Log: dirent.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.10  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.9  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.8  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.7  2013/09/27 06:12:15  chas
 * dflag_t size was incorrect set to correct uint16_t now
 *
 * Revision 1.6  2013/06/08 07:31:50  chas
 * Direcory entry as Gordon Ramsey would say DONE!! So it can read directory entries and get the file type and info to be ready to go into a metadata or read from to get the info.
 *
 * Revision 1.5  2013/06/07 11:49:14  chas
 * Lots if work done on dirent. Still more work to do. It currently core dumps when you put in an invalid file name. Converting a payload back into a direct isn;t working yet either.
 *
 * Revision 1.4  2013/06/06 23:47:04  chas
 * Sorted out calculations of the available freespace so beacon id now TOTALLY DONE. Tidied up the allocator accordingly so now you only have to pass a pointer to the freespace and it sets the freespace descriptor up accordingly. So you can create beacons without freesoace or with in the allocator.
 *
 * Revision 1.3  2013/04/03 05:18:51  chas
 * Major changes yet again to flags. The set function now only needs
 * nnn.set(&_flag) rather than _flag = nnn.set(_flag)
 * much neater and shorter and less eror prone.
 * Still issues with dirent timestamps. Have to have more intelligence
 * in the timestamp constructor's so the right one is chosen.
 *
 * Revision 1.2  2013/04/01 09:57:55  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.1  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 */
