/*
 * $Id: dirflags.h,v 1.1.1.1 2014/10/15 11:55:35 lloydwood Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _DIRFLAGS_H
#define _DIRFLAGS_H

using namespace std;

#include "saratoga.h"

namespace saratoga 
{

/*
 * Directory Entry Dflag Field Format - 16 bit unsigned integer (uint16_t)
 *
 *             111111
 *  01234567 89012345
 * +--------+--------+
 * |1     AA|BBC     |
 * +--------+--------+
 *
 * 1  == Directory Entry Start of Frame (Bit 0)
 * AA == Directory Entry Properties (Bits 6-7)
 * BB == Directory Entry Descriptor (Bits 8-9)
 * C  == Directory Entry File or Bundle (Bit 10)
 */

/*
 * Saratoga Directory Entry Descriptor Size - Bits 8-9
 *  METADATA, DATA
 */
class Dflag
{
protected:
	dflag_t	_shift;		// Number of bits to shift 
	dflag_t	_set;		// The mask for the bits covered
public:
	inline Dflag(const dflag_t s, const dflag_t m)	{ 
		_shift = s; _set = m << s; };
	inline ~Dflag()			{ _shift = 0; _set = 0; };
	inline dflag_t	dfget(dflag_t f)	{ return((f & _set) >> _shift); };
	inline dflag_t dfset(dflag_t c, dflag_t f) { 
		return(((c) & ~_set) | ((f) << _shift)); };
};

enum d_descriptor 
{
	D_DESCRIPTOR_16 = 0x00,
	D_DESCRIPTOR_32 = 0x01,
	D_DESCRIPTOR_64 = 0x02,
	D_DESCRIPTOR_128 = 0x03 
};

class Ddescriptor : private Dflag
{
protected:
	static const dflag_t	bits = 2;
	static const dflag_t	msb = 8;
	static const dflag_t	dshift = (16 - bits - msb);
	static const dflag_t	dmask = 0x03;
	enum d_descriptor		_descriptor;
public:
	// Constructor set the descriptor.
	Ddescriptor(enum d_descriptor f) : Dflag(dshift, dmask) { _descriptor = f; };
	Ddescriptor(dflag_t f) : Dflag(dshift, dmask) { 
		_descriptor = (enum d_descriptor) (dfget(f)); };

	// Given Current Dflag set the bits for this descriptor
	void	set(dflag_t* c) { *c = dfset(*c, _descriptor); };
	// dflag_t			set(dflag_t c) { return(dfset(c, descriptor)); };
	void	set(enum d_descriptor f) { _descriptor = f; };
	
	size_t	length();

	// What is the descriptor
	enum d_descriptor	get() { return(_descriptor); };
	enum d_descriptor	get(dflag_t f) { return((enum d_descriptor) dfget(f)); };
	enum d_descriptor	descriptor() { return(_descriptor); };

	dflag_t	shift() { return(dshift); };
	dflag_t	mask() { return(dmask); };


	// Print out the current descriptor
	string		print();
};

/*
 *  Saratoga Directory Entry Start of Directory Entry - Bit 0
 *  Must be 1
 */
enum d_sod
{
	D_SOD_NO = 0x00,	// INVALID!!!!
	D_SOD_YES = 0x01
};

class Dsod : private Dflag
{
protected:
	static const dflag_t bits = 1;
	static const dflag_t msb = 0;
	static const dflag_t dshift = (16 - bits - msb);
	static const dflag_t dmask = 0x01;
	enum d_sod		_sod;
public:
	Dsod() : Dflag(dshift, dmask) { _sod = D_SOD_YES; }; // It is always 1
	Dsod(enum d_sod f) : Dflag(dshift, dmask) { _sod = D_SOD_YES; }; // It is always 1
	Dsod(dflag_t f) : Dflag(dshift, dmask) { 
		_sod = (enum d_sod) (dfget(f)); };

	// Given Current Sflag set the bits for this sod
	void	set() { _sod = D_SOD_YES; };
	void	set(dflag_t* c) { *c = (dfset(*c, _sod)); };
	void	set(enum d_sod f ) { _sod = f; };

	// What is the sod
	enum d_sod	get() { return(_sod); };
	enum d_sod	get(dflag_t f) { return((enum d_sod) dfget(f)); };
	enum d_sod	sod() { return(_sod); };
	
	dflag_t	shift() { return(dshift); };
	dflag_t	mask() { return(dmask); };
	
	// Print out the current sod
	string		print();
};

/*
 * Saratoga Directory Entry Properties - Bits 6-7
 *  METADATA, DATA
 */
enum d_prop 
{
	D_PROP_FILE = 0x00,
	D_PROP_DIR = 0x01,
	D_PROP_SPECIALFILE = 0x02,
	D_PROP_SPECIALDIR = 0x03 
};

class Dprop : private Dflag
{
protected:
	static const dflag_t	bits = 2;
	static const dflag_t	msb = 6;
	static const dflag_t	dshift = (16 - bits - msb);
	static const dflag_t	dmask = 0x03;
	enum d_prop		_prop;
public:
	// Constructor set the prop.
	Dprop(enum d_prop f) : Dflag(dshift, dmask) { _prop = f; };
	Dprop(dflag_t f) : Dflag(dshift, dmask) { _prop = (enum d_prop) (dfget(f)); };

	// Given Current Sflag set the bits for this prop
	void	set(dflag_t* c) { *c = (dfset(*c, _prop)); };
	void	set(enum d_prop f ) { _prop = f; };
	
	// What is the prop
	enum d_prop	get() { return(_prop); };
	enum d_prop	get(dflag_t f) { return((enum d_prop) dfget(f)); };
	enum d_prop	prop() { return(_prop); };
	
	dflag_t	shift() { return(dshift); };
	dflag_t	mask() { return(dmask); };
	
	// Print out the current prop
	string		print();
};

/*
 * Saratoga Directory Entry File or Bundle - Bit 10
 *  METADATA, DATA
 */
enum d_forb 
{
	D_FORB_FILE = 0x00,
	D_FORB_BUNDLE = 0x01
};

class Dforb : private Dflag
{
protected:
	static const dflag_t	bits = 1;
	static const dflag_t	msb = 10;
	static const dflag_t	dshift = (16 - bits - msb);
	static const dflag_t	dmask = 0x01;
	enum d_forb		_forb;
public:
	// Constructor set the fileorbundle.
	Dforb(enum d_forb f) : Dflag(dshift, dmask) { _forb = f; };
	Dforb(dflag_t f) : Dflag(dshift, dmask) { _forb = (enum d_forb) (dfget(f)); };

	// Given Current Sflag set the bits for this fileorbundle
	void	set(dflag_t* c) { *c = (dfset(*c, _forb)); };
	void	set(enum d_forb f ) { _forb = f; };
	
	// What is the fileorbundle
	enum d_forb	get() { return(_forb); };
	enum d_forb	get(dflag_t f) { return((enum d_forb) dfget(f)); };
	enum d_forb	forb() { return(_forb); };
	
	dflag_t	shift() { return(dshift); };
	dflag_t	mask() { return(dmask); };
	
	// Print out the current fileorbundle
	string		print();
};

} // Namesapce saratoga

#endif // _DIRFLAGS_H

/*
 * $Log: dirflags.h,v $
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.2  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.1  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 */
