/*
 * $Id: fileio.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2014, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <iostream>
#include <cstring>
#include <string>
#include <limits>
#include <sys/statvfs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits>
#include "screen.h"
#include "sysinfo.h"
#include "fileio.h"

using namespace std;

namespace saratoga
{

fileio::fileio(string fname, enum rorw rw)
{
	string	s;
	sarwin::screen	msg;
	char	*sfname = new char[fname.length() + 1];

	_fname = fname;
	strcpy(sfname, fname.c_str());
	switch(rw)
	{
	case FILE_WRITE:
		_rorw = FILE_WRITE;
		_fd = open(sfname, O_CREAT | O_WRONLY | O_EXCL | O_LARGEFILE | O_SYNC | O_TRUNC, 
			S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
		if (_fd < 0)
		{
			msg.perror(errno, "fileio(%s): Cannot create file", sfname);
			_rorw = FILE_UNDEF;
			_fname = "";
			delete sfname;
			return;
		}
		msg.info("Created File %s for FILE_WRITE _fd=%d", sfname, _fd);
		break;
		// open a file for reading
	case FILE_READ:
		_rorw = FILE_READ;
		_fd = open(sfname, O_RDONLY | O_LARGEFILE | O_SYNC);
		if (_fd < 0)
		{
			msg.perror(errno, "fileio(%s): Cannot open file", sfname);
			_rorw = FILE_UNDEF;
			_fname = "";
			delete sfname;
			return;
		}
		msg.info("Created File %s for FILE_READ _fd=%d", sfname, _fd);
		break;
		// opne / creat a file for reading
	default:
		msg.error("fileio(%s): Undefined File open mode", sfname);
		_rorw = FILE_UNDEF;
		_fname = "";
		_fd = -1;
		delete sfname;
		return;
	}
	delete sfname;
}

// Open a file for reading
int
fileio::fcreat(string fname)
{
	sarwin::screen	msg;
	char	*sfname = new char[fname.length() + 1];

	if (_fd != -1)
	{
		msg.error("fileio::fcreat(%s): File %d already open\n", sfname, _fd);
		_rorw = FILE_UNDEF;
		_fname = "";
		delete sfname;
		return _fd;
	}
	strcpy(sfname, fname.c_str());
	_fd = open(sfname, O_CREAT | O_EXCL | O_LARGEFILE | O_SYNC | O_TRUNC, S_IRWXU | S_IROTH);
	if (_fd < 0)
	{
		msg.perror(errno, "fileio::fcreat(%s): Cannot create file", sfname);
		_rorw = FILE_UNDEF;
		_fname = "";
		delete sfname;
		return(-1);
	}
	_rorw = FILE_WRITE;
	_fname = fname;
	delete sfname;
	return(_fd);
}

// Open a file for reading
int
fileio::fopen(string fname)
{
	sarwin::screen	msg;
	char	*sfname = new char[fname.length() + 1];
	
	strcpy(sfname, fname.c_str());
	_fname = fname;
	_rorw = FILE_READ;
	_fd = open(sfname, O_LARGEFILE | O_SYNC);
	if (_fd < 0)
		msg.perror(errno, "fileio::fopen(%s): Cannot open file", sfname);
	delete sfname;
	return(_fd);
}

void
fileio::fclose()
{
	if (_fd > 2)
		close(_fd);
	_fd = -1;
	_rorw = FILE_UNDEF;
	_fname = "";
}

int
fileio::funlink()
{
	sarwin::screen	msg;
	char	*sfname = new char[_fname.length() + 1];
	int	ret;

	strcpy(sfname, _fname.c_str());
	this->fclose();
	if ((ret = unlink(sfname)) != 0)
		msg.perror(errno, "fileio::funlink(%s): ", sfname);
	delete sfname;
	return ret;
}

offset_t
fileio::fseek(offset_t offset)
{
	sarwin::screen	msg;
	offset_t	ret = 0;
	sysinfo		fs(_fname);

	if (offset > fs.diskfree())
	{
		msg.error("fileio::fseek(%d): Cannot seek beyond size of file system\n", _fd);
		return(-1);
	}
	if ((ret = lseek64(_fd, (off64_t) offset, SEEK_SET)) < 0)
	{
		msg.perror(errno, "fileio::fseek(%d):", _fd);
		return(-1);
	}
	if (ret != (off_t) offset)
	{
		msg.error("fileio::fseek(%d) cant seek to offset\n", _fd);
		return(-1);
	}
	return ret;
}

offset_t
fileio::flen()
{
	sarwin::screen	msg;
	offset_t ret = 0;
	sysinfo		fs(_fname);

	if ((ret = lseek64(_fd, (off64_t) 0, SEEK_END)) < 0)
	{
		msg.perror(errno, "fileio::flen(%d):", _fd);
		return(-1);
	}
	if (ret > fs.diskfree())
	{
		msg.error("fileio::flen(%d): Cannot seek beyond size of file system\n", _fd);
		return(-1);
	}
	return(ret);
}

ssize_t
fileio::fread(void *b, size_t count)
{
	sarwin::screen	msg;
	ssize_t i;

	if (_fd < 0 || _fd == 1 || _fd == 2)
	{
		msg.error("fileio::fread(), Cannot read from fd %d\n", _fd);
		return -1;
	}
	if ((i = read(_fd, b, count)) < 0)
	{
		msg.perror(errno, "fileio::fread(%d): Cannot read %d bytes", _fd, count);
		return(-1);
	}
	if (i != (ssize_t) count)
		msg.error("fileio::fread(%d): Only read %d bytes of %d\n", _fd, i, count);
	return(i);
}

ssize_t
fileio::fwrite(const void *b, size_t count)
{
	sarwin::screen	msg;
	ssize_t i;
	sysinfo		fs(_fname);

	if (_fd <= 0)
	{
		msg.error("fileio::fwrite(), Cannot write to fd %d\n", _fd);
		return -1;
	}
	i = write(_fd, b, count);
	if (i < 0)
	{
		msg.perror(errno, "fileio::fwrite(%d): Cannot write %d bytes", _fd, count);
		return(-1);
	}
	if (i != (ssize_t) count)
		msg.error("fileio::fwrite(%d): Only wrote %d bytes of %d\n", _fd, i, count);
	if (fdatasync(_fd) == -1)
		msg.perror(errno, "fileio::fwrite(%d): Can't fsync", _fd);
	return(i);
}

}; // namespace saratoga

/*
 * $Log: fileio.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 */

