/*
 * $Id: flags.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

/*
 * Handle the Saratoga Header and Saratoga Directory Entry Flags Fields
 * We can set them, get them, print them
 *
 * Basic principle is to have an enum that holds the values of each particular
 * flag and then a class to handle variables & functions attaining to that flag.
 *
 * There is lots of bit manipulation going on here. 
 * The saratoga header uses a 32 bit unsigned integer (uint32_t) flags field.
 * The saratoga directory header uses a 16 bit unsigned integer (uint16_t) flags field.
 */

#include <iostream>

#include <cstring>
#include <string>
#include "sarflags.h"
#include "timestamp.h"
#include "dirent.h"

using namespace std;

namespace saratoga
{

string
printbits(flag_t fshift, flag_t fmask)
{
	string s("");
#ifdef DEBUGFLAGS
	// Very verbose it prints out the flag masks
	uint32_t shift = (uint32_t) fshift;
	uint32_t mask = (uint32_t) fmask;

	bool flags[32];
	for (size_t i = 0; i < 32; i++)
		flags[i] = 0;
	size_t numbits = 0;

	switch(mask)
	{
	case 0x01: // 1
		numbits = 1;
		break;
	case 0x03: // 11
		numbits = 2;
		break;
	case 0x07: // 111
		numbits = 3;
		break;
	case 0x0F: // 1111
		numbits = 4;
		break;
	case 0x1F: // 11111
		numbits = 5;
		break;
	case 0x3F: // 111111
		numbits = 6;
		break;
	case 0x7F: // 1111111
		numbits = 7;
		break;
	case 0xFF: // 11111111
		numbits = 8;
		break;
	}
	for (size_t i = 0; i < numbits; i++)
		flags[31 - shift - i] = 1;
	s += "\n                     1                   2                   3\n";
	s += " 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1\n";
	s += "|";
	for (size_t i = 0; i < 32; i++)
	{
		if (flags[i] == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
#endif
	return s;
}

// Print out the current flag bits
string
printflags(flag_t f)
{
	string s("");
#ifdef DEBUGFLAGS

	cout << "FLAGS=" << f << endl;
	s += "\n                     1                   2                   3\n";
	s += " 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1\n";
	s += "|";
	for (int i = 0; i < 32; i++)
	{
		flag_t tmpf = f;
		tmpf <<= i;
		tmpf >>= 31;
		if (tmpf == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
#endif
	return s;
}

string
printdbits(dflag_t fshift, dflag_t fmask)
{
	string s("");
#ifdef DEBUGFLAGS
	// Very verbose it prints out the directory flag masks
	uint16_t shift = (uint16_t) fshift;
	uint16_t mask = (uint16_t) fmask;

	bool flags[16];
	for (size_t i = 0; i < 16; i++)
		flags[i] = 0;
	size_t numbits = 0;

	switch(mask)
	{
	case 0x01: // 1
		numbits = 1;
		break;
	case 0x03: // 11
		numbits = 2;
		break;
	case 0x07: // 111
		numbits = 3;
		break;
	case 0x0F: // 1111
		numbits = 4;
		break;
	case 0x1F: // 11111
		numbits = 5;
		break;
	case 0x3F: // 111111
		numbits = 6;
		break;
	case 0x7F: // 1111111
		numbits = 7;
		break;
	case 0xFF: // 11111111
		numbits = 8;
		break;
	}
	for (size_t i = 0; i < numbits; i++)
		flags[15 - shift - i] = 1;
	s += "\n                   1\n";
	s += " 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5\n";
	s += "|";
	for (size_t i = 0; i < 16; i++)
	{
		if (flags[i] == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	// Now print out what the flags are set to
	/*
	for (size_t i = 0; i < 16; i++)
	{
		if (f >> i == 1)
			s += "1|";
		else
			s += " |";
	}
	*/
	s += "\n";
#endif
	return s;
}

/*
 **********************************************************************************************************
 * Fversion functions
 */

// Print out the current saratoga version
string
Fversion::print()
{
	string s("Saratoga ");

	switch(Fversion::get())
	{
	case F_VERSION_0:
		s += "Version 0";
		break;
	case F_VERSION_1:
		s += "Version 1";
		break;
	default:
		s += "Unsupported Version";
		break;
	}
	s += printbits(Fversion::shift(), Fversion::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fframetype functions
 */

// Print out the frametype 
string
Fframetype::print()
{
	string s("");

	switch(Fframetype::get())
	{
	case F_FRAMETYPE_BEACON:
		s += "BEACON";
		break;
	case F_FRAMETYPE_REQUEST:
		s += "REQUEST";
		break;
	case F_FRAMETYPE_METADATA:
		s += "METADATA";
		break;
	case F_FRAMETYPE_DATA:
		s += "DATA";
		break;
	case F_FRAMETYPE_STATUS:
		s += "ERRCODE";
		break;
	default:
		s += "Unrecognised Frame Type";
		break;
	}
	s += printbits(Fframetype::shift(), Fframetype::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fdescriptor functions
 */

// Print out the descriptor 
string
Fdescriptor::print()
{
	string s("Descriptor ");

	switch(Fdescriptor::get())
	{
	case F_DESCRIPTOR_16:
		s += "16 Bit";
		break;
	case F_DESCRIPTOR_32:
		s += "32 Bit";
		break;
	case F_DESCRIPTOR_64:
		s += "64 Bit";
		break;
	case F_DESCRIPTOR_128:
#ifdef UINT128_T
		s+= "128 Bit";
#else
		s += "128 Bit UNSUPPORTED";
#endif
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fdescriptor::shift(), Fdescriptor::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fbundle functions
 */

// Print out the bundle 
string
Fbundle::print()
{
	string s("Bundles ");

	switch(Fbundle::get())
	{
	case F_BUNDLE_NO:
		s += "Off";
		break;
	case F_BUNDLE_YES:
		s += "On";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fbundle::shift(), Fbundle::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fstream funtions
 */

// Print out the stream 
string
Fstreams::print()
{
	string s("Streams ");

	switch(Fstreams::get())
	{
	case F_STREAMS_NO:
		s += "Off";
		break;
	case F_STREAMS_YES:
		s += "On";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fstreams::shift(), Fstreams::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Ftransfer functions
 */

// Print out the transfer 
string
Ftransfer::print()
{
	string s("Transfering ");

	switch(Ftransfer::get())
	{
	case F_TRANSFER_FILE:
		s += "File";
		break;
	case F_TRANSFER_DIR:
		s += "Directory";
		break;
	case F_TRANSFER_BUNDLE:
		s += "Bundle";
		break;
	case F_TRANSFER_STREAM:
		s += "Stream";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Ftransfer::shift(), Ftransfer::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Ftimestamp functions
 */

// Print out the timestamp 
string
Ftstamp::print()
{
	string s("Timestamps ");

	switch(Ftstamp::get())
	{
	case F_TIMESTAMP_NO:
		s += "Off";
		break;
	case F_TIMESTAMP_YES:
		s += "On";
		break;
	default:
		s + "Invalid";
		break;
	}
	s += printbits(Ftstamp::shift(), Ftstamp::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fprogress functions
 */

// Print out the progress 
string
Fprogress::print()
{
	string s("Progress ");

	switch(Fprogress::get())
	{
	case F_PROGRESS_INPROG:
		s += "In Progress";
		break;
	case F_PROGRESS_TERMINATED:
		s += "Terminated";
		break;
	default:
		s += "Invalid";
		break;
	}

	s += printbits(Fprogress::shift(), Fprogress::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Ftxwilling functions
 */

// Print out the txwilling 
string
Ftxwilling::print()
{
	string s("TX Willing ");

	switch(Ftxwilling::get())
	{
	case F_TXWILLING_NO:
		s += "No";
		break;
	case F_TXWILLING_INVALID:
		s += "Invalid";
		break;
	case F_TXWILLING_CAPABLE:
		s += "Yes but not right now";
		break;
	case F_TXWILLING_YES:
		s += "Yes";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Ftxwilling::shift(), Ftxwilling::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Freliable functions
 */

// Print out the metadata UDP Only Flag 
string
Fmetadata_udptype::print()
{
	string s("Udplite Unreliability ");

	switch(Fmetadata_udptype::get())
	{
	case F_UDPONLY:
		s += "Only";
		break;
	case F_UDPLITE:
		s += "or UDPLITE";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fmetadata_udptype::shift(), Fmetadata_udptype::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fmetadatarecvd functions
 */

// Print out the metadatarecvd 
string
Fmetadatarecvd::print()
{
	string s("Metadata ");

	switch(Fmetadatarecvd::get())
	{
	case F_METADATARECVD_YES:
		s += "Received";
		break;
	case F_METADATARECVD_NO:
		s += "Not Received";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fmetadatarecvd::shift(), Fmetadatarecvd::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fallholes functions
 */

// Print out the allholes 
string
Fallholes::print()
{
	string s("All Holes ");

	switch(Fallholes::get())
	{
	case F_ALLHOLES_YES:
		s += "Received";
		break;
	case F_ALLHOLES_NO:
		s += "Not Received Yet";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fallholes::shift(), Fallholes::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fsessiontype functions
 */

// Print out the sessiontype 
string
Fsessiontype::print()
{
	string s("Session ");

	switch(Fsessiontype::get())
	{
	case F_SESSIONTYPE_GETFILEORDIR:
		s += "Get File or Directory";
		break;
	case F_SESSIONTYPE_DELETEFILE:
		s += "Delete File";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fsessiontype::shift(), Fsessiontype::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Frxwilling functions
 */

// Print out the rxwilling 
string
Frxwilling::print()
{
	string s("RX Willing ");

	switch(Frxwilling::get())
	{
	case F_RXWILLING_NO:
		s += "No";
		break;
	case F_RXWILLING_INVALID:
		s += "Invalid";
		break;
	case F_RXWILLING_CAPABLE:
		s += "Yes but not right now";
		break;
	case F_RXWILLING_YES:
		s += "Yes";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Frxwilling::shift(), Frxwilling::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fholes functions
 */

// Print out the holes 
string
Fholes::print()
{
	string s("Holes ");

	switch(Fholes::get())
	{
	case F_HOLES_REQUESTED:
		s += "Requested";
		break;
	case F_HOLES_SENTVOLUNTARILY:
		s += "Sent Voluntarily";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fholes::shift(), Fholes::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Ffileordir functions
 */

// Print out the fileordir 
string
Ffileordir::print()
{
	string s("Transferring ");

	switch(Ffileordir::get())
	{
	case F_FILEORDIR_FILE:
		s += "File";
		break;
	case F_FILEORDIR_DIRECTORY:
		s += "Directory";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Ffileordir::shift(), Ffileordir::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Freqstatus functions
 */

// Print out the reqstatus 
string
Freqstatus::print()
{
	string s("Status ");

	switch(Freqstatus::get())
	{
	case F_REQSTATUS_NO:
		s += "Not Requested";
		break;
	case F_REQSTATUS_YES:
		s += "Requested";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Freqstatus::shift(), Freqstatus::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fudptype functions
 */

// Print out the udptype 
string
Fudptype::print()
{
	string s("UDP ");

	switch(Fudptype::get())
	{
	case F_UDPONLY:
		s += "Only";
		break;
	case F_UDPLITE:
		s += "or UDPLITE";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Fudptype::shift(), Fudptype::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Feod functions
 */

// Print out the eod 
string
Feod::print()
{
	string s("End of Data ");

	switch(Feod::get())
	{
	case F_EOD_NO:
		s += "No";
		break;
	case F_EOD_YES:
		s += "Set";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Feod::shift(), Feod::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Ffreespace functions
 */

// Print out the freespace 
string
Ffreespace::print()
{
	string s("Freespace ");

	switch(Ffreespace::get())
	{
	case F_FREESPACE_NO:
		s += "Not Advertised";
		break;
	case F_FREESPACE_YES:
		s += "Advertsed";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Ffreespace::shift(), Ffreespace::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Ffreespace descriptor functions
 */

// Print out the freespace descriptor size
string
Ffreespaced::print()
{
	string s("");

	switch(Ffreespaced::get())
	{
	case F_FREESPACED_16:
		s += "16 Bit";
		break;
	case F_FREESPACED_32:
		s += "32 Bit";
		break;
	case F_FREESPACED_64:
		s += "64 Bit";
		break;
	case F_FREESPACED_128:
#ifdef UINT128_T
		s += "128 Bit";
#else
		s += "128 Bit UNSUPPORTED";
#endif
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Ffreespaced::shift(), Ffreespaced::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fcsumlen functions
 */

// Print out the csum 
string
Fcsumlen::print()
{
	string s("Csum Len ");

	switch(Fcsumlen::get())
	{
	case F_CSUMLEN_NONE:
		s += "0 bits";
		break;
	case F_CSUMLEN_CRC32:
		s += "32 bits";
		break;
	case F_CSUMLEN_MD5:
		s += "128 bits";
		break;
	case F_CSUMLEN_SHA1:
		s += "160 bits";
		break;
	default:
		s += "Unsupported";
		break;
	}
	s += printbits(Fcsumlen::shift(), Fcsumlen::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fcsum functions
 */

// Print out the csum 
string
Fcsum::print()
{
	string s("Checksum ");

	switch(Fcsum::get())
	{
	case F_CSUM_NONE:
		s += "None";
		break;
	case F_CSUM_CRC32:
		s += "CRC32";
		break;
	case F_CSUM_MD5:
		s += "MD5";
		break;
	case F_CSUM_SHA1:
		s += "SHA1";
		break;
	default:
		s += "Invalid";
		break;
	}
	// s += printbits(Fcsum::shift(), Fcsum::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Fstatus functions
 */

// Print out the Error Code 
string
Ferrcode::print()
{
	string s("Error Code ");

	switch(Ferrcode::get())
	{
	case F_ERRCODE_SUCCESS:
		s += "Success";
		break;
	case F_ERRCODE_UNSPEC:
		s += "Unspecified Error";
		break;
	case F_ERRCODE_NOSEND:
		s += "Cannot Send";
		break;
	case F_ERRCODE_NORECEIVE:
		s += "Cannot Receive";
		break;
	case F_ERRCODE_NOFILE:
		s += "File Not Found";
		break;
	case F_ERRCODE_NOACCESS:
		s += "Access Denied";
		break;
	case F_ERRCODE_NOID:
		s += "No Transaction ID Received";
		break;
	case F_ERRCODE_NODELETE:
		s += "Cannot Delete File";
		break;
	case F_ERRCODE_TOOBIG:
		s += "File is Too Big";
		break;
	case F_ERRCODE_BADDESC:
		s += "Bad Descriptor";
		break;
	case F_ERRCODE_BADPACKET:
		s += "Bad Saratoga Frame";
		break;
	case F_ERRCODE_BADFLAG:
		s += "Invalid Saratoga Flag";
		break;
	case F_ERRCODE_SHUTDOWN:
		s += "Shut Down Transaction";
		break;
	case F_ERRCODE_PAUSE:
		s += "Pause Transaction";
		break;
	case F_ERRCODE_RESUME:
		s += "Resume Transaction";
		break;
	case F_ERRCODE_INUSE:
		s += "File currently in use";
		break;
	case F_ERRCODE_NOMETADATA:
		s += "Metadata Has Not Been Received";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printbits(Ferrcode::shift(), Ferrcode::mask());
	return(s);
}

/*
 **********************************************************************************************************
 **********************************************************************************************************
 */

/*
 **********************************************************************************************************
 * Ddescriptor functions
 */

// Print out the descriptor 
string
Ddescriptor::print()
{
	string s("Directory Descriptor ");

	switch(Ddescriptor::get())
	{
	case D_DESCRIPTOR_16:
		s += "16 Bit";
		break;
	case D_DESCRIPTOR_32:
		s += "32 Bit";
		break;
	case D_DESCRIPTOR_64:
		s += "64 Bit";
		break;
	case D_DESCRIPTOR_128:
#ifdef UINT128_T
		s += "128 Bit";
#else
		s += "128 Bit UNSUPPORTED";
#endif
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printdbits(Ddescriptor::shift(), Ddescriptor::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Dproperties functions
 */

// Print out the properties 
string
Dprop::print()
{
	string s("Directory Properties ");

	switch(Dprop::get())
	{
	case D_PROP_FILE:
		s += "File";
		break;
	case D_PROP_DIR:
		s += "Directory";
		break;
	case D_PROP_SPECIALFILE:
		s += "Special File";
		break;
	case D_PROP_SPECIALDIR:
		s += "Special Directory";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printdbits(Dprop::shift(), Dprop::mask());
	return(s);
}

/*
 **********************************************************************************************************
 * Dfileorbundle functions
 */

// Print out the fileorbundle 
string
Dforb::print()
{
	string s("Directory file or bundle type is ");

	switch(Dforb::get())
	{
	case D_FORB_FILE:
		s += "File";
		break;
	case D_FORB_BUNDLE:
		s += "Bundle";
		break;
	default:
		s += "Invalid";
		break;
	}
	s += printdbits(Dforb::shift(), Dforb::mask());
	return(s);
}

// Start of Directory Entry Bit
string
Dsod::print()
{
		string s("Start of Directory Entry is ");

		switch(Dsod::get())
		{
		case D_SOD_YES:
			s += "Set";
			break;
		default:
			s += "Invalid";
		}
		s += printdbits(Dsod::shift(), Dsod::mask());
		return(s);
}

/*
 **********************************************************************************************************
 * Ttimestamp functions
 */

// Print out the timestamp flags
string
Ttimestamp::print()
{
	sarwin::screen	msg;

	string s("Timestamp ");
	switch(Ttimestamp::get())
	{
	case T_TSTAMP_UDEF:
		s += "Undef";
		break;
	case T_TSTAMP_32:
		s += "32";
		break;
	case T_TSTAMP_64:
		s += "64";
		break;
	case T_TSTAMP_32_32:
		s += "32_32";
		break;
	case T_TSTAMP_64_32:
		s += "64_32";
		break;
	case T_TSTAMP_32_Y2K:
		s += "32_Y2K";
		break;
	default:
		msg.error("Invalid Timestamp Flag: %u", (uint16_t) Ttimestamp::get());
		s += "Invalid";
		break;
	}
	s += " ";
	return(s);
}

// How may bytes in my descriptor
size_t
Fdescriptor::length()
{
	sarwin::screen	msg;

	switch(Fdescriptor::get())
	{
	case F_DESCRIPTOR_16:
		return sizeof(uint16_t);
	case F_DESCRIPTOR_32:
		return sizeof(uint32_t);
	case F_DESCRIPTOR_64:
		return sizeof(uint64_t);
	case F_DESCRIPTOR_128:
#ifdef UINT128_T
		return sizeof(uint128_t);
#else
		msg.error("Descriptor size of 128 bit illegal");
#endif
		break;
	}
	msg.error("Descriptor size illegal");
	return 2;
}

// How may bytes in my freespace descriptor
size_t
Ffreespaced::length()
{
	sarwin::screen	msg;

	switch(Ffreespaced::get())
	{
	case F_FREESPACED_16:
		return sizeof(uint16_t);
	case F_FREESPACED_32:
		return sizeof(uint32_t);
	case F_FREESPACED_64:
		return sizeof(uint64_t);
	case F_FREESPACED_128:
#ifdef UINT128_T
		return sizeof(uint128_t);
#else
		msg.error("Descriptor size of 128 bit illegal");
#endif
		break;
	}
	msg.error("Freespace Descriptor size illegal");
	return 2;
}

}; // namespace saratoga

/*
 **********************************************************************************************************
 **********************************************************************************************************
 */

/*
 * $Log: flags.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.19  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.18  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.17  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.16  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.15  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.14  2013/09/27 06:06:08  chas
 * Directory Flag length change 16 bits
 *
 * Revision 1.13  2013/06/07 11:49:14  chas
 * Lots if work done on dirent. Still more work to do. It currently core dumps when you put in an invalid file name. Converting a payload back into a direct isn;t working yet either.
 *
 * Revision 1.12  2013/06/05 07:20:08  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.11  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.10  2013/03/31 02:07:53  chas
 * Timestamp adding a type to handle bundle timestamps which
 * are required in the dirent structure. THese timestamps
 * have an epoch 1 Jan 2000. Not same as posix which is 1970.
 * All the maths is there to handle them as well if you mix
 * and match between the differnt epoch's
 *
 * Revision 1.9  2013/03/30 09:16:40  chas
 * Tidying up flag formats and comments.
 *
 * Revision 1.8  2013/03/30 06:17:07  chas
 * Major fixup of flags. We now have the number of bits for each flag
 * and the most significant bit. This along wth the formula
 * 32 - bits - msb gives us the correct shift to do
 *
 * Revision 1.7  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 * Revision 1.6  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.5  2013/03/28 08:56:35  chas
 * Moe bugs in the flags for data. Im shifting the worng number of bits.
 * Think this will be a generic problem to sort as we go through the flags
 * for each frame type.
 *
 * Revision 1.4  2013/03/28 08:22:37  chas
 * Found an error in flags. Some wrong bit shifting for data flags
 * Progressing along with data class. And making printf's a bit more
 * portable with PRIuXX's for _session and _offset types
 *
 * Revision 1.3  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.2  2013/03/15 12:52:01  chas
 * Catching 128 bit Descriptor illegalities in the Flags Code
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 * Revision 1.6  2011-02-22 10:19:14  chas
 * Removed nast #defines and replaced with static const for all of the
 * flag bit handling classes
 *
 * Revision 1.5  2011-01-18 10:15:43  chas
 * Refining logging and error printing classes
 *
 * Revision 1.4  2011-01-05 05:28:32  chas
 * Changes to make flags classes available to all modules
 * and starting to get beacon class implemented.
 * The flags.h defines all of the flag classes and flag.c
 * contains the functions to manipulate them. The set functions
 * set a flag based upon its enumarated toye or if you pass
 * it a uint32_t header then it sets the bits in that flag.
 * Get works similarly.
 *
 * Revision 1.3  2011-01-03 23:05:19  chas
 * Changes License to official 3 Clause BSD License with Charles Smith as Author and
 * Vallona Networks as Organisation.
 * This will be used as the official License in all codeset and binaries of this
 * implementation.
 *
 * Revision 1.2  2010-12-30 13:17:26  chas
 * Moved the enumerator types to flags.h and removed their defintions
 * from flags.cpp so other modules can refer to the enumerated types
 *
 * Revision 1.1.1.1  2010-12-30 12:35:45  chas
 * Open source development of Saratoga in C++ under a 2 clause BSD License
 * Date of commencement December 30 2010 - During the XMAS Holiday break
 * flags.cpp uses classes for all flags used in the saratoga 32 bit header
 * and the 16 bit directory entry headers.
 *
 */

