/*
 * $Id: holes.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2014, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <iostream>
#include <cstring>
#include <string>
#include <limits>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "holes.h"


namespace saratoga
{

// Compare two holes used in sort
bool
compare_hole(hole &h1, hole &h2)
{
	if (h1.begin() > h2.begin())
		return false;
	if (h1.begin() == h2.begin() && h1.end() > h2.end())
		return false;
	return true;
}

/*
 **********************************************************************
 * HOLES
 **********************************************************************
 */

// Create a hole, make sure it is valid and won;t exceed limits of unsigned integer
// Holes are defined by a begin (0 being the first byte) and a length
// They are actually stored as an encopassed begin & end
hole::hole(offset_t s, offset_t l)
{
 	sarwin::screen        msg;
	offset_t        max = std::numeric_limits<offset_t>::max();
	
	_begin = s;
	if ( l == 0)
	{
		msg.error("hole(%" PRIu64 ", %" PRIu64 "): Hole size of 0",
			(uint64_t) s, (uint64_t) l);
		_end = 0;
		return;
	}
	if ((s + l) > max || (s + l) < 0) // We have wrapped offset
	{
		msg.error("hole(%" PRIu64 ", %" PRIu64"): Hole size wraps offset_t",
			(uint64_t) s, (uint64_t) l);
		_end = 0;
		return;
	}
	_end = _begin + l - 1;
}

// Reset an existing hole to different begin & length
void
hole::set(offset_t s, offset_t l) 
{
	sarwin::screen        msg;
	offset_t        max = std::numeric_limits<offset_t>::max();

	if ( l == 0)
	{
		msg.error("hole.set(%" PRIu64 ", %" PRIu64 "): Hole size of 0",
			(uint64_t) s, (uint64_t) l);
		_begin = 0;
		_end = 0;
		return;
	}
	if ((s + l) > max || (s + l) < s) // We have wrapped offset
	{
		msg.error("hole.set(%" PRIu64 ", %" PRIu64"): Hole size wraps offset_t",
			(uint64_t) s, (uint64_t) l);
		_begin = 0;
		_end = 0;
		return;
	}
	_begin = s;
	_end = _begin + l - 1;
}

// Set the begin offset of an existing hole
void
hole::setbegin(offset_t s)
{
	sarwin::screen        msg;
	offset_t        max = std::numeric_limits<offset_t>::max();

	if (_end < s)
	{
		msg.error("Invalid hole.setbegin(%" PRIu64 ", %" PRIu64"): End < Start",
			(uint64_t) s, (uint64_t) _end);
		return;
	}
	if ((_end - s) > max || (s - _end) > s) // We have wrapped offset
	{
		msg.error("Invalid hole.setbegin(%" PRIu64 ", %" PRIu64"): Hole size wraps offset_t",
			(uint64_t) s, (uint64_t) _end);
		return;
	}
	_begin = s;
}

// Set's the end offset of an existing hole
void
hole::setend(offset_t e)
{
	sarwin::screen        msg;
	offset_t        max = std::numeric_limits<offset_t>::max();

	if (_begin > e)
	{
		msg.error("Invalid hole.setend(%" PRIu64 ", %" PRIu64"): Start > End",
			(uint64_t) _begin, (uint64_t) e);
		return;
	}
	if (e > max) // We have wrapped offset
	{
		msg.error("Invalid hole.setend(%" PRIu64 ", %" PRIu64"): Hole size wraps offset_t",
			(uint64_t) _begin, (uint64_t) e);
		return;
	}
	if (e == 0)
	{
		msg.warning("hole.setend(%" PRIu64 ", %" PRIu64"): End is 0",
			(uint64_t) _begin, (uint64_t) e);
		_begin = 0;
		_end = 0;
		return;
	}
	_end = e;
}

void
hole::setlength(offset_t l)
{
	sarwin::screen        msg;
	offset_t        max = std::numeric_limits<offset_t>::max();

	if (l == 0)
	{
		_begin = 0;
		_end = 0;
	}
	if ( _end - _begin == 0)
	{
		msg.warning("hole.setlength(%" PRIu64 ", %" PRIu64 "): Hole size of 0",
			(uint64_t) _begin, (uint64_t) l);
		return;
	}
	if ((_begin + l) > max || (_begin + l) < _begin) // We have wrapped offset
	{
		msg.error("hole.setgth(%" PRIu64 ", %" PRIu64"): Hole size wraps offset_t",
			(uint64_t) _begin, (uint64_t) l);
		return;
	}
	_end = _begin + l - 1;
}

string
hole::print() 
{
	char	tmp[128];
	string s = "";

	sprintf(tmp, "hole: Start=%" PRIu64 " End=%" PRIu64 " Length=%" PRIu64 "",
		(uint64_t) this->begin(), (uint64_t) this->end(), this->length());
	s += tmp;
	return(s);
}

/*
 ***********************************************************************************
 * Handle lists of holes
 ***********************************************************************************
 */

// Create a holes list with a first member
holes::holes(offset_t begin, offset_t len) 
{
	sarwin::screen	msg;
	
	hole h(begin, len);
	if (h.length() != 0)
	{
		msg.error("holes(%" PRIu64 ", %" PRIu64 "): Maximum # %zu holes in list already)",
			begin,
			len,
			_max);
		return;
	}
	msg.debug(1, "Created First Hole: ");
	msg.debug(1, h.print());
	if (len > 0)
		_holes.push_back(h);
}

// Given a hole Create a holes list with a first member
holes::holes(hole &h) 
{
	sarwin::screen	msg;

	if (_holes.size() != _max)
	{
		msg.error("holes(%" PRIu64 ", %" PRIu64 "): Maximum # %zu holes in list already)",
			h.begin(),
			h.length(),
			_max);
		return;
	}
	msg.debug(1, "Created First Hole: ");
	msg.debug(1, h.print());
	if (h.length() != 0)
		_holes.push_back(h);
}

// Print list of holes
string
holes::print()
{
	string s = "Holes:\n";
	for (std::list<hole>::iterator i = _holes.begin(); i != _holes.end(); i++)
	{
		s += "\t";
		s += i->print();
		s += "\n";
	}
	return(s);
};

// If a hole exists then yes
bool
holes::exists(const hole &h)
{
	for (std::list<hole>::iterator i = _holes.begin(); i != _holes.end(); i++)
	{
		if ( *i == h )
			return(true);
	}
	return (false);
}

// If a hole exists then yes
bool
holes::exists(offset_t s, offset_t l)
{
	hole h(s, l);
	for (std::list<hole>::iterator i = _holes.begin(); i != _holes.end(); i++)
	{
		if ( *i == h )
			return(true);
	}
	return (false);
}

// Fix up a sorted list of holes removing
// duplicate and holes that are redundant
// Keeps the list of holes as small as possible
void
holes::addcompress()
{
	string	s;
	sarwin::screen	msg;

	msg.debug(1, "Fixlist Called with current list");

	// We must begin with a sorted list of holes
	_holes.sort(compare_hole);

	// Keep on traversing the list till we are optimised		
scanagain:
	for (std::list<hole>::iterator curhole = _holes.begin(); curhole != _holes.end(); curhole++)
	{
		s = "Current is: " + curhole->print();
		msg.debug(1, s);
		// What was in the previous hole
		if (_holes.size() <= 1)
			return;
		if (curhole == _holes.end())
			return; // We are done as we are at last hole in the list;
		// Do we have a previous hole in the list
		std::list<hole>::iterator nexthole = curhole;
		nexthole++; // Point to the next hole
		// We have two holes begin at the same place
		if (curhole->begin() == nexthole->begin())
		{
			s = "curbegin == nextbegin " + curhole->print() + " " + nexthole->print();
			msg.debug(1, s);
			// The holes are the same so remove dups and smaller holes
			if (curhole->length() <= nexthole->length())
			{
				hole tmp = *curhole;
				_holes.remove(*curhole); // removes them all
				_holes.push_back(tmp); // put one back
				goto scanagain;
			}
			else // The next hole is encapsulated by the first so remove it
			{
				_holes.remove(*nexthole);
				goto scanagain;
			}
		}
		// Our current hole extends into the next hole so fix the next hole
		if (curhole->end() >= nexthole->begin())
		{
			s = "curend >= nextbegin " + curhole->print() + " " + nexthole->print();
			msg.debug(1, s);
			// Our current hole extends beyond the next hole
			if (nexthole->end() <= curhole->end() + 1)
			{
				_holes.remove(*nexthole);
				goto scanagain;
			}
			offset_t newlen = nexthole->end() - curhole->end();
			nexthole->set(curhole->end() + 1, newlen);
			goto scanagain;
		}
		// Merge adjacent holes as required
		if (curhole->end() + 1 >= nexthole->begin())
		{
			offset_t tmp = nexthole->end() - curhole->begin() + 1;
			curhole->setlength(tmp);
			_holes.remove(*nexthole);
			goto scanagain;
		}	
	}
}

// Add a hole to the list
void
holes::add(offset_t begin, offset_t len)
{
	sarwin::screen	msg;

	hole newhole(begin, len);

	// We don't add holes of length 0
	if (newhole.length() == 0)
		return;

	// We are the first hole in the list just push it on
	if (_holes.size() == 0)
	{
		msg.debug(1, "Added First Hole: ");
		msg.debug(1,  newhole.print());
		_holes.push_back(newhole);
		return;
	}
	else
	{
		// Put the hole at the end of the list
		msg.debug(1, "Added Next Hole: ");
		msg.debug(1, newhole.print());
		_holes.push_back(newhole);
		// Now optimse the holes list
		this->addcompress();
		return;
	}
}

void
holes::remove(offset_t begin, offset_t len)
{
	sarwin::screen	msg;
	hole fill(begin, len);

	if (fill.length() == 0)
		return;

	msg.debug(1,"Removing Hole: ");
	msg.debug(1, fill.print());
	// No holes in the current list
	if (_holes.size() == 0)
	{
		msg.debug(1,"No holes in list: ");
		return;
	}

	// We have a list of holes
	std::list<hole>::iterator curhole = _holes.begin();
	while (curhole != _holes.end())
	{
		msg.debug(1, "Current Hole is: ");
		msg.debug(1, curhole->print());
		// The hole is further along on the list
		// -------------			Curhole
		//              ------------->		Hole
		if (fill.begin() > curhole->end()) // Try next
		{
			msg.debug(2, "A");
			curhole++;
			continue;
		}
		if (fill.end() < curhole->begin())
		{
			msg.debug(2, "B");
			break;
		}

		if (fill.begin() <= curhole->begin())
		{
			msg.debug(2, "C");
			// Adjust fill if required
			if (fill.begin() != curhole->begin())
			{
				msg.debug(2, "0");
				fill.set(curhole->begin(), fill.end() - curhole->begin() + 1);
			}
			if (fill.end() == curhole->end()) // Just remove the hole
			{
				// Just delete the current hole
				// -------------	Curhole
				// -------------	Fill
				// 			Curhole After
				// So just delete it
				msg.debug(2, "1");
				_holes.remove(*curhole);
				break;
			}
			if (fill.end() > curhole->end())
			{
				// Fill extends beyond current hole
				// delete it and adjust fill
				// -------------	Curhole
				// -------------->	Fill
				// 			Curhole After
				//              ->	Fill After
				// So delete it and move on to the next hole
				msg.debug(2, "2");
				hole	tmp = *curhole;
				curhole++;
				_holes.remove(tmp);
				fill.set(tmp.end() + 1, fill.length() - (tmp.end() - fill.begin()) - 1); 
				continue;
			}
			if (fill.end() < curhole->end())
			{
				// Fill is at the beginning of this hole
				// Adjust the hole and we are done
				// 01234567890
				// ----------   Curhole curhole
				// ---          Fill
				//    -------     Newholes
				msg.debug(2, "3");
				curhole->set(fill.end() + 1, curhole->end() - fill.end());
				break;
			}
			msg.error("holes.remove(%" PRIu64 ", %" PRIu64"): INVALID CAN'T GET HERE\n", 
				fill.begin(), fill.end());
		}
		if (fill.begin() > curhole->begin())
		{
			msg.debug(2, "D");
			if (fill.end() == curhole->end())
			{
				msg.debug(2, "1");
				// 012345678901
				// ------------		Curhole  12 - (11 - 6) = 6
				//       ------		Fill
				// ------		Curhole After
				curhole->setlength(curhole->length() - (fill.end() - fill.begin()));
				break;
			}
			if (fill.end() > curhole->end())
			{
				msg.debug(2, "2");
				// 012345678901234567
				// x          y
				// ------------         Curhole
				//      a          b
				//      ------------     Fill
				//     c                        
				// -----                Curhole After
				//             d   e
				//             -----	Fill After

				// c = a -1
				// d = y + 1
				// b = e
				// Fill len = e - d + 1
				// curhole len = a - x
				offset_t tmp = curhole->end();
				curhole->setlength(fill.begin() - curhole->begin());
				fill.set(tmp + 1, fill.end() - tmp);
				curhole++;
				continue;
			}
			if (fill.end() < curhole->end())
			{
				msg.debug(2, "3");
				hole newfront(curhole->begin(), fill.begin() - curhole->begin());
				hole newend(fill.end() + 1, curhole->end() - fill.end());
				// Change the current holes end position
				curhole->setlength(fill.begin() - curhole->begin());
				// And add a new one to the end part
				this->add(newend.begin(), newend.length());
				break;
			}
		}
		msg.debug(2, "E");
		curhole++;	
	}
	msg.debug(2, "F");
	msg.debug(2, "\n");
}

}; // namespace saratoga

/*
 * $Log: holes.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 */

