/*
 * $Id: holes.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2014, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _HOLES_H
#define _HOLES_H

#include <iostream>
#include <cstring>
#include <string>
#include <list>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"

namespace saratoga
{
/*
 **********************************************************************
 * HOLES
 **********************************************************************
 */

// A container for a single hole
class hole
{
private:
	offset_t	_begin;	// Where the hole begin
	offset_t	_end;	// How long the hole is
public:
	// Create a hole begin at s with length l

	hole(offset_t s, offset_t l);

	// Create new hole from existing hole
	hole(const hole &h)
	{
		_begin = h._begin;
		_end = h._end;
	}

	~hole() { };

	hole& operator =(const hole &h)
	{
		_begin = h._begin;
		_end = h._end;
		return(*this);
	};

	bool operator ==(const hole &rhs)
	{
		return( rhs._begin == _begin && rhs._end == _end);
	};

	bool operator !=(const hole &rhs)
	{
		return !( rhs._begin == _begin && rhs._end == _end);
	};

	// Comparison of 2 holes
	bool is_equal(const hole &h1, const hole &h2)
	{
		return (h1._begin == h2._begin && h1._end == h2._end);
	};
	

	// Set begin and length of an existing hole
	void 		set(offset_t s, offset_t l);
	// Set begin and offset of an existing hole
	void 		setbegin(offset_t s);
	// Set end of an existing hole
	void 		setend(offset_t e);
	// Set len (adjust end) of an existing hole
	void		setlength(offset_t l);

	// The begin of the hole
	offset_t		begin() { return _begin; };

	// How long is the hole
	offset_t		length() { if (_end >= _begin)
					return (_end - _begin + 1); 
					return 0; };

	// The end of the hole
	offset_t		end() { return _end; };

	string	print();
};

// Compare 2 holes to sort a list
bool compare_hole(hole &h1, hole &h2);

// A container for multiple holes

class holes
{
private:
	const size_t	_max = 10; // A maximum # of holes
	std::list<hole>	_holes;
	void addcompress(); // Compress list of holes after one added
public:
	// Empty holes container
	holes() { };

	~holes() { this->zap(); };

	// Holes with first entry in list
	holes(offset_t begin, offset_t len);

	// Holes with first entry in list
	holes(hole &h);

	// Add hole to the list of holes
	void add(offset_t begin, offset_t len);

	// Zap the whole lsit of holes
	void zap()
	{
		while (!_holes.empty())
		{
			// delete _holes.front();
			_holes.pop_front();
		}
	};

	// Shortcut to add a new hole to a list
	holes& operator +=(hole &h)
	{
		this->add(h.begin(), h.length());
		return(*this);
	}

	// Shortcut to add a new hole list to a list of holes
	holes& operator +=(holes &hl)
	{
		for (std::list<hole>::iterator i = hl._holes.begin(); i != hl._holes.end(); i++)
			this->add(i->begin(), i->length());
		return(*this);
	}

	// Remove a hole from the list of holes
	void remove(offset_t s, offset_t l);

	// Remove a hole from the list of holes
	void remove(hole &h)
	{
		this->remove(h.begin(), h.length());
	}

	holes& operator -=(hole &h)
	{
		this->remove(h.begin(), h.length());
		return(*this);
	}

	// Remove a list of holes from a hole list
	holes& operator -=(holes &hl)
	{
		for (std::list<hole>::iterator i = hl._holes.begin(); i != hl._holes.end(); i++)
			this->remove(i->begin(), i->length());
		return(*this);
	}

	// true if a matching hole exists
	bool exists(const hole &h);
	
	bool exists(offset_t s, offset_t l);


	// Print the list of holes
	string print();
};

} // Namespace saratoga

#endif // _HOLES_H

/*
 * $Log: holes.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 *
 */

