/*
 * $Id: ip.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2014, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _IP_H
#define _IP_H

#include <iostream>
#include <cstring>
#include <string>
#include <list>
#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS	// So we can use STD_u64 in printf's
#endif
#include <inttypes.h>

/* Socket handling includes */
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/select.h>
#include <ifaddrs.h>
#include <netdb.h>

#include "screen.h"

#include "sarflags.h"

// #include "timestamp.h"

using namespace std;

namespace sarnet
{

/*
 **********************************************************************
 * IP
 **********************************************************************
 */

// Multicast direction
enum mcast_dir
{
	MCAST_IN = 0x00,
	MCAST_OUT = 0x01
};

union in_storage
{
	struct in_addr		v4;
	struct in6_addr		v6;
};

// typedef union inv4or6	IPVER;

class ip
{
private:
	const int OTHER	= -1;
	union in_storage	_ip; // The IP in_addr or in6_addr data
	int			_family; // What are V4 or V6 or OTHER
public:
	// Blank placeholder
	ip() { bzero(&_ip, sizeof(union in_storage)); _family = OTHER; };
	// An IP Address filled in
	ip(string addr);
	// A blank placeholder for a certain protocol
	ip(int protocol) { 
		_family = protocol; 
		if (_family == AF_INET)
			bzero(&_ip.v4, sizeof(struct in_addr)); 
		else if (_family == AF_INET6)
			bzero(&_ip.v6, sizeof(struct in6_addr));
		else
			bzero(&_ip, sizeof(union in_storage));
		return;
	};

	// Given a pointer to another ip create an identical one
	ip(ip *p) {
		_family = p->family();
		bzero(&_ip, sizeof(union in_storage));
		if (_family == AF_INET)
			bcopy(&p->_ip.v4, &_ip.v4, sizeof(struct in_addr));
		else if (_family == AF_INET6)
			bcopy(&p->_ip.v6, &_ip.v6, sizeof(struct in6_addr));
	}

	~ip() { 
		bzero(&_ip, sizeof(union in_storage)); _family = OTHER; 
	};

	ip(struct in_addr* a)
	{
		_family = AF_INET;
		bzero(&_ip, sizeof(union in_storage));
		bcopy(a, &_ip.v4, sizeof(struct in_addr));
	}

	ip(struct in6_addr* a)
	{
		_family = AF_INET6;
		bzero(&_ip, sizeof(union in_storage));
		bcopy(a, &_ip.v6, sizeof(struct in6_addr));
	}

	// Assignment via x = y;
	ip& operator =(const ip& addr) {
		_family = addr._family;
		bzero(&_ip, sizeof(union in_storage));
		bcopy(&addr._ip, &_ip, sizeof(union in_storage));
		return(*this);
	};

	// True is ip x == y
	bool operator ==(const ip &rhs)
	{
		if (_family != rhs._family)
			return(false);
		if (_family == AF_INET)
		{
			if (memcmp(&_ip, &rhs._ip.v4, sizeof(struct in_addr)) != 0)
				return(false);
			return(true);
		}
		if (_family == AF_INET6)
		{
			if (memcmp(&_ip, &rhs._ip.v6, sizeof(struct in6_addr)) != 0)
				return(false);
			return(true);
		}
		if (_family == OTHER)
			return(true);
		return(false);
	};

	bool operator ==(const string s)
	{
		ip	tmp(s);
		
		if (tmp._family != AF_INET && tmp._family != AF_INET6)
			return(false);

		if (_family != tmp._family)
			return(false);
		if (_family == AF_INET)
		{
			if (memcmp(&_ip, &tmp._ip.v4, sizeof(struct in_addr)) != 0)
				return(false);
			return(true);
		}
		if (_family == AF_INET6)
		{
			if (memcmp(&_ip, &tmp._ip.v6, sizeof(struct in6_addr)) != 0)
				return(false);
			return(true);
		}
		return(false);
	}

	// True is ip x != y
	bool operator !=(const ip &rhs)
	{
		if (_family != rhs._family)
			return(true);
		if (_family == AF_INET)
		{
			if (memcmp(&_ip, &rhs._ip.v4, sizeof(struct in_addr)) != 0)
				return(true);
			return(false);
		}
		if (_family == AF_INET6)
		{
			if (memcmp(&_ip, &rhs._ip.v6, sizeof(struct in6_addr)) != 0)
				return(true);
			return(false);
		}
		if (_family == OTHER)
			return(false);
		return(true);
	}

	bool operator !=(const string s)
	{
		ip	tmp(s);
		
		if (tmp._family != AF_INET && tmp._family != AF_INET6)
			return(true);

		if (_family != tmp._family)
			return(true);
		if (_family == AF_INET)
		{
			if (memcmp(&_ip, &tmp._ip.v4, sizeof(struct in_addr)) != 0)
				return(true);
			return(false);
		}
		if (_family == AF_INET6)
		{
			if (memcmp(&_ip, &tmp._ip.v6, sizeof(struct in6_addr)) != 0)
				return(true);
			return(false);
		}
		return(true);
	}

	// What are we v4 or v6
	int family() { return(_family); };
	bool isother() { return(_family == OTHER); };
	bool isv4() { return(_family == AF_INET); };
	bool isv6() { return(_family == AF_INET6); };
	struct in_addr *addr4() { return &_ip.v4; };
	struct in6_addr *addr6() { return &_ip.v6; };

	string	print();
};

/*
 *********************************************************************************
 * INTERFACES lo0, eth0 ...
 *********************************************************************************
 */

class netiface
{
private:
	ip	*_ifip;	// IP Address
	int	_ifindex;	// What index # it is used for V6 multicast
	string	_ifname;	// What interface name it is on eg. eth0
	char	_mac[6];	// What is the MAC address
public:
	netiface(string ipaddr, string ifname) { 
		extern char *if_nametomac(char*, string);
		char s[IF_NAMESIZE];
		_ifip = new ip(ipaddr); 
		strcpy(s, ifname.c_str());
		_ifindex = if_nametoindex(s);
		_ifname = ifname;
		if_nametomac(_mac, ifname);
	};

	~netiface() {};

	string ifname() { return _ifname; };
	int	ifindex() { return _ifindex; };	
	ip *ifip() { return(_ifip); };
	string mac();
	bool ifmatch(string s) { return (s == _ifname); };
	bool ifmatch(ip *myip) { return (*myip == *_ifip); };
	bool familymatch(int fam) { return (fam == _ifip->family()); };

	string print();
};

// System list of network interfaces
class netifaces
{
private:
	std::list<netiface *>	_iface; // List of local IP Addresses
					// associated to interfaces
public:
	netifaces();
	~netifaces() { this->zap(); };

	std::list<netiface *>::iterator begin() { return(_iface.begin()); };
	std::list<netiface *>::iterator end() { return(_iface.end()); };

	void zap() { 
		while (! _iface.empty()) 
		{ 
			delete _iface.front();
			_iface.pop_front(); 
		} 
	};
	
	// Is the address given in our local iface list
	bool islocal(string s)
	{
		ip	tmp(s);
		for (std::list<netiface *>::iterator i = _iface.begin(); i != _iface.end(); i++)
		{
			if ((*i)->ifmatch(&tmp))
				return(true);
		}
		return(false);
	}

	// Return first entry in list for AF_INET or AF_INET6
	// ignoring loopbacks
	netiface *first(int fam)
	{
		for (std::list<netiface *>::iterator i = _iface.begin(); i != _iface.end(); i++)
		{
			if (!(*i)->familymatch(fam))
				continue;
			ip	tmp((*i)->ifip());
			if (tmp != "127.0.0.1" && tmp != "::1")
				return((*i));
		}
		return(NULL);
	}

	string print() {
		string s;
		for (std::list<netiface *>::iterator i = _iface.begin(); i != _iface.end(); i++)
		{
			s += (*i)->print();
			s += "\n";
		}
		return(s);
	};
};

/*
 **********************************************************************
 * UDP
 **********************************************************************
 */

class udpbuffer
{
private:
	char	*_b;
	size_t	_len;
public:
	udpbuffer() { _b = NULL; _len = 0; };

	udpbuffer(char *b, size_t len) {
		_b = new char[len];
		_len = len;
		memcpy(_b, b, _len);
	}

	udpbuffer(udpbuffer *b) {
		_len = b->len();
		_b = new char[_len];
		memcpy(_b, b->buf(), _len);
	}

	~udpbuffer() { };

	char*	buf() { return(_b); };
	size_t	len() { return(_len); };
};
	
class udp
{
private:
	struct sockaddr_storage	_sa; // sockaddr info and it is big enough to hold v4 & v6 info
	int	_fd; // file descriptor
	std::list<udpbuffer>	_buf; // Frames queued to send
	bool	_readytotx;	// Sets FD_SET() or FD_CLR() for tx
	
	static const ssize_t		_jumbosize = 8192; // Jumbo ethernet frame size
	static const ssize_t		_ethsize = 1500; // Normal ethernet frame size
	static const ssize_t		_udpheader = 8; // Size of the udp header
	static const ssize_t		_v4header = 20; // Max Size of an ipv4 header
	static const ssize_t		_v6header = 40; // Size of an ipv6 header
	
	ssize_t				_maxbuff = _ethsize;

	ssize_t _maxframesize() {
		if (this->family() == AF_INET)
			return (_maxbuff - _v4header - _udpheader);
		else
			return (_maxbuff - _v6header - _udpheader);
	}; 
public:

	udp() { _fd = -1; _buf.empty(); bzero(&_sa, sizeof (struct sockaddr_storage)); };
	//IPV4
	// Create a socket to a particular address and port
	udp(string addr, uint16_t port);

	// Create & listen (bind) to inbound sockets on a port
	udp(int proto, uint16_t port);

	udp(struct sockaddr_storage *p);

	// Use for multicast to bind to existing input or output socket	
	udp(enum mcast_dir direction, udp *p, string addr, uint16_t port);

	~udp();

	// Copy a socket assignment
	udp&	operator=(udp& s1) {
		_fd = s1.fd();
		_readytotx = s1.ready();
		bzero(&_sa, sizeof(struct sockaddr_storage));
		bcopy(&_sa, s1.addr(), sizeof(struct sockaddr_storage));
		return(*this);
	};

	bool	operator==(const udp &s)
	{
		return ((_fd == s._fd) &&
			(_readytotx == s._readytotx) &&
			(memcmp(&_sa, &s._sa, 
				sizeof(struct sockaddr_storage)) == 0));
	};

	void zap() 
	{ 
		while (! _buf.empty())
		{
			// delete _buf.front();
			 _buf.pop_front(); 
		}
	};

	ssize_t	framesize() { return _maxframesize(); };

	// What are we v4 or v6
	int family() { 
		struct sockaddr *s = (struct sockaddr *) &_sa;
		return(s->sa_family); 
	};

	// Handle integer setsockopt
	bool setopt(int option, void *optval, socklen_t optsize);

	// Is the option set OK ?
	bool getopt(int option, void *optval, socklen_t *optlen);

	bool set(string addr, uint16_t port);

	// Add a buffer to the list to be sent
	ssize_t tx(char *buf, size_t buflen) {
//		extern string printsarflags(flag_t);
//		flag_t f = 0;
		udpbuffer *tmp;
		
		_readytotx = true;
		tmp = new udpbuffer(buf, buflen);
		_buf.push_back(*tmp); 
//		cerr << "Checking Before TX";
//		memcpy(&f, tmp->buf(), sizeof(flag_t));
//		f = (flag_t) ntohl(f);
//		cerr << printsarflags(f) << endl;
		delete tmp;
		return(buflen);
	}

	// Receive a buffer, return # chars sent - 
	// You catch the error if <0 
	ssize_t	rx(char *buf);

	// Socket #
	uint16_t port();

	// fd
	int	fd() { return(_fd); };

	// Actually transmit the frames in the buffers
	int tx();

	// Are we ready to tx (controls select()
	bool ready() { return _readytotx; };

	// Printable IP Address
	string straddr();

	struct in_addr *addr4() { 
		struct sockaddr_in *p = (struct sockaddr_in *) &_sa;
		return(&p->sin_addr); 
	}

	struct in6_addr *addr6() 
	{
		struct sockaddr_in6 *p = (struct sockaddr_in6 *) &_sa;
		return(&p->sin6_addr);
	}

	struct sockaddr_storage *addr() { return &_sa; };
	
// All the grizly detail
	string print();
/*
	netiface *ifinfo() {
		netifaces	ifaces;
		for (std::list<netiface>::iterator i = ifaces.ifacebegin(); i != ifaces.ifaceend(); i++)
		{
			if (i->ifmatch(&tmp))
				return(true);
		}
		return(false);
	}
 */
		
};

// List of udp opened sockets to write to
class peers
{
private:
	const size_t	_max = 100;	// Maximum number of peers
	std::list<udp>	_peers;		// List of peers
public:
	peers() { };
	~peers() { this->zap(); };

	// Add a UDP peer to the list and open socket to it
	void	add(string addr, uint16_t port);

	std::list<udp>::iterator begin() { return(_peers.begin()); };
	std::list<udp>::iterator end() { return(_peers.end()); };

	// Remove a peer (socket id is the fd) and close the socket;
	// as it calls the destructor for udp socket
	void	remove(int fd) {
		for (std::list<udp>::iterator i = _peers.begin(); i != _peers.end(); i++)
		{
			if (i->fd() == fd)
			{
				//NNEED TO WRITE udp1 = udp (=operator)
				udp	tmp;
				tmp = *i;
				_peers.remove(*i);
				break;
			}
		}
	};

	void	zap() { 
		while (! _peers.empty()) 
		{
			// delete _peers.front();
			_peers.pop_front(); 
		}
	};

	// Is the udp socket in our list of peers
	// if so then return a pointer to it
	udp*	match(sarnet::ip &a);
};

}; // Namespace sarnet

#endif // _IP_H

/*
 * $Log: ip.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.1  2014/10/04 03:09:32  chas
 * Moved all IP related class and routines from ipsock to ip.cpp * ip.h
 * as there is a classto handle ip and a separate class to
 * handle udp now.
 *
 */

