/*
 * $Id: metadata.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */


#include <iostream>
#include <cstring>
#include <string>

#include "saratoga.h"
#include "sarflags.h"
#include "timestamp.h"
#include "frame.h"
#include "./checksums/checksum.h"
#include "metadata.h"

using namespace std;

namespace saratoga
{

// Create metadata frame from scratch 
// with no checksum
metadata::metadata( enum f_descriptor des,
	enum f_transfer tfr,
	enum f_progress prog,
	session_t	session,
	string		fname)
{
	sarwin::screen	msg;
	uint32_t	tmp_32;

	Fversion	version(F_VERSION_1);
	Fframetype	frametype(F_FRAMETYPE_METADATA);
	Fdescriptor	descriptor(des);
	Ftransfer	transfer(tfr); // What are we transferring
	Fprogress	progress(prog); // Are we in progress or terminating
	Fudptype	udptype(F_UDPONLY); // We only do UDP
	Fcsum		csum(F_CSUM_NONE);
	Fcsumlen	csumlen(F_CSUMLEN_NONE);
	
	_flags = 0;
	version.set(&_flags);
	frametype.set(&_flags);
	descriptor.set(&_flags);
	transfer.set(&_flags);
	progress.set(&_flags);
	udptype.set(&_flags);
	csum.set(&_flags);
	csumlen.set(&_flags);

	_session = session;

	// Create the directory entry
	_dirent = new dirent(fname);

	// Well there is no checksum but this keeps printing happy
	// and is consistent with the checksum superclass
	_checksum = new checksums::none(fname);

	// Work out the payload size
	size_t paylen = sizeof(flag_t) + sizeof(session_t);
	paylen += _dirent->paylen();
	paylen += _checksum->length();
	_paylen = paylen;
	_payload = new char[_paylen];

	// Now put the metadata into it ready for transmission
	char *payload = _payload;

	// Session
	tmp_32 = htonl(_session);
	memcpy(payload, &tmp_32, sizeof(session_t));
	payload += sizeof(session_t);

	// Directory Entry
	memcpy(payload, _dirent->payload(), _dirent->paylen());
}

// Create metadata frame from scratch 
// with a checksum
metadata::metadata( enum f_descriptor des,
	enum f_transfer tfr,
	enum f_progress prog,
	enum f_csum	cs,
	session_t	session,
	string		fname)
{
	sarwin::screen	msg;
	uint32_t	tmp_32;

	Fversion	version(F_VERSION_1);
	Fframetype	frametype(F_FRAMETYPE_METADATA);
	Fdescriptor	descriptor(des);
	Ftransfer	transfer(tfr); // What are we transferring
	Fprogress	progress(prog); // Are we in progress or terminating
	Fudptype	udptype(F_UDPONLY); // We only do UDP
	Fcsum		csum(cs);
	Fcsumlen	csumlen(F_CSUMLEN_NONE);

	// Set the flags	
	_flags = 0;
	version.set(&_flags);
	frametype.set(&_flags);
	descriptor.set(&_flags);
	transfer.set(&_flags);
	progress.set(&_flags);
	udptype.set(&_flags);

	// Create the directory entry
	_dirent = new dirent(fname);

	// Session Id
	_session = session;

	// Work out the payload size
	size_t paylen = sizeof(flag_t) + sizeof(session_t) + _dirent->paylen();

	if (_dirent->prop() == D_PROP_FILE || _dirent->prop() == D_PROP_DIR)
	{
		// The appropriate checksum
		switch(csum.get())
		{
		case F_CSUM_NONE:
			csum.set(F_CSUM_NONE);
			csumlen.set(F_CSUMLEN_NONE);
			_checksum = new checksums::none(fname);
			break;
		case F_CSUM_CRC32:
			csumlen.set(F_CSUMLEN_CRC32);
			_checksum = new checksums::crc32(fname);
			break;
		case F_CSUM_MD5:
			csumlen.set(F_CSUMLEN_MD5);
			_checksum = new checksums::md5(fname);
			break;
		case F_CSUM_SHA1:
			csumlen.set(F_CSUMLEN_SHA1);
			_checksum = new checksums::sha1(fname);
			break;
		default:
			msg.error("Invalid Checksum Type Using none");
			csum.set(F_CSUM_NONE);
			csumlen.set(F_CSUMLEN_NONE);
			_checksum = new checksums::none(fname);
		}
	}
	else
	{
		csum.set(F_CSUM_NONE);
		csumlen.set(F_CSUMLEN_NONE);
		_checksum = new checksums::none(fname);
	}
	paylen += ((size_t) _checksum->length() * 4); //# 32 bit values * 4
	csumlen.set(&_flags);
	csum.set(&_flags);

	_paylen = paylen;
	_payload = new char[_paylen];

	// Now copy the metadata into a payload ready for transmission
	char *payload = _payload;

	// Flags
	memcpy(payload, &_flags, sizeof(flag_t));
	payload += sizeof(flag_t);
	// Session
	tmp_32 = htonl(_session);
	memcpy(payload, &tmp_32, sizeof(session_t));
	payload += sizeof(session_t);

	// Checksum
	size_t	clen = _checksum->length();
	uint32_t *vals = _checksum->value();
	for (size_t i = 0; i < clen; i++)
	{
		tmp_32 = (uint32_t) htonl(vals[i]);
		memcpy(payload, &tmp_32, sizeof(uint32_t));
		payload += sizeof(uint32_t);
	}

	// Directory Entry
	memcpy(payload, _dirent->payload(), _dirent->paylen());
	// this->print();
}

/*
 * Given a buffer and length, assemble the metadata
 */
metadata::metadata(char *payload,
	size_t paylen)
{
	uint32_t	tmp_32;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif
	sarwin::screen	msg;

	if (payload == NULL)
	{
		msg.error("metadata(frame): Is NULL\n");
		exit(1);
	}
	// Assemble the frame info
	_paylen = paylen;
	_payload = new char[paylen];
	memcpy(_payload, payload, _paylen);

	// Grab the flags
	memcpy(&_flags, payload, sizeof(flag_t));
	payload += sizeof(flag_t);
	paylen -= sizeof(flag_t);

	Fdescriptor	descriptor(_flags);
	Ftransfer	transfer(_flags);
	Fprogress	progress(_flags);
	Fcsumlen	csumlen(_flags);
	Fcsum		csum(_flags);

	// Session
	memcpy(&tmp_32, payload, sizeof(session_t));
	_session = (session_t) ntohl(tmp_32);
	payload += sizeof(session_t);
	paylen -= sizeof(session_t);

	// Optional checksum
	switch(csum.get())
	{
	case F_CSUM_NONE:
		_checksum = new checksums::none();
		break;
	case F_CSUM_CRC32:
		uint32_t cr;
		memcpy(&cr, payload, sizeof(uint32_t));
		cr = ntohl(cr);
		_checksum = new checksums::crc32(cr);
		payload += sizeof(uint32_t);
		paylen -= sizeof(uint32_t);
		break;
	case F_CSUM_MD5:
		uint32_t md[4];
		for (int i = 0; i < 4; i++)
		{
			memcpy(&md[i], payload, sizeof(uint32_t));
			md[i] = (uint32_t) ntohl(md[i]);
			payload += sizeof(uint32_t);
			paylen -= sizeof(uint32_t);
		}
		_checksum = new checksums::md5(md);
		break;
	case F_CSUM_SHA1:
		uint32_t sh[5];
		for (int i = 0; i < 5; i++)
		{
			memcpy(&sh[i], payload, sizeof(uint32_t));
			md[i] = (uint32_t) ntohl(sh[i]);
			payload += sizeof(uint32_t);
			paylen -= sizeof(uint32_t);
		}
		_checksum = new checksums::sha1(sh);
		break;
	default:
		msg.error("Invalid Checksum Type Using none");
		_checksum = new checksums::none();
	}

	// Directory Entry
	_dirent = new dirent(payload, paylen);
}


// Print out the current flag bits
string
metadata::printflags()
{
	string s("");

	s += "\n    METADATA FLAGS       1                   2                   3\n";
	s += "    |0 1 2 3 4 5 6 7|8 9 0 1 2 3 4 5|6 7 8 9 0 1 2 3|4 5 6 7 8 9 0 1|\n";
	s += "    |";
	for (int i = 0; i < 32; i++)
	{
		flag_t tmpf = _flags;
		tmpf <<= i;
		tmpf >>= 31;
		if (tmpf == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	return s;
}

/*
 * Print out the metadata
 *	flags
 * 	session
 * 	checksum (if required)
 * 	directory entry
 */
string
metadata::print()
{
	char	tmp[128];

	 // These are the valid flags types applicable to a metadata
	Fversion	version(_flags);
	Fframetype	frametype(_flags);
	Fdescriptor	descriptor(_flags);
	Ftransfer	transfer(_flags);
	Fprogress	progress(_flags);
	Fcsumlen	csumlen(_flags);
	Fcsum		csum(_flags);

	Ftstamp		tstamp(_flags);
	Freqstatus	reqstatus(_flags);

	string s(frametype.print());
	s += this->printflags();
	s += "    ";
	s += version.print();
	s += "\n    ";
	s += descriptor.print();
	s += "\n    ";
	s += transfer.print();
	s += "\n    ";
	s += tstamp.print();
	s += "\n    ";
	s += reqstatus.print();
	s += "\n    ";
	sprintf(tmp, 
		"Session: %" PRIu32 "", (uint32_t) _session);
	s += tmp;
	s += "\n    ";
	s += _checksum->print();
	s += "\n    ";
	s += _dirent->print();
	return(s);
}

}; // Namespace saratoga

/*
 * $Log: metadata.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.12  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.11  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.10  2013/10/07 05:08:11  chas
 * Added status frame formatting. Well first pass code for it. We have a class
 * for status and a class for holes. No where near completed. Just a marker.
 *
 * Revision 1.9  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.8  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.7  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.6  2013/06/05 10:09:46  chas
 * Progress on metadata. Some additional work on checksums added constructor that takes in uint32_t's for each type to store them. Now I am assuming in saratoga that the way we transmit checksums is in a series of ntohl (uint32_t) values and that when we receive them we do a htonl to get the values back. Is 1 uint for crc32, 4 for md5 and 5 for sha1. I'll talk to LLoyd and see what we need to add into the internet draft.
 *
 * Revision 1.5  2013/06/01 06:30:09  chas
 * Removed _flags from each individual frame type (BEACON, DATA, METADATA...) and ut the _flags in a friend frame class.
 *
 * Revision 1.4  2013/04/03 05:18:51  chas
 * Major changes yet again to flags. The set function now only needs
 * nnn.set(&_flag) rather than _flag = nnn.set(_flag)
 * much neater and shorter and less eror prone.
 * Still issues with dirent timestamps. Have to have more intelligence
 * in the timestamp constructor's so the right one is chosen.
 *
 * Revision 1.3  2013/04/01 09:57:55  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.2  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 * Revision 1.1  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 */

