/*
 * $Id: metadata.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _METADATA_H
#define _METADATA_H

#include <iostream>
#include <cstring>
#include <string>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "frame.h"
#include "./checksums/checksum.h"
#include "dirent.h"

namespace saratoga
{
/*
 **********************************************************************
 * METADATA
 **********************************************************************
 */

class metadata: public frame
{
private:
	flag_t			_flags; // Frame Flags
 	session_t		_session; // Session #
	checksums::checksum	*_checksum; // WHat the Checksum is
	dirent			*_dirent; // What's in the directory entry
public:

	// This is how we assemble a local metadata
	// To get ready for transmission

	// This is for a metadta with no checksum
	metadata( enum f_descriptor,
		enum f_transfer,
		enum f_progress,
		session_t,	// Session ID
		string);	// File name	

	// This is for metadata with a checksum
	metadata( enum f_descriptor,
		enum f_transfer,
		enum f_progress,
		enum f_csum,	// Type of checksum
		session_t,	// Session ID
		string);	// File name

	// We have received a remote frame that is METADATA
	metadata(char *,	// Pointer to buffer received
		size_t);	// Total length of buffer

	~metadata() {
		delete[] _payload; // Delete frames payload
		_paylen = 0; // Set it's lenght to 0
		if (_checksum != NULL)
			delete _checksum;
		if (_dirent != NULL)
			delete _dirent;
		_session = 0;
		_flags = 0;
	};

	flag_t		flags() { return _flags; };

	// Get various flags applicable to metadata
	enum f_descriptor	descriptor();
	enum f_transfer		transfer();
	enum f_progress		progress();
	enum f_csumlen		csumlen();
	enum f_csum		csum();

	// Set various flags applicable to metadata
	enum f_descriptor	descriptor(f_descriptor);
	enum f_transfer		transfer(f_transfer);
	enum f_progress		progress(f_progress);
	enum f_csumlen		csumlen(f_csumlen);
	enum f_csum		csum(f_csum);
	
	// Yes I am a METADATA frame
	f_frametype type() { return F_FRAMETYPE_METADATA; };

	// What is the session number for this transaction
	session_t	session() { return _session; };

	// The checksum
	checksums::checksum*	checksum() { return _checksum; };

	// The directory entry
	dirent*		dir() { return _dirent; };

	// The number of bytes in the checksum
	size_t		bytes();

	// The metadata frames payload
	char 	*payload() { return _payload; };
	size_t	paylen() { return _paylen; };

	ssize_t tx(sarnet::udp *sock) {
		string s = "METADATA TX: ";
		s += this->print();
		cout << s << endl;
		return(-1);
	};

	ssize_t rx() {
		string s = "METADATA RX: ";
		s += this->print();
		cout << s << endl;
		return(-1);
	};

	string	printflags();
	string	print();
};

} // Namespace saratoga

#endif // _METADATA_H

/*
 * $Log: metadata.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.9  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.8  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.7  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.6  2013/06/05 01:44:19  chas
 * Sorted out frame's
 *
 * Revision 1.5  2013/06/01 06:30:09  chas
 * Removed _flags from each individual frame type (BEACON, DATA, METADATA...) and ut the _flags in a friend frame class.
 *
 * Revision 1.4  2013/04/01 09:57:56  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.3  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 * Revision 1.2  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 */

