/*
 * $Id: sarflags.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

/*
 * Handle the Saratoga Header and Saratoga Directory Entry Sflags Fields
 * We can set them, get them, print them
 *
 * Basic principle is to have an enum that holds the values of each particular
 * flag and then a class to handle variables & functions attaining to that flag.
 *
 * There is lots of bit manipulation going on here. 
 * The saratoga header uses a 32 bit unsigned integer (uint32_t) flags field.
 * The saratoga directory header uses a 16 bit unsigned integer (uint16_t) flags field.
 */

/*
 **********************************************************************************************************
 */

#ifndef _SARFLAGS_H
#define _SARFLAGS_H

#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS	// So we can use STD_u64 in printf's
#endif
#include <inttypes.h>
#include <string>
using namespace std;

#include "saratoga.h"

namespace saratoga
{

/*
 * Saratoga Sflag Header Field Format - 32 bit unsigned integer (flag_t)
 *
 *             111111 11112222 22222233
 *  01234567 89012345 67890123 45678901
 * +--------+--------+--------+--------+
 * |        |        |        |        |
 * +--------+--------+--------+--------+
 *
 *
 * BEACON FRAME FLAGS
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 * |0|0|1|-> Version 1 - f_version
 * | | | |0|0|0|0|0|-> Beacon Frame - f_frametype
 * | | | | | | | | |X|X|-> Descriptor - f_descriptor
 * | | | | | | | | | | |X|-> Bundles - f_bundle
 * | | | | | | | | | | | |X|-> Streaming - f_streams
 * | | | | | | | | | | | | |X|X| | |-> Tx Willing - f_txwilling
 * | | | | | | | | | | | | | | |X|X|-> Rx Willing - f_rxwilling
 * | | | | | | | | | | | | | | | | |X|-> UDP Lite - f_udptype
 * | | | | | | | | | | | | | | | | | |X|-> Freespace Advertise - f_freespace
 * | | | | | | | | | | | | | | | | | | |X|X|-> Freespace Descriptor - f_freespaced
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *
 *******************************************************************
 
 * REQUEST FRAME FLAGS
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 * |0|0|1|-> Version 1 - f_version
 * | | | |0|0|0|0|1|-> Request Frame - f_frametype
 * | | | | | | | | |X|X|-> Descriptor - f_descriptor
 * | | | | | | | | | | |X|-> Bundles - f_bundles
 * | | | | | | | | | | | |X|-> Streams - f_streams
 * | | | | | | | | | | | | |X|X|-> Willingness - f_txwilling
 * | | | | | | | | | | | | | | |X|-> Session Type - f_sessiontype
 * | | | | | | | | | | | | | | | |X|-> File or Directory - f_fileordir
 * | | | | | | | | | | | | | | | | |X|-> UDP Lite - f_udptype
 * | | | | | | | | | | | | | | | | | | | | | | | | |X|X|X|X|X|X|X|X|-> Request Type
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *
 *******************************************************************
 
 * METADATA FRAME FLAGS
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 * |0|0|1|-> Version 1 - f_version
 * | | | |0|0|0|1|0|-> Metadata Frame - f_frametype
 * | | | | | | | | |X|X|-> Descriptor - f_descriptor
 * | | | | | | | | | | |X|X|-> Type of Transfer - f_transfer
 * | | | | | | | | | | | | |X|-> Transfer in Progress - f_progress
 * | | | | | | | | | | | | | |X|-> Reliability - f_udptype
 * | | | | | | | | | | | | | | | | | | | | | | | | |X|X|X|X|-> Checksum Length - f_csumlen
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | |X|X|X|X|-> Checksum Type - f_csum
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *
 *******************************************************************
 
 * DATA FRAME FLAGS
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 * |0|0|1|-> Version 1 - f_version
 * | | | |0|0|0|1|1|-> Data Frame - f_frametype
 * | | | | | | | | |X|X|-> Descriptor - f_descriptor
 * | | | | | | | | | | |X|X|-> Type of Transfer - f_transfer
 * | | | | | | | | | | | | |X|-> Timestamps - f_tstamp
 * | | | | | | | | | | | | | | | |X|-> Request Status - f_reqstatus
 * | | | | | | | | | | | | | | | | |X|-> End of Data - f_eod
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *
 *******************************************************************
 
 * STATUS FRAME FLAGS
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 * |0|0|1|-> Version 1 - f_version
 * | | | |0|0|1|0|0|-> Status Frame - f_frametype
 * | | | | | | | | |X|X|-> Descriptor - f_descriptor
 * | | | | | | | | | | | | |X|-> Timestamp - f_tstamp
 * | | | | | | | | | | | | | |X|->Metadata Received - f_metadatarecvd
 * | | | | | | | | | | | | | | |X|-> All Holes - f_allholes
 * | | | | | | | | | | | | | | | |X|-> Holes Requested or Sent - f_holes
 * | | | | | | | | | | | | | | | | | | | | | | | | |X|X|X|X|X|X|X|X|-> Error Code - f_errcode
 * | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | | |
 *  0                   1                   2                   3  
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *
 *******************************************************************

 *
 * Saratoga Dflag Header Field Format - 16 bit unsigned integer (dflag_t)
 *
 *             1     
 *  01234567 89012345
 * +--------+--------+
 * |        |        |
 * +--------+--------+
 *
 * DIRECTORY ENTRY FLAGS
 *  0                   1           
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 
 * | | | | | | | | | | | | | | | | |
 * |1|-> Bit 0 is always set
 * | | | | | | |X|X|-> Dirent Properties - d_properties
 * | | | | | | | | |X|X|-> Dirent Descriptor - d_descriptor
 * | | | | | | | | | | |X|-> Dirent File or Bundle - d_forb
 * | | | | | | | | | | | | | | | | |
 *  0                   1           
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 
 *
 *******************************************************************

 *
 * Saratoga Tflag Header Field Format - 8 bit unsigned integer (tflag_t)
 *           
 *  01234567 
 * +--------+
 * |        |
 * +--------+
 *
 * TIMESTAMP FLAGS
 *  0 1 2 3 4 5 6 7
 * | | | | | | | | |
 * | | | | | |X|X|X|-> Timestamp Type - t_timestamp
 * | | | | | | | | |
 *  0 1 2 3 4 5 6 7
 *
 *******************************************************************
 */

class Sflag
{
protected:
	flag_t	shift;		// Number of bits to shift 
	flag_t	set;		// The set mask for the bit field covered
public:
	inline Sflag(const flag_t s, const flag_t m) { 
		shift = s; set = m << s; };
	inline ~Sflag()	{ shift = 0; set = 0; };
	inline flag_t	fget(flag_t f) { return((f & set) >> shift); };
	inline flag_t	fset(flag_t c, flag_t f) { 
		return(((c) & ~set) | ((f) << shift)); };
};

/*
 * Saratoga Version Number - Bits 0-1
 * BEACON,REQUEST,METADATA,DATA,STATUS
 */
enum f_version 
{ 
	F_VERSION_0 = 0x00,
	F_VERSION_1 = 0x01
};

class Fversion : private Sflag
{
protected:
	static const flag_t	bits = 3;
	static const flag_t	msb = 0;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x07;
	enum f_version		version;
public:
	// Constructors set the version. 
	// The default is Saratoga V1
	inline Fversion()		: Sflag(fshift, fmask) { version = F_VERSION_1; };
	// We want to expicitly set the Version
	inline Fversion(enum f_version v) : Sflag(fshift, fmask) { version = v; };
	// We have a saratoga 32 bit flag set version to what it holds
	inline Fversion(flag_t f)	: Sflag(fshift, fmask) { version = (enum f_version) fget(f); };

	// Given Current Sflag set the bits for this saratoga version
	inline void		set(flag_t* c) { *c = fset(*c, version); };
	inline void		set(enum f_version v ) { version = v; };
	
	// What is the version
	inline enum f_version	get() { return(version); };
	inline enum f_version	get(flag_t f) { return((enum f_version) fget(f)); };

	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current saratoga version
	string		print();
};

/*
 * Saratoga Frame Type - Bits 3-7
 *  BEACON,REQUEST,METADATA,DATA,STATUS
 */
enum f_frametype 
{ 
	F_FRAMETYPE_BEACON = 0x00,
	F_FRAMETYPE_REQUEST = 0x01,
	F_FRAMETYPE_METADATA = 0x02,
	F_FRAMETYPE_DATA = 0x03,
	F_FRAMETYPE_STATUS = 0x04 
};

class Fframetype : private Sflag
{
protected:
	static const flag_t	bits = 5;
	static const flag_t	msb = 3;
	static const flag_t	fshift = 32 - bits - msb;
	static const flag_t	fmask = 0x1F;
	enum f_frametype		frametype;
public:
	// Constructor set the frametype.
	Fframetype(enum f_frametype f) : Sflag(fshift, fmask) { frametype = f; };
	Fframetype(flag_t f) : Sflag(fshift, fmask) {  frametype = (enum f_frametype) fget(f); };

	// Given Current Sflag set the bits for this frametype
	void		set(flag_t* c) { *c = fset(*c, frametype); };
	void		set(enum f_frametype f ) { frametype = f; };
	
	// What is the frametype
	enum f_frametype	get() { return(frametype); };
	enum f_frametype	get(flag_t f) { return((enum f_frametype) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };
	// Print out the current frametype
	string	print();
};

/*
 * Saratoga Descriptor Size - Bits 8-9
 *  BEACON,REQUEST,METADATA,DATA,STATUS
 */
enum f_descriptor 
{
	F_DESCRIPTOR_16 = 0x00,
	F_DESCRIPTOR_32 = 0x01,
	F_DESCRIPTOR_64 = 0x02,
	F_DESCRIPTOR_128 = 0x03 
};

class Fdescriptor : private Sflag
{
protected:
	static const flag_t	bits = 2;
	static const flag_t	msb = 8;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x03;
	enum f_descriptor		descriptor;
public:
	// Constructor set the descriptor.
	Fdescriptor(enum f_descriptor f) : Sflag(fshift, fmask) { descriptor = f; };
	Fdescriptor(flag_t f) : Sflag(fshift, fmask) { 
		descriptor = (enum f_descriptor) fget(f); };

	// Given Current Sflag set the bits for this descriptor
	void		set(flag_t* c) { *c = fset(*c, descriptor); };
	void		set(enum f_descriptor f ) { descriptor = f; };
	
	// What is the descriptor
	enum f_descriptor	get() { return(descriptor); };
	enum f_descriptor	get(flag_t f) { return((enum f_descriptor) fget(f)); };

	// Number of bytes in descriptor
	size_t		length();
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };
	// Print out the current descriptor
	string		print();
};

/*
 * Saratoga Bundle support - Bit 10
 *  BEACON,REQUEST
 */
enum f_bundle 
{ 
	F_BUNDLE_NO = 0x00,
	F_BUNDLE_YES = 0x01 
};

class Fbundle : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 10;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_bundle		bundle;
public:
	// Constructor set the bundle.
	Fbundle(enum f_bundle f) : Sflag(fshift, fmask) { bundle = f; };
	Fbundle(flag_t f) : Sflag(fshift, fmask) { bundle = (enum f_bundle) fget(f); };

	// Given Current Sflag set the bits for this bundle
	void		set(flag_t* c) { *c = fset(*c, bundle); };
	void		set(enum f_bundle f ) { bundle = f; };
	
	// What is the bundle
	enum f_bundle	get() { return(bundle); };
	enum f_bundle	get(flag_t f) { return((enum f_bundle) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current bundle
	string		print();
};

/*
 * Streams support - Bit 11
 *  BEACON,REQUEST
 */
enum f_streams 
{
	F_STREAMS_NO = 0x00,
	F_STREAMS_YES = 0x01 
};

class Fstreams : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 11;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_streams		streams;
public:
	// Constructor set the streams.
	Fstreams(enum f_streams f) : Sflag(fshift, fmask) { streams = f; };
	Fstreams(flag_t f) : Sflag(fshift, fmask) { streams = (enum f_streams) (fget(f)); };

	// Given Current Sflag set the bits for this streams
	void		set(flag_t* c) { *c = fset(*c, streams); };
	void		set(enum f_streams f ) { streams = f; };
	
	// What is the streams
	enum f_streams	get() { return(streams); };
	enum f_streams	get(flag_t f) { return((enum f_streams) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current streams
	string		print();
};

/* 
 * Transfer Type - Bits 10-11
 *  METADATA,DATA 
 */
enum f_transfer 
{
	F_TRANSFER_FILE = 0x00,
	F_TRANSFER_DIR = 0x01,
	F_TRANSFER_BUNDLE = 0x02,
	F_TRANSFER_STREAM = 0x03 
};

class Ftransfer : private Sflag
{
protected:
	static const flag_t	bits = 2;
	static const flag_t	msb = 10;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x03;
	enum f_transfer		transfer;
public:
	// Constructor set the transfer.
	Ftransfer(enum f_transfer f) : Sflag(fshift, fmask) { transfer = f; };
	Ftransfer(flag_t f) : Sflag(fshift, fmask) { transfer = (enum f_transfer) (fget(f)); };

	// Given Current Sflag set the bits for this transfer
	void		set(flag_t* c) { *c = fset(*c, transfer); };
	void		set(enum f_transfer f ) { transfer = f; };
	
	// What is the transfer
	enum f_transfer	get() { return(transfer); };
	enum f_transfer	get(flag_t f) { return((enum f_transfer) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current transfer
	string		print();
};

/*
 * Timestamp/Nonce - Bit 12
 *  DATA,STATUS
 */
enum f_tstamp
{
	F_TIMESTAMP_NO = 0x00,
	F_TIMESTAMP_YES = 0x01 
};

class Ftstamp : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 12;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_tstamp		tstamp;
public:
	// Constructor set the tstamp.
	Ftstamp(enum f_tstamp f) : Sflag(fshift, fmask) { tstamp = f; };
	Ftstamp(flag_t f) : Sflag(fshift, fmask) { tstamp = (enum f_tstamp) (fget(f)); };

	// Given Current Sflag set the bits for this tstamp
	void		set(flag_t* c) { *c = fset(*c, tstamp); };
	void		set(enum f_tstamp f ) { tstamp = f; };
	
	// What is the tstamp
	enum f_tstamp	get() { return(tstamp); };
	enum f_tstamp	get(flag_t f) { return((enum f_tstamp) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current tstamp
	string		print();
};

/*
 * Tranfer Progress - Bit 12
 *  METADATA
 */
enum f_progress 
{
	F_PROGRESS_INPROG = 0x00,
	F_PROGRESS_TERMINATED = 0x01 
};

class Fprogress : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 12;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_progress		progress;
public:
	// Constructor set the progress.
	Fprogress(enum f_progress f) : Sflag(fshift, fmask) { progress = f; };
	Fprogress(flag_t f) : Sflag(fshift, fmask) { progress = (enum f_progress) (fget(f)); };

	// Given Current Sflag set the bits for this progress
	void		set(flag_t* c) { *c = fset(*c, progress); };
	void		set(enum f_progress f ) { progress = f; };
	
	// What is the progress
	enum f_progress	get() { return(progress); };
	enum f_progress	get(flag_t f) { return((enum f_progress) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current progress
	string		print();
};

/*
 * Transmitter Willingness - Bits 12-13
 *  BEACON,REQUEST
 */
enum f_txwilling 
{
	F_TXWILLING_NO = 0x00,
	F_TXWILLING_INVALID = 0x01,
	F_TXWILLING_CAPABLE = 0x02,
	F_TXWILLING_YES = 0x03 
};

class Ftxwilling : private Sflag
{
protected:
	static const flag_t	bits = 2;
	static const flag_t	msb = 12;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x03;
	enum f_txwilling		txwilling;
public:
	// Constructor set the txwilling.
	Ftxwilling(enum f_txwilling f) : Sflag(fshift, fmask) { txwilling = f; };
	Ftxwilling(flag_t f) : Sflag(fshift, fmask) { txwilling = (enum f_txwilling) (fget(f)); };

	// Given Current Sflag set the bits for this txwilling
	void		set(flag_t* c) { *c = fset(*c, txwilling); };
	void		set(enum f_txwilling f ) { txwilling = f; };
	
	// What is the txwilling
	enum f_txwilling	get() { return(txwilling); };
	enum f_txwilling	get(flag_t f) { return((enum f_txwilling) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current txwilling
	string		print();
};

/*
 * UDP & UDP-Lite Support - Bit 13
 * METADATA
 * This is defined also as bit 16 in BEACON & REQUEST
 * but the flag tpyes are the same and do the same thing just in a different place
 */
 enum f_udptype 
 {
 	F_UDPONLY = 0x00,
 	F_UDPLITE = 0x01 
 };

class Fmetadata_udptype : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 13;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_udptype		udptype;
public:
	// Constructor set the udptype.
	Fmetadata_udptype(enum f_udptype f) : Sflag(fshift, fmask) { udptype = f; };
	Fmetadata_udptype(flag_t f) : Sflag(fshift, fmask) { udptype = (enum f_udptype) (fget(f)); };

	// Given Current Sflag set the bits for this udptype
	void		set(flag_t* c) { *c = fset(*c, udptype); };
	void		set(enum f_udptype f ) { udptype = f; };
	
	// What is the udptype
	enum f_udptype	get() { return(udptype); };
	enum f_udptype	get(flag_t f) { return((enum f_udptype) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current udptype
	string		print();
};

/*
 * Metadata Received - Bit 13
 *  STATUS 
 */
enum f_metadatarecvd 
{
	F_METADATARECVD_YES = 0x00,
	F_METADATARECVD_NO = 0x01 
};

class Fmetadatarecvd : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 13;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_metadatarecvd		metadatarecvd;
public:
	// Constructor set the metadatarecvd.
	Fmetadatarecvd(enum f_metadatarecvd f) : Sflag(fshift, fmask) { metadatarecvd = f; };
	Fmetadatarecvd(flag_t f) : Sflag(fshift, fmask) { metadatarecvd = (enum f_metadatarecvd) (fget(f)); };

	// Given Current Sflag set the bits for this metadatarecvd
	void		set(flag_t* c) { *c = fset(*c, metadatarecvd); };
	void		set(enum f_metadatarecvd f ) { metadatarecvd = f; };
	
	// What is the metadatarecvd
	enum f_metadatarecvd	get() { return(metadatarecvd); };
	enum f_metadatarecvd	get(flag_t f) { return((enum f_metadatarecvd) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current metadatarecvd
	string		print();
};

/*
 * All Holes in this packet - Bit 14
 *  STATUS 
 */
enum f_allholes 
{
	F_ALLHOLES_YES = 0x00,
	F_ALLHOLES_NO = 0x01 
};

class Fallholes : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 14;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_allholes		allholes;
public:
	// Constructor set the allholes.
	Fallholes(enum f_allholes f) : Sflag(fshift, fmask) { allholes = f; };
	Fallholes(flag_t f) : Sflag(fshift, fmask) { allholes = (enum f_allholes) (fget(f)); };

	// Given Current Sflag set the bits for this allholes
	void		set(flag_t* c) { *c = fset(*c, allholes); };
	void		set(enum f_allholes f ) { allholes = f; };
	
	// What is the allholes
	enum f_allholes	get() { return(allholes); };
	enum f_allholes	get(flag_t f) { return((enum f_allholes) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current allholes
	string		print();
};

/*
 * Session Type - Bit 14
 *  REQUEST 
 */
enum f_sessiontype 
{
	F_SESSIONTYPE_GETFILEORDIR = 0x00,
	F_SESSIONTYPE_DELETEFILE = 0x01 
};

class Fsessiontype : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 14;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_sessiontype		sessiontype;
public:
	// Constructor set the sessiontype.
	Fsessiontype(enum f_sessiontype f) : Sflag(fshift, fmask) { sessiontype = f; };
	Fsessiontype(flag_t f) : Sflag(fshift, fmask) { sessiontype = (enum f_sessiontype) (fget(f)); };

	// Given Current Sflag set the bits for this sessiontype
	void		set(flag_t* c) { *c = fset(*c, sessiontype); };
	void		set(enum f_sessiontype f ) { sessiontype = f; };
	
	// What is the sessiontype
	enum f_sessiontype	get() { return(sessiontype); };
	enum f_sessiontype	get(flag_t f) { return((enum f_sessiontype) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current sessiontype
	string		print();
};

/*
 * Receiver Willingness - Bits 14-15
 *  BEACON 
 */
enum f_rxwilling 
{
	F_RXWILLING_NO = 0x00,
	F_RXWILLING_INVALID = 0x01,
	F_RXWILLING_CAPABLE = 0x02,
	F_RXWILLING_YES = 0x03 
};

class Frxwilling : private Sflag
{
protected:
	static const flag_t	bits = 2;
	static const flag_t	msb = 14;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x03;
	enum f_rxwilling		rxwilling;
public:
	// Constructor set the rxwilling.
	Frxwilling(enum f_rxwilling f) : Sflag(fshift, fmask) { rxwilling = f; };
	Frxwilling(flag_t f) : Sflag(fshift, fmask) { rxwilling = (enum f_rxwilling) (fget(f)); };

	// Given Current Sflag set the bits for this rxwilling
	void		set(flag_t* c) { *c = fset(*c, rxwilling); };
	void		set(enum f_rxwilling f ) { rxwilling = f; };
	
	// What is the rxwilling
	enum f_rxwilling	get() { return(rxwilling); };
	enum f_rxwilling	get(flag_t f) { return((enum f_rxwilling) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current rxwilling
	string		print();
};

/*
 * Holes requested or sent voluntarily - Bit 15
 *  STATUS 
 */
enum f_holes 
{
	F_HOLES_REQUESTED = 0x00,
	F_HOLES_SENTVOLUNTARILY = 0x01 
};

class Fholes : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 15;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_holes		holes;
public:
	// Constructor set the holes.
	Fholes(enum f_holes f) : Sflag(fshift, fmask) { holes = f; };
	Fholes(flag_t f) : Sflag(fshift, fmask) { holes = (enum f_holes) (fget(f)); };

	// Given Current Sflag set the bits for this holes
	void		set(flag_t* c) { *c = fset(*c, holes); };
	void		set(enum f_holes f ) { holes = f; };
	
	// What is the holes
	enum f_holes	get() { return(holes); };
	enum f_holes	get(flag_t f) { return((enum f_holes) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current holes
	string		print();
};

/*
 * File or Directory - Bit 15
 *  REQUEST 
 */
enum f_fileordir 
{
	F_FILEORDIR_FILE = 0x00,
	F_FILEORDIR_DIRECTORY = 0x01 
};

class Ffileordir : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 15;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_fileordir		fileordir;
public:
	// Constructor set the fileordir.
	Ffileordir(enum f_fileordir f) : Sflag(fshift, fmask) { fileordir = f; };
	Ffileordir(flag_t f) : Sflag(fshift, fmask) { fileordir = (enum f_fileordir) (fget(f)); };

	// Given Current Sflag set the bits for this fileordir
	void		set(flag_t* c) { *c = fset(*c, fileordir); };
	void		set(enum f_fileordir f ) { fileordir = f; };
	
	// What is the fileordir
	enum f_fileordir	get() { return(fileordir); };
	enum f_fileordir	get(flag_t f) { return((enum f_fileordir) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current fileordir
	string		print();
};

/*
 * Request Status - Bit 15
 *  DATA 
 */
enum f_reqstatus 
{
	F_REQSTATUS_NO = 0x00,
	F_REQSTATUS_YES = 0x01 
};

class Freqstatus : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 15;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_reqstatus		reqstatus;
public:
	// Constructor set the reqstatus.
	Freqstatus(enum f_reqstatus f) : Sflag(fshift, fmask) { reqstatus = f; };
	Freqstatus(flag_t f) : Sflag(fshift, fmask) { reqstatus = (enum f_reqstatus) (fget(f)); };

	// Given Current Sflag set the bits for this reqstatus
	void		set(flag_t* c) { *c = fset(*c, reqstatus); };
	void		set(enum f_reqstatus f ) { reqstatus = f; };
	
	// What is the reqstatus
	enum f_reqstatus	get() { return(reqstatus); };
	enum f_reqstatus	get(flag_t f) { return((enum f_reqstatus) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current reqstatus
	string		print();
};

/*
 * UDP & UDP-Lite Support - Bit 16
 *  BEACON,REQUEST
 * Already defined previously but is same here
 *
 * enum f_udptype 
 * {
 * 	F_UDPONLY = 0x00,
 * 	F_UDPLITE = 0x01 
 * };
 */
class Fudptype : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 16;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_udptype		udptype;
public:
	// Constructor set the udptype.
	Fudptype(enum f_udptype f) : Sflag(fshift, fmask) { udptype = f; };
	Fudptype(flag_t f) : Sflag(fshift, fmask) { udptype = (enum f_udptype) (fget(f)); };

	// Given Current Sflag set the bits for this udptype
	void		set(flag_t* c) { *c = fset(*c, udptype); };
	void		set(enum f_udptype f ) { udptype = f; };
	
	// What is the udptype
	enum f_udptype	get() { return(udptype); };
	enum f_udptype	get(flag_t f) { return((enum f_udptype) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current udptype
	string		print();
};

/*
 * Last Frame in Data Transfer, used to indicate end of file/streams - Bit 16
 *  DATA
 */
enum f_eod 
{
	F_EOD_NO = 0x00,
	F_EOD_YES = 0x01 
};

class Feod : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 16;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_eod		eod;
public:
	// Constructor set the eod.
	Feod(enum f_eod f) : Sflag(fshift, fmask) { eod = f; };
	Feod(flag_t f) : Sflag(fshift, fmask) { eod = (enum f_eod) (fget(f)); };

	// Given Current Sflag set the bits for this eod
	void		set(flag_t* c) { *c = fset(*c, eod); };
	void		set(enum f_eod f ) { eod = f; };
	
	// What is the eod
	enum f_eod	get() { return(eod); };
	enum f_eod	get(flag_t f) { return((enum f_eod) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current eod
	string		print();
};

/*
 * Advertise Available Free Space - Bit 17
 * Applicable to: BEACON
 */
enum f_freespace {
	F_FREESPACE_NO = 0x00,
	F_FREESPACE_YES = 0x01 };

class Ffreespace : private Sflag
{
protected:
	static const flag_t	bits = 1;
	static const flag_t	msb = 17;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x01;
	enum f_freespace	freespace;
public:
	// Constructor set the freespace.
	Ffreespace(enum f_freespace f) : Sflag(fshift, fmask) { freespace = f; };
	Ffreespace(flag_t f) : Sflag(fshift, fmask) { freespace = (enum f_freespace) (fget(f)); };

	// Given Current Sflag set the bits for this freespace
	void		set(flag_t* c) { *c = fset(*c, freespace); };
	void		set(enum f_freespace f ) { freespace = f; };
	
	// What is the freespace
	enum f_freespace	get() { return(freespace); };
	enum f_freespace	get(flag_t f) { return((enum f_freespace) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current freespace
	string		print();
};

/*
 * Advertise Descriptor Size of Free Space - Bits 18-19
 * BEACON
 */
enum f_freespaced {
	F_FREESPACED_16 = 0x00,
	F_FREESPACED_32 = 0x01,
	F_FREESPACED_64 = 0x02,
	F_FREESPACED_128 = 0x03 };

class Ffreespaced : private Sflag
{
protected:
	static const flag_t	bits = 2;
	static const flag_t	msb = 18;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x03;
	enum f_freespaced	freespaced;
public:
	// Constructor set the freespaced.
	Ffreespaced(enum f_freespaced f) : Sflag(fshift, fmask) { freespaced = f; };
	Ffreespaced(flag_t f) : Sflag(fshift, fmask) { freespaced = (enum f_freespaced) (fget(f)); };

	// Given Current Sflag set the bits for this freespaced
	void		set(flag_t* c) { *c = fset(*c, freespaced); };
	void		set(enum f_freespaced f ) { freespaced = f; };
	
	// What is the freespaced
	enum f_freespaced	get() { return(freespaced); };
	enum f_freespaced	get(flag_t f) { return((enum f_freespaced) fget(f)); };

	size_t		length();
	// Print out the current freespaced
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	string		print();
};

/*
 * Checksum Length - Bits 24-27
 * METADATA
 */
enum f_csumlen 
{
	F_CSUMLEN_NONE = 0x00,	// No Checksum
	F_CSUMLEN_CRC32 = 0x01,	// 32 x 1 For CRC32 - 32 Bit
	F_CSUMLEN_MD5 = 0x04,	// 32 x 4 FOr MD5 - 128 Bit
	F_CSUMLEN_SHA1 = 0x05 	// 32 x 5 For SHA-1 - 160 Bit
};

class Fcsumlen : private Sflag
{
protected:
	static const flag_t	bits = 4;
	static const flag_t	msb = 24;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x0F;
	enum f_csumlen		csumlen;
public:
	// Constructor set the csumlen.
	Fcsumlen(enum f_csumlen f) : Sflag(fshift, fmask) { csumlen = f; };
	// Lets make the defaule none
	Fcsumlen(flag_t f) : Sflag(fshift, fmask) { csumlen = (enum f_csumlen) (fget(f)); };

	// Given Current Sflag set the bits for this csumlen
	void		set(flag_t* c) { *c = fset(*c, csumlen); };
	void		set(enum f_csumlen f ) { csumlen = f; };
	
	// What is the csum
	enum f_csumlen	get() { return(csumlen); };
	enum f_csumlen	get(flag_t f) { return((enum f_csumlen) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current csumlen
	string		print();
};

/*
 * Checksum - Bits 28-31
 *  METADATA 
 */
enum f_csum 
{
	F_CSUM_NONE = 0x00,
	F_CSUM_CRC32 = 0x01,
	F_CSUM_MD5 = 0x02,
	F_CSUM_SHA1 = 0x03 
};

class Fcsum : private Sflag
{
protected:
	static const flag_t	bits = 4;
	static const flag_t	msb = 28;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0x0F;
	enum f_csum		csum;
public:
	// Constructor set the csum.
	Fcsum(enum f_csum f) : Sflag(fshift, fmask) { csum = f; };
	Fcsum(flag_t f) : Sflag(fshift, fmask) { csum = (enum f_csum) (fget(f)); };

	// Given Current Sflag set the bits for this csum
	void		set(flag_t* c) { *c = fset(*c, csum); };
	void		set(enum f_csum f ) { csum = f; };
	
	// What is the csum
	enum f_csum	get() { return(csum); };
	enum f_csum	get(flag_t f) { return((enum f_csum) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current csum
	string		print();
};

/*
 * Status = Bits 24-31
 *  STATUS
 */
enum f_errcode 
{
	F_ERRCODE_SUCCESS =		0x00,
	F_ERRCODE_UNSPEC =		0x01,
	F_ERRCODE_NOSEND =		0x02,
	F_ERRCODE_NORECEIVE =		0x03,
	F_ERRCODE_NOFILE =		0x04,
	F_ERRCODE_NOACCESS =		0x05,
	F_ERRCODE_NOID =		0x06,
	F_ERRCODE_NODELETE =		0x07,
	F_ERRCODE_TOOBIG =		0x08,
	F_ERRCODE_BADDESC =		0x09,
	F_ERRCODE_BADPACKET =		0x0A,
	F_ERRCODE_BADFLAG =		0x0B,
	F_ERRCODE_SHUTDOWN =		0x0C,
	F_ERRCODE_PAUSE =		0x0D,
	F_ERRCODE_RESUME =		0x0E,
	F_ERRCODE_INUSE =		0x0F,
	F_ERRCODE_NOMETADATA =		0x10 
};

class Ferrcode : private Sflag
{
protected:
	static const flag_t	bits = 8;
	static const flag_t	msb = 24;
	static const flag_t	fshift = (32 - bits - msb);
	static const flag_t	fmask = 0xFF;
	enum f_errcode		errcode;
public:
	// Constructor set the errcode.
	Ferrcode(enum f_errcode f) : Sflag(fshift, fmask) { errcode = f; };
	Ferrcode(flag_t f) : Sflag(fshift, fmask) { errcode = (enum f_errcode) (fget(f)); };

	// Given Current Sflag set the bits for this errcode
	void		set(flag_t* c) { *c = fset(*c, errcode); };
	void		set(enum f_errcode f ) { errcode = f; };
	
	// What is the errcode
	enum f_errcode	get() { return(errcode); };
	enum f_errcode	get(flag_t f) { return((enum f_errcode) fget(f)); };
	
	flag_t	shift() { return(fshift); };
	flag_t	mask() { return(fmask); };

	// Print out the current errcode
	string		print();
};

} // Namespace saratoga

#endif // _SARFLAGS_H

/*
 * $Log: sarflags.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.19  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.18  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.17  2013/10/07 05:08:11  chas
 * Added status frame formatting. Well first pass code for it. We have a class
 * for status and a class for holes. No where near completed. Just a marker.
 *
 * Revision 1.16  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.15  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.14  2013/06/05 07:20:09  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.13  2013/04/03 05:18:51  chas
 * Major changes yet again to flags. The set function now only needs
 * nnn.set(&_flag) rather than _flag = nnn.set(_flag)
 * much neater and shorter and less eror prone.
 * Still issues with dirent timestamps. Have to have more intelligence
 * in the timestamp constructor's so the right one is chosen.
 *
 * Revision 1.12  2013/04/01 09:57:56  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.11  2013/03/31 02:07:53  chas
 * Timestamp adding a type to handle bundle timestamps which
 * are required in the dirent structure. THese timestamps
 * have an epoch 1 Jan 2000. Not same as posix which is 1970.
 * All the maths is there to handle them as well if you mix
 * and match between the differnt epoch's
 *
 * Revision 1.10  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 * Revision 1.9  2013/03/30 09:16:40  chas
 * Tidying up flag formats and comments.
 *
 * Revision 1.8  2013/03/30 06:17:07  chas
 * Major fixup of flags. We now have the number of bits for each flag
 * and the most significant bit. This along wth the formula
 * 32 - bits - msb gives us the correct shift to do
 *
 * Revision 1.7  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 * Revision 1.6  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.5  2013/03/28 12:23:13  chas
 * Working on better documenting the flags fields and what flags are applicable
 * to what frame types. Created table at top of sarflags.h
 *
 * Revision 1.4  2013/03/28 08:56:35  chas
 * Moe bugs in the flags for data. Im shifting the worng number of bits.
 * Think this will be a generic problem to sort as we go through the flags
 * for each frame type.
 *
 * Revision 1.3  2013/03/28 08:22:37  chas
 * Found an error in flags. Some wrong bit shifting for data flags
 * Progressing along with data class. And making printf's a bit more
 * portable with PRIuXX's for _session and _offset types
 *
 * Revision 1.2  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 * Revision 1.5  2011-02-22 10:19:15  chas
 * Removed nast #defines and replaced with static const for all of the
 * flag bit handling classes
 *
 * Revision 1.4  2011-01-18 10:15:43  chas
 * Refining logging and error printing classes
 *
 * Revision 1.3  2011-01-05 05:28:32  chas
 * Changes to make flags classes available to all modules
 * and starting to get beacon class implemented.
 * The flags.h defines all of the flag classes and flag.c
 * contains the functions to manipulate them. The set functions
 * set a flag based upon its enumarated toye or if you pass
 * it a uint32_t header then it sets the bits in that flag.
 * Get works similarly.
 *
 * Revision 1.2  2011-01-03 23:05:19  chas
 * Changes License to official 3 Clause BSD License with Charles Smith as Author and
 * Vallona Networks as Organisation.
 * This will be used as the official License in all codeset and binaries of this
 * implementation.
 *
 * Revision 1.1  2010-12-30 13:17:26  chas
 * Moved the enumerator types to flags.h and removed their defintions
 * from flags.cpp so other modules can refer to the enumerated types
 *
 * Revision 1.1.1.1  2010-12-30 12:44:56  chas
 * Open source development of Saratoga in C++ under a 2 clause BSD License.
 *
 * Date of commencement December 30 2010 - During the XMAS Holiday break
 *
 * flags.cpp uses enumerated types & classes to handle all flags used in 
 * the saratoga 32 bit header and the 16 bit directory entry headers.
 *
 */

