/*
 * $Id: screen.cpp,v 1.1 2015/01/09 22:17:48 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

/*
 * Handle formatted (printf style) error and debug outputs
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>
#include <cstring>
#include "saratoga.h"
#include "cli.h"
#include "screen.h"
#include "globals.h"

using namespace std;

namespace sarwin
{

WINDOW *errwin;
WINDOW *errbox;
WINDOW *cmdwin;
WINDOW *cmdbox;

// Curses stuff
WINDOW *
create_box(int height, int width, int starty, int startx)
{
	WINDOW	*box_win;

	box_win = newwin(height, width, starty, startx);
	wattron(box_win, COLOR_PAIR(MAGENTA_BLACK));
	box(box_win, 0, 0);
	wrefresh(box_win);
	wattroff(box_win, COLOR_PAIR(MAGENTA_BLACK));
	return(box_win);
}

WINDOW *
create_newwin(int height, int width, int starty, int startx)
{
	WINDOW	*my_win;

	my_win = newwin(height - 2, width - 2, starty + 1, startx + 1);
	wrefresh(my_win);
	return(my_win);
}

void
destroy_win(WINDOW *local_win)
{
	wborder(local_win, ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ');
	wrefresh(local_win);
	delwin(local_win);
}

// Send a string to a window and wrap the text as required 
// Change the output colour to what is specified
void
printwindow(WINDOW *w, char *s, short colorpair)
{
	int column = 0;


	wattron(w, COLOR_PAIR(colorpair));
//	waddstr(w, s);

	for (unsigned int i = 0; i < strlen(s); i++)
	{
		switch(s[i])
		{
		case '\n':
		case '\r':
			column = 0;
	//		waddch(w, s[i]);
			waddch(w, '\n');
			break;
		default:
			column++;
			if (column > COLS - 2)
			{
				waddch(w, '\n');
				column = 0;
			}
			waddch(w, s[i]);
			break;
		}

	}
	wrefresh(w);
	wattroff(w, COLOR_PAIR(colorpair));
}

void
initwindows()
{
	// Initialise curses
	initscr();
	cbreak();
	noecho();
	nonl();
	// Set up the colours
	if (has_colors() != FALSE)
	{
		start_color();
		init_pair(1, COLOR_RED, COLOR_BLACK);
		init_pair(2, COLOR_GREEN, COLOR_BLACK);
		init_pair(3, COLOR_YELLOW, COLOR_BLACK);
		init_pair(4, COLOR_BLUE, COLOR_BLACK);
		init_pair(5, COLOR_MAGENTA, COLOR_BLACK);
		init_pair(6, COLOR_CYAN, COLOR_BLACK);
		init_pair(7, COLOR_WHITE, COLOR_BLACK);
		init_pair(8, COLOR_BLACK, COLOR_RED);
		init_pair(9, COLOR_BLACK, COLOR_GREEN);
		init_pair(10, COLOR_BLACK, COLOR_YELLOW);
		init_pair(11, COLOR_BLACK, COLOR_BLUE);
		init_pair(12, COLOR_BLACK, COLOR_MAGENTA);
		init_pair(13, COLOR_BLACK, COLOR_CYAN);
		init_pair(14, COLOR_BLACK, COLOR_WHITE);
	}
	else
	{
		fprintf(stderr, "Colours not supported\n");
		endwin();
		exit(1);
	}

	// Set up the command line window - bottom half of scr
	cmdbox = create_box(LINES / 2, COLS, LINES / 2, 0);
	cmdwin = create_newwin(LINES / 2, COLS, LINES / 2, 0);
	scrollok(cmdwin, TRUE); // We can scroll a window
	wrefresh(cmdbox);
	wrefresh(cmdwin);
	// Set up the error window - top half of scr
	errbox = create_box(LINES / 2, COLS, 0, 0);
	errwin = create_newwin(LINES / 2, COLS, 0, 0);
	scrollok(errwin, TRUE); // We can scroll the window
	wrefresh(errbox);
	wrefresh(errwin);
}

void
endwindows()
{
	sleep(5);
	destroy_win(errwin);
	destroy_win(errbox);
	destroy_win(cmdwin);
	destroy_win(cmdbox);
	endwin();
}

// *******************************************************************

/*
 * Close existing debug/error output file and open new one
 * but don't close an existing if its stderr or stdout
 */
void
screen::setout(string fname)
{
	extern WINDOW	*errwin;
	FILE	*newfp = NULL;

	char *cname = new char[fname.size() + 1];
	strcpy(cname, fname.c_str());
	if ((newfp = fopen(cname, "a")) == NULL)
	{
		wprintw(errwin, "screen::setout(%s) Can't open for output\n", cname);
		wrefresh(errwin);
	}
	else
		if (_fp != NULL && _fp != stderr && _fp != stdout)
			fclose(_fp);
		_fp = newfp;
	delete[] cname;
}

/*
 * print a single character to command window - white on black
 */
void
screen::std(char c)
{
	extern WINDOW *cmdwin;
	char		cfmt[2];

	cfmt[0] = c;
	cfmt[1] = '\0';
	printwindow(cmdwin, cfmt, WHITE_BLACK);
}

/*
 * print formatted message to command window - white on black
 */
void
screen::std(string fmt, ...)
{
	extern WINDOW *cmdwin;
	char	tmp[2048];
	va_list		args;
	string		sfmt(fmt);

	sfmt += '\n';
	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(tmp, cfmt, args);
	va_end(args);
	printwindow(cmdwin, tmp, WHITE_BLACK);
	delete[] cfmt;
}

/*
 * print formatted prompt message to command window (no cr at end)- white on black
 */
void
screen::prompt(string fmt, ...)
{
	extern WINDOW *cmdwin;
	char	tmp[2048];
	va_list		args;
	string		sfmt(fmt);

	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(tmp, cfmt, args);
	va_end(args);
	printwindow(cmdwin, tmp, WHITE_BLACK);
	delete[] cfmt;
}
/*
 * print formatted information message to command window - yellow on black
 */
void
screen::info(string fmt, ...)
{
	extern WINDOW *cmdwin;
	char	tmp[2048];
	va_list		args;
	string		sfmt(fmt);

	sfmt += "\n";
	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(tmp, cfmt, args);
	va_end(args);
	printwindow(cmdwin, tmp, YELLOW_BLACK);
	delete[] cfmt;
}

/*
 * print formatted information message to error window - yellow on black
 */
void
screen::msg(string fmt, ...)
{
	extern WINDOW *errwin;
	char	tmp[2048];
	va_list		args;
	string		sfmt(fmt);

	sfmt += "\n";
	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(tmp, cfmt, args);
	va_end(args);
	printwindow(errwin, tmp, YELLOW_BLACK);
	delete[] cfmt;
}

/*
 * Print out debug msg if the level is >= setlev to error window - cyan on black
 */
void
screen::debug(int level, string fmt, ...)
{
	extern WINDOW *errwin;
	
	char	tmp[2048];
	char	header[128];
	va_list		args;
	string		sfmt;

	using namespace saratoga;

//	if (_fp == NULL)
//		return;
	// Only output debug's less than the level set
	// e.g. If the level set is 2 then only 2,1,0 debug's are output
	if (level <= c_debug.level())
	{
//		fprintf(_fp, "DEBUG%1d: ", setlev);
		sprintf(header, "DEBUG %d: ", level);
		sfmt = header;
		sfmt += fmt;
		sfmt += "\n";
		char *cfmt = new char[sfmt.size() + 1];
		strcpy(cfmt, sfmt.c_str());
		va_start(args, fmt);
		vsprintf(tmp, cfmt, args);
//		vfprintf(_fp, cfmt, args);
		va_end(args);
		printwindow(errwin, tmp, CYAN_BLACK);
		delete[] cfmt;
//		fflush(_fp);
	}
}

/*
 * Print out error msg to error window - red on black
 */
void
screen::error(string fmt, ...)
{
	extern WINDOW *errwin;
	char	tmp[2048];
	va_list		args;
	string		sfmt("ERROR: ");

//	if (_fp == NULL)
//		return;
	sfmt += fmt;
	sfmt += "\n";
	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
//	vfprintf(_fp, cfmt, args);
	vsprintf(tmp, cfmt, args);
	va_end(args);
	
	printwindow(errwin, tmp, RED_BLACK);
	delete[] cfmt;
	fflush(_fp);
}

/*
 * Print out warning msg to error window - magenta on black
 */
void
screen::warning(string fmt, ...)
{
	extern WINDOW *errwin;
	va_list		args;
	string		sfmt("WARNING: ");
	char	tmp[2048];

	sfmt += fmt;
	sfmt += "\n";
	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(tmp, cfmt, args);
	va_end(args);

	printwindow(errwin, tmp, MAGENTA_BLACK);
	delete[] cfmt;
}

/*
 * Print out formatted perror to error window - red on black
 * we pass in errno
 */
void
screen::perror(int err, string fmt, ...)
{
	extern WINDOW *errwin;
	va_list		args;
	string		sfmt("PERROR: ");
	char		s[1024];
	char		pe[1024];
	char		tmp[2048];

	sfmt += fmt;
	char *cfmt = new char[sfmt.length() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(s, cfmt, args);
	va_end(args);
	errno = err;
	strcpy(pe, strerror(errno));
	sprintf(tmp, "%s %s\n", s, pe);
	printwindow(errwin, tmp, RED_BLACK);
	delete[] cfmt;
}

/*
 * Print out a fatal error to error window and die gracefully - black on red
 */
void
screen::fatal(string fmt, ...)
{
	extern WINDOW *errwin;
	va_list		args;
	string		sfmt("FATAL ERROR: ");
	char		tmp[2048];

	sfmt += fmt;
	sfmt += "\n";
	char *cfmt = new char[sfmt.size() + 1];
	strcpy(cfmt, sfmt.c_str());
	va_start(args, fmt);
	vsprintf(tmp, cfmt, args);
	va_end(args);
	printwindow(errwin, tmp, BLACK_RED);
	endwindows();
	delete[] cfmt;
	exit(1);
}

}; // namespace sarwin

/*
 * $Log: screen.cpp,v $
 * Revision 1.1  2015/01/09 22:17:48  charlesetsmith
 * Screen curses code for all output. We have two windows, the top one is used for
 * error and log messaging and the bottom is used for command line interaction.
 * No you cna;t use curses "getch" or "wgetch" when you are polling for input
 * using a select() call (as we do here in saratoga). So I am using good old
 * getchar(). THis is OK given that all of the user input is always in the
 * bottom window. Actually moving forward I don;t know if this can be solved.
 * So anyway ALL of the curses dependencies are in the "screen.cpp" and "screen.h"
 * files so moving over to a separate windowing or gui system is just going to
 * be a matter of fiddling the finctions in this. I have also put them into
 * a separate namespace "sarwin". THe ONLY interaction between "sarwin" and
 * "saratoga" variables is the c_debug. Which is the global class variable that
 * sets the debug level. You can adjust the debug level to increase/decrease the
 * amount of info coming to you in the log window.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.2  2013/04/01 09:57:55  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 * Revision 1.1  2011-01-18 10:15:43  chas
 * Refining logging and error printing classes
 *
 */

