/*
 * $Id: status.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */


#include <iostream>
#include <cstring>
#include <string>

#include "saratoga.h"
#include "sarflags.h"
#include "timestamp.h"
#include "frame.h"
#include "holes.h"
#include "status.h"

using namespace std;

namespace saratoga
{

// Create a status frame from scratch
status::status( enum f_descriptor des,
	enum f_tstamp 	tsf,
	enum f_metadatarecvd md,
	enum f_allholes ah,
	enum f_holes	ho,
	enum f_errcode	st,	
	session_t	session,
	timestamp	*ts,
	offset_t	progress,
	offset_t	inresponseto,
	offset_t	holcnt,
	hole		h[])
{
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif

	screen	msg;

	Fversion	version(F_VERSION_1);
	Fframetype	frametype(F_FRAMETYPE_STATUS);
	Fdescriptor	descriptor(des);
	Ftstamp		tstamp(tsf);
	Fmetadatarecvd	metadatarecvd(md);
	Fallholes	allholes(ah);
	Fholes		fholes(ho);
	Ferrcode	errcode(st);
	
	// Work out how big our frame has to be and allocate it
	size_t fsize = sizeof(flag_t) + sizeof(session_t);
	fsize += descriptor.length(); // Offset
	if (tstamp.get() == F_TIMESTAMP_YES) // Optional timestamp
	{
		if (ts != NULL)
		{
			_timestamp = new timestamp(ts->ttype(), ts->secs(), ts->nsecs()); //Copy it
			fsize += _timestamp->length();
		}
		else
			tstamp.set(F_TIMESTAMP_NO);
	}
	fsize += (descriptor.length() * 2); // progress, inresponseto
	fsize += (descriptor.length() * 2 * holcnt); // and the holes if any

	_payload = new char[fsize];
	_paylen = fsize;
	char	*sbufp = _payload;

	// Set and copy flags
	_flags = 0;
	version.set(&_flags);
	frametype.set(&_flags);
	descriptor.set(&_flags);
	tstamp.set(&_flags);
	metadatarecvd.set(&_flags);
	allholes.set(&_flags);
	errcode.set(&_flags);	
	memcpy(sbufp, &_flags, sizeof(flag_t));
	sbufp += sizeof(flag_t);

	// Set and copy session
	_session = session;
	tmp_32 = htonl(session);
	memcpy(sbufp, &tmp_32, sizeof(session_t));
	sbufp += sizeof(session_t);

	// Set and copy timestamp if needed
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		char *tbuf = _timestamp->hton();
		memcpy(sbufp, tbuf, _timestamp->length());
		sbufp += _timestamp->length();
	}

	_progress = progress;
	_inresponseto = inresponseto;
	switch(descriptor.get())
	{
	case F_DESCRIPTOR_16:
		tmp_16 = htons((uint16_t) _progress);
		memcpy(sbufp, &tmp_16, sizeof(uint16_t));
		sbufp += sizeof(uint16_t);
		tmp_16 = htons((uint16_t) _inresponseto);
		memcpy(sbufp, &tmp_16, sizeof(uint16_t));
		sbufp += sizeof(uint16_t);
		break;
	case F_DESCRIPTOR_32:
		tmp_32 = htonl((uint32_t) _progress);
		memcpy(sbufp, &tmp_32, sizeof(uint32_t));
		sbufp += sizeof(uint32_t);
		tmp_32 = htonl((uint32_t) _inresponseto);
		memcpy(sbufp, &tmp_32, sizeof(uint32_t));
		sbufp += sizeof(uint32_t);
		break;
	case F_DESCRIPTOR_64:
		tmp_64 = htonll(_progress);
		memcpy(sbufp, &tmp_64, sizeof(uint64_t));
		sbufp += sizeof(uint64_t);
		tmp_64 = htonll(_inresponseto);
		memcpy(sbufp, &tmp_64, sizeof(uint64_t));
		sbufp += sizeof(uint64_t);
		break;
	case F_DESCRIPTOR_128:
	#ifdef UINT128_T
			tmp_128 = htonlll(_progress);
			memcpy(sbufp, &tmp_128, sizeof(uint128_t));
			sbufp += sizeof(uint128_t);
			tmp_128 = htonlll(_inresponseto);
			memcpy(sbufp, &tmp_128, sizeof(uint128_t));
			sbufp += sizeof(uint128_t);
	#else
			msg.error("status(): Descriptor 128 Bit Size Not Supported\n");
			break;
	#endif
	default:
			msg.error("status(): Invalid Descriptor Size");
			exit(1);
		break;
	}

	// And the holes if any
	_holecnt = holcnt;
	for (int i = 0; i < holcnt; i++)
	{
		_holes[i] = new hole(h[i]->start(), h[i]->end());
		switch(descriptor.get())
		{
		case F_DESCRIPTOR_16:
			tmp_16 = htons((uint16_t) _holes[i].start());
			memcpy(sbufp, &tmp_16, sizeof(uint16_t));
			sbufp += sizeof(uint16_t);
			tmp_16 = htons((uint16_t) _holes[i].end());
			memcpy(sbufp, &tmp_16, sizeof(uint16_t));
			sbufp += sizeof(uint16_t);
			break;
		case F_DESCRIPTOR_32:
			tmp_32 = htonl((uint32_t) _holes[i].start());
			memcpy(sbufp, &tmp_32, sizeof(uint32_t));
			sbufp += sizeof(uint32_t);
			tmp_32 = htonl((uint32_t) _holes[i].end());
			memcpy(sbufp, &tmp_32, sizeof(uint32_t));
			sbufp += sizeof(uint32_t);
			break;
		case F_DESCRIPTOR_64:
			tmp_64 = htonll(_holes[i].start());
			memcpy(sbufp, &tmp_64, sizeof(uint64_t));
			sbufp += sizeof(uint64_t);
			tmp_64 = htonll(_holes[i].end());
			memcpy(sbufp, &tmp_64, sizeof(uint64_t));
			sbufp += sizeof(uint64_t);
			break;
		case F_DESCRIPTOR_128:
		#ifdef UINT128_T
				tmp_128 = htonlll(_holes[i].start());
				memcpy(sbufp, &tmp_128, sizeof(uint128_t));
				sbufp += sizeof(uint128_t);
				tmp_128 = htonlll(_holes[i].end());
				memcpy(sbufp, &tmp_128, sizeof(uint128_t));
				sbufp += sizeof(uint128_t);
		#else
				msg.error("status(): Descriptor 128 Bit Size Not Supported\n");
				break;
		#endif
		default:
				msg.error("status(): Invalid Descriptor Size");
				exit(1);
			break;
		}
	}
}

/*
 * Given a buffer and length, assemble the status
 */
status::status(char *payload, size_t paylen)
{
	uint16_t	tmp_16;
	uint32_t	tmp_32;
	uint64_t	tmp_64;
#ifdef UINT128_T
	uint128_t	tmp_128;
#endif
	offset_t	tmp1;
	offset_t	tmp2;
	screen	msg;

	// Copy the frame info
	_paylen = paylen;
	_payload = new char[_paylen];
	memcpy(_payload, payload, paylen);

	// Get the frame info
	// flags and payload

	// Assemble the flags info
	memcpy(&_flags, payload, sizeof(flag_t));
	payload += sizeof(flag_t);
	paylen -= sizeof(flag_t);

	Fdescriptor descriptor(_flags);
	Ftstamp tstamp(_flags);
	Fmetadatarecvd	metadatarecvd(_flags);
	Fallholes	allholes(_flags);
	Ferrcode		errcode(_flags);
	
	// Assemble the status info

	// Session ID
	memcpy(&tmp_32, payload, sizeof(uint32_t));
	_session = (session_t) ntohl(tmp_32);
	payload += sizeof(session_t);
	paylen -= sizeof(session_t);

	// Optional timestamp on status frame
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		_timestamp = new timestamp(payload);
		payload += _timestamp->length();
		paylen -= _timestamp->length();
	}

	// Progress & Inresponseto 
	switch(descriptor.get())
	{
	case F_DESCRIPTOR_16:
		memcpy(&tmp_16, payload, sizeof(uint16_t));
		_progress = (offset_t) ntohs(tmp_16);
		payload += sizeof(uint16_t);
		paylen -= sizeof(uint16_t);
		memcpy(&tmp_16, payload, sizeof(uint16_t));
		_inresponseto = (offset_t) ntohs(tmp_16);
		payload += sizeof(uint16_t);
		paylen -= sizeof(uint16_t);
		break;
	case F_DESCRIPTOR_32:
		memcpy(&tmp_32, payload, sizeof(uint32_t));
		_progress = (offset_t) ntohs(tmp_32);
		payload += sizeof(uint32_t);
		paylen -= sizeof(uint32_t);
		memcpy(&tmp_32, payload, sizeof(uint32_t));
		_inresponseto = (offset_t) ntohs(tmp_32);
		payload += sizeof(uint32_t);
		paylen -= sizeof(uint32_t);
		break;
	case F_DESCRIPTOR_64:
		memcpy(&tmp_64, payload, sizeof(uint64_t));
		_progress = (offset_t) ntohs(tmp_64);
		payload += sizeof(uint64_t);
		paylen -= sizeof(uint64_t);
		memcpy(&tmp_64, payload, sizeof(uint64_t));
		_inresponseto = (offset_t) ntohs(tmp_64);
		payload += sizeof(uint64_t);
		paylen -= sizeof(uint64_t);
		break;
	case F_DESCRIPTOR_128:
#ifdef UINT128_T
		memcpy(&tmp_128, payload, sizeof(uint128_t));
		_progress = (offset_t) ntohs(tmp_128);
		payload += sizeof(uint128_t);
		paylen -= sizeof(uint128_t);
		memcpy(&tmp_128, payload, sizeof(uint128_t));
		_inresponseto = (offset_t) ntohs(tmp_128);
		payload += sizeof(uint128_t);
		paylen -= sizeof(uint128_t);
#else
		msg.error("status(frame): Descriptor 128 Bit Size Not Supported\n");
#endif
		break;
	}

	// Anything left are holes
	_holecnt = paylen / (descriptor.length() * 2);
	for (int i = 0; i < _holecnt; i++)
	{
		switch(descriptor.get())
		{
		case F_DESCRIPTOR_16:
			memcpy(&tmp_16, payload, sizeof(uint16_t));
			tmp1 = (offset_t) ntohs(tmp_16);
			payload += sizeof(uint16_t);
			paylen -= sizeof(uint16_t);
			memcpy(&tmp_16, payload, sizeof(uint16_t));
			tmp2 = (offset_t) ntohs(tmp_16);
			payload += sizeof(uint16_t);
			paylen -= sizeof(uint16_t);
			break;
		case F_DESCRIPTOR_32:
			memcpy(&tmp_32, payload, sizeof(uint32_t));
			tmp1 = (offset_t) ntohs(tmp_32);
			payload += sizeof(uint32_t);
			paylen -= sizeof(uint32_t);
			memcpy(&tmp_32, payload, sizeof(uint32_t));
			tmp2 = (offset_t) ntohs(tmp_32);
			payload += sizeof(uint32_t);
			paylen -= sizeof(uint32_t);
			break;
		case F_DESCRIPTOR_64:
			memcpy(&tmp_64, payload, sizeof(uint64_t));
			tmp1 = (offset_t) ntohs(tmp_64);
			payload += sizeof(uint64_t);
			paylen -= sizeof(uint64_t);
			memcpy(&tmp_64, payload, sizeof(uint64_t));
			tmp2 = (offset_t) ntohs(tmp_64);
			payload += sizeof(uint64_t);
			paylen -= sizeof(uint64_t);
			break;
		case F_DESCRIPTOR_128:
	#ifdef UINT128_T
			memcpy(&tmp_128, payload, sizeof(uint128_t));
			tmp1 = (offset_t) ntohs(tmp_128);
			payload += sizeof(uint128_t);
			paylen -= sizeof(uint128_t);
			memcpy(&tmp_128, payload, sizeof(uint128_t));
			tmp2 = (offset_t) ntohs(tmp_128);
			payload += sizeof(uint128_t);
			paylen -= sizeof(uint128_t);
	#else
			msg.error("status(frame): Descriptor 128 Bit Size Not Supported\n");
	#endif
			break;
		}
		_holes[i] = new hole(tmp1, tmp2);
	}
}

// Print out the current flag bits
string
status::printflags()
{
	string s("");

	// cout << "FLAGS=" << f << endl;
	s += "\n    STATUS FLAGS         1                   2                   3\n";
	s += "    |0 1 2 3 4 5 6 7|8 9 0 1 2 3 4 5|6 7 8 9 0 1 2 3|4 5 6 7 8 9 0 1|\n";
	s += "    |";
	for (int i = 0; i < 32; i++)
	{
		flag_t tmpf = _flags;
		tmpf <<= i;
		tmpf >>= 31;
		if (tmpf == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	return s;
}

/*
 * Print out the status flags & holes
 */
string
status::print()
{
	char	tmp[128];

	/*
	 * These are the valid flags types applicable
	 * to a status
	 */
	Fversion	version(_flags);
	Fframetype	frametype(_flags);
	Fdescriptor	descriptor(_flags);
	Ftstamp tstamp(_flags);
	Fmetadatarecvd	metadatarecvd(_flags);
	Fallholes	allholes(_flags);
	Ferrcode		errcode(_flags);
	
	string s(frametype.print());
	s += this->printflags();
	s += "    ";
	s += version.print();
	s += "\n    ";
	s += descriptor.print();
	s += "\n    ";
	s += tstamp.print();
	s += "\n    ";
	s += metadatarecvd.print();
	s += "\n    ";
	s += allholes.print();
	s += "\n    ";
	s += status.print();
	s += "\n    ";
	sprintf(tmp, 
		"Session: %" PRIu32 "", (uint32_t) _session);
	s += tmp;
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		s += "\n    ";
		timestamp *t = new timestamp;
		s += t->print();
		delete t;
	}
	s += "\n    ";
	sprintf(tmp, 
		"Progress: %" PRIu64 "", (uint64_t) _progress);
	s += tmp;
	s += "\n    ";
	sprintf(tmp, 
		"In Response To: %" PRIu64 "", (uint64_t) _inresponseto);
	s += tmp;
	for (int i = 0; i < _holecnt; i++)
	{
		sprintf(tmp, 
			"Hole[%d]:%" PRIu64 " to %" PRIu64 "", 
				i, 
				(uint64_t) _holes[i].start(), 
				(uint64_t) _holes[i].end());
		s += tmp;
		s += "\n    ";
	}
	return(s);
}

/*
 * Print out the status flags & holes
 */
string
status::print()
{
	char	tmp[128];

	/*
	 * These are the valid flags types applicable
	 * to a status
	 */
	Fversion	version(_flags);
	Fframetype	frametype(_flags);
	Fdescriptor	descriptor(_flags);
	Ftstamp tstamp(_flags);
	Fmetadatarecvd	metadatarecvd(_flags);
	Fallholes	allholes(_flags);
	Ferrcode		errcode(_flags);
	
	string s(frametype.print());
	s += this->printflags();
	s += "    ";
	s += version.print();
	s += "\n    ";
	s += descriptor.print();
	s += "\n    ";
	s += tstamp.print();
	s += "\n    ";
	s += metadatarecvd.print();
	s += "\n    ";
	s += allholes.print();
	s += "\n    ";
	s += status.print();
	s += "\n    ";
	sprintf(tmp, 
		"Session: %" PRIu32 "", (uint32_t) _session);
	s += tmp;
	if (tstamp.get() == F_TIMESTAMP_YES)
	{
		s += "\n    ";
		timestamp *t = new timestamp;
		s += t->print();
		delete t;
	}
	s += "\n    ";
	sprintf(tmp, 
		"Progress: %" PRIu64 "", (uint64_t) _progress);
	s += tmp;
	s += "\n    ";
	sprintf(tmp, 
		"In Response To: %" PRIu64 "", (uint64_t) _inresponseto);
	s += tmp;
	for (int i = 0; i < _holecnt; i++)
	{
		sprintf(tmp, 
			"Hole[%d]:%" PRIu64 " to %" PRIu64 "", 
				i, 
				(uint64_t) _holes[i].start(), 
				(uint64_t) _holes[i].end());
		s += tmp;
		s += "\n    ";
	}
	return(s);
}

// Set various flags applicable to status frames
enum f_metadatarecvd
status::metadatarecvd(enum f_metadatarecvd f)
{
	Fmetadatarecvd	metadatarecvd(f);
	metadatarecvd.set(&_flags);
	return metadatarecvd.get();
}

enum f_allholes
status::allholes(enum f_allholes h)
{
	Fallholes	allholes(h);
	allholes.set(&_flags);
	return allholes.get();
}

enum f_holes
status::fholes(enum f_holes h)
{
	Fholes	fh(h);
	fh.set(&_flags);
	return fh.get();
}

enum f_tstamp
status::tstamp(enum f_tstamp t)
{
	Ftstamp	tstamp(t);
	tstamp.set(&_flags);
	return tstamp.get();
}

enum f_errcode
status::errcode(enum f_errcode t)
{
	Ferrcode	errcode(t);
	errcode.set(&_flags);
	return errcode.get();
}

// Get various flags applicable to status frames
enum f_metadatarecvd
status::metadatarecvd() {
	Fmetadatarecvd t(_flags);
	return t.get();
}

enum f_errcode
status::errcode() {
	Ferrcode s(_flags);
	return s.get();
}

enum f_allholes
status::allholes() {
	Fallholes e(_flags);
	return e.get();
}

enum f_holes
status::holes() {
	Fholes e(_flags);
	return e.get();
}

enum f_tstamp
status::tstamp() {
	Ftstamp t(_flags);
	return t.get();
}

}; // Namespace saratoga

/*
 * $Log: status.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.2  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.1  2013/10/07 05:08:11  chas
 * Added status frame formatting. Well first pass code for it. We have a class
 * for status and a class for holes. No where near completed. Just a marker.
 *
 *
 */

