/*
 * $Id: status.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _STATUS_H
#define _STATUS_H

#include <iostream>
#include <cstring>
#include <string>

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "frame.h"
#include "holes.h"

using namespace std;

namespace saratoga
{
/*
 **********************************************************************
 * STATUS
 **********************************************************************
 */

class status: public frame
{
private:
	flag_t		_flags;		// status's Flags
        session_t	_session;	// The session ID
        timestamp	*_timestamp;	// If we have a timestamp what is it
	offset_t	_progress;	// Progress offset
	offset_t	_inresponseto;	// Covering to offset
	offset_t	_holecnt;	// number of holes in this frame
	offset_t	_maxholes;	// Maximum # holes in a frame
	hole		_holes;	// Holes for this frame
public:

	// This is how we assemble a local status
	// To get ready for transmission
	// The optional timestamp is created within
	// this constructor. There is a data::timestamp()
	// to amend it if needed.
	status( enum f_descriptor,	// Flags
		enum f_tstamp,
		enum f_metadatarecvd,
		enum f_allholes,
		enum f_holes,
		enum f_errcode,
		session_t,	// Session
		timestamp *,	// Optional Timestamp
		offset_t,	// progress
		offset_t,	// inresponseto
		offset_t,	// number of holes
		hole[]);	// the holes
	
	// We have received a remote frame that is STATUS
	status(char *,	// Pointer to buffer received
		size_t);	// Total length of buffer

	~status() {
		delete[] _payload; // From frame
		_paylen = 0; // From frame

		_flags = 0;
		_session = 0;
		if (_timestamp != NULL)
			delete _timestamp;
		_progress = 0;
		_inresponseto = 0;
		_holecnt = 0;
		_maxholes = 0;
		// delete[] _holes;
	};

	// Get various flags applicable to status
	enum f_metadatarecvd	metadatarecvd();
	enum f_errcode		errcode();
	enum f_allholes		allholes();
	enum f_holes		holes();
	enum f_tstamp		tstamp();

	// Set various flags applicable to status
	enum f_metadatarecvd	metadatarecvd(f_metadatarecvd);
	enum f_errcode		errcode(f_errcode);
	enum f_allholes		allholes(f_allholes);
	enum f_holes		holes(f_holes);
	enum f_tstamp		tstamp(f_tstamp);
	
	// What is the session number for this transaction
	session_t	session() { return _session; };

	// Progress and Inresponsto offsets
	offset_t	progress() { return _progress; };
	offset_t	progress(offset_t o) { _progress = o; return _progress; };
	offset_t	inresponseto() { return _inresponseto; };
	offset_t	inresponseto(offset_t o) { _inresponseto = o; return _inresponseto; };

	// The status frames flags
	flag_t	flags() { return _flags; };

	// Yes I am a status frame
	f_frametype type() { return F_FRAMETYPE_STATUS; };

	ssize_t tx(udp *sock) {
		return(sock->tx(_payload, _paylen));
	};

	ssize_t rx() {
		string s = "STATUS RX: ";
		s += this->print();
		cout << s << endl;
		return(-1);
	};

	string	printflags();
	string	print();
};

} // Namespace saratoga

#endif // _STATUS_H

/*
 * $Log: status.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.2  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.1  2013/10/07 05:08:11  chas
 * Added status frame formatting. Well first pass code for it. We have a class
 * for status and a class for holes. No where near completed. Just a marker.
 *
 *
 */

