/*
 * $Id: timestamp.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <iostream>
#include <cstring>
#include <string>

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"

using namespace std;

namespace saratoga
{

/*
 * Given a timestamp buffer assemble the timestamp
 * The timestamp buffer is 16 bytes long with the last
 * byte holding the format and fields held in the previous 15 bytes
 * we use this when we read in a timestamp buffer from the peer
 */
timestamp::timestamp(char b[])
{
	uint32_t	tmp_32;
	uint64_t	tmp_64;
	sarwin::screen	msg;

	_ttype = (t_timestamp) b[15];

	switch(_ttype)
	{
	case T_TSTAMP_UDEF:
		_secs = 0;
		_nsecs = 0;
		break;
	case T_TSTAMP_32:
	case T_TSTAMP_32_Y2K:
		memcpy(&tmp_32, &b[0], sizeof(uint32_t));
		_secs = (int64_t) ntohl(tmp_32);
		_nsecs = 0;
		break;
	case T_TSTAMP_64:
		memcpy(&tmp_64, &b[0], sizeof(uint64_t));
		_secs = (int64_t) ntohll(tmp_64);
		_nsecs = 0;
		break;
	case T_TSTAMP_32_32:
		memcpy(&tmp_32, &b[0], sizeof(uint32_t));
		_secs = (int64_t) ntohl(tmp_32);
		memcpy(&tmp_32, &b[4], sizeof(uint32_t));
		_nsecs = (int64_t) ntohl(tmp_32);
		break;
	case T_TSTAMP_64_32:
		memcpy(&tmp_64, &b[0], sizeof(uint64_t));
		_secs = (int64_t) ntohll(tmp_64);
		memcpy(&tmp_32, &b[8], sizeof(uint32_t));
		_nsecs = (int64_t) ntohl(tmp_32);
		break;
	default:
		msg.error("timestamp(char b[]):Invalid Timestamp Type %u", (uint_t) _ttype);
		_secs = 0;
		_nsecs = 0;
		break;
	}
}

/*
 * Set the timestap to the current Zulu time
 */
timestamp::timestamp(enum t_timestamp t)
{
	time_t		now;
	struct timespec	res;
	sarwin::screen	msg;

	if (clock_getres(CLOCK_REALTIME, &res) < 0)
	{
		/*
		 * Get current time and populate it into our timestamp
		 * we dont have nanosec resolution
		 */
		time(&now);
		switch(t)
		{
		case T_TSTAMP_32:
		case T_TSTAMP_32_32:
		case T_TSTAMP_64:
		case T_TSTAMP_64_32:
			_ttype = t;
			_secs = (int64_t) now;
			_nsecs = 0;
			return;
		case T_TSTAMP_32_Y2K:
			_ttype = t;
			_secs = (int64_t) now;
			_secs += _y2kconv; // Add to get y2k epoch secs
			return;
		default:
			_ttype = T_TSTAMP_UDEF;
			_secs = 0;
			_nsecs = 0;
			return;
		}
	}
	else
	{
		if (clock_gettime(CLOCK_REALTIME, &res) < 0)
		{
			msg.error("timestmap::current() clock_gettime returned error\n");
			_ttype = T_TSTAMP_UDEF;
			_secs = 0;
			_nsecs = 0;
			return;
		}
		switch(t)
		{
		case T_TSTAMP_32:
		case T_TSTAMP_64:
			_secs = (int64_t) res.tv_sec;
			_nsecs = 0;
			_ttype = t;
			return;
		case T_TSTAMP_32_32:
		case T_TSTAMP_64_32:
			_secs = (int64_t) res.tv_sec;
			_nsecs = (int64_t) res.tv_nsec;
			_ttype = t;
			return;
		case T_TSTAMP_32_Y2K:
			_ttype = t;
			_secs = (int64_t) res.tv_sec;
			_secs += _y2kconv; // Add to get y2k epoch secs
			_nsecs = 0;
			return;
		default:
			msg.error("timestmap::current() clock_gettime undefined timestamp\n");
			_secs = 0;
			_nsecs = 0;
			_ttype = T_TSTAMP_UDEF;
			return;
		}
	}	
}

/*
 * Set the timestamp according to an exsting time_t time
 */
timestamp::timestamp(enum t_timestamp t, time_t tm)
{
	Ttimestamp	tstamp(t);
	_ttype = tstamp.set(t);

	switch(tstamp.get())
	{
	case T_TSTAMP_32:
	case T_TSTAMP_32_32:
	case T_TSTAMP_64:
	case T_TSTAMP_64_32:
		_secs = (int64_t) tm;
		_nsecs = 0;
		break;
	case T_TSTAMP_32_Y2K:
		_secs = (int64_t) tm;
		_secs += _y2kconv; // Add to get y2k epoch secs
		break;
	default:
		_ttype = T_TSTAMP_UDEF;
		_secs = 0;
		_nsecs = 0;
		break;
	}
}

/*
 * Set the timestap to the current Zulu time - We use 32_32 as the default
 */
timestamp::timestamp()
{
	time_t		now;
	struct timespec	res;
	sarwin::screen	msg;

	if (clock_getres(CLOCK_REALTIME, &res) < 0)
	{
		/*
		 * Get current time and populate it into our timestamp
		 * we dont have nanosec resolution
		 */
		time(&now);
		_ttype = T_TSTAMP_32_32;
		_secs = (int64_t) now;
		_nsecs = 0;
		return;
	}
	else
	{
		if (clock_gettime(CLOCK_REALTIME, &res) < 0)
		{
			msg.error("timestmap::current() clock_gettime returned error\n");
			_ttype = T_TSTAMP_UDEF;
			_secs = 0;
			_nsecs = 0;
			return;
		}
		_ttype = T_TSTAMP_32_32;
		_secs = (int64_t) res.tv_sec;
		_nsecs = (int64_t) res.tv_nsec;
	}
	return;
}

/*
 * We are done with this timestamp
 */
timestamp::~timestamp()
{
	_ttype = T_TSTAMP_UDEF;
	_secs = 0;
	_nsecs = 0;
}

string
timestamp::print()
{
	if (this == NULL)
		return("No Timestamp");

	Ttimestamp	timestamp(_ttype);
	string		s("");

	s += timestamp.print();
	s += timestamp::asctime();
	return(s);
}

/*
 * Return an ascii readable string of the date and time
 * it is a UTC time and decodes the struct tm
 * it gets rid of that anoying \n returnd by asctime
 */
string
timestamp::asctime()
{
	string s = "";
	char	tmp[128];
	struct tm	*t;
	char		tb[64];
	sarwin::screen	msg;

	/*
	 * we make sure everything is in UTC time zone
	 */
	switch(_ttype)
	{
	case T_TSTAMP_UDEF:
		s += "USER-DEFINED TIMESTAMP";
		break;
	case T_TSTAMP_32:
	case T_TSTAMP_64:
		t = gmtime((time_t *) &_secs);
		asctime_r(t, tb);
		tb[strlen(tb) - 1] = '\0';
		sprintf(tmp, "%s UTC",
			tb);
		s += tmp;
		break;
	case T_TSTAMP_32_Y2K:
		uint64_t	secs;
		secs = _secs - _y2kconv;
		t = gmtime((time_t *) &secs);
		asctime_r(t, tb);
		tb[strlen(tb) - 1] = '\0';
		sprintf(tmp, "%s UTC", tb);
		s += tmp;
		break;
	case T_TSTAMP_32_32:
		t = gmtime((time_t *) &_secs);
		asctime_r(t, tb);
		tb[strlen(tb) - 1] = '\0';
		sprintf(tmp, "%s and %lu ms UTC",
			tb,
			_nsecs / 1000000);
		s += tmp;
		break;
	case T_TSTAMP_64_32:
		t = gmtime((time_t *) &_secs);
		asctime_r(t, tb);
		tb[strlen(tb) - 1] = '\0';
		sprintf(tmp, "%s and %lu ms UTC",
			tb,
			_nsecs / 1000000);
		s += tmp;
		break;
	default:
		msg.error("timestamp::asctime(): Unfamiliar Timestamp Type %u", (uint_t) _ttype);
		s += "UNFAMILIAR TIMESTAMP TYPE";
		break;
	}
	return(s);
}

/*
 * Add two timestamps together.
 */
const timestamp 
operator+(const timestamp& t1, const timestamp& t2)
{
	int64_t	tmpnsecs = 0;
	int64_t	tmpsecs = 0;
	int64_t	tmp1secs = 0;
	int64_t	tmp2secs = 0;

	tmp1secs = t1._secs;
	tmp2secs = t2._secs;
	// Convert to posix epoch if needed
	if (t1._ttype == T_TSTAMP_32_Y2K)
		tmp1secs += t1._y2kconv;
	if (t2._ttype == T_TSTAMP_32_Y2K)
		tmp2secs += t2._y2kconv;

	tmpsecs = tmp1secs + tmp2secs;
	tmpnsecs = t1._nsecs + t2._nsecs;
	tmpsecs += tmpnsecs / 1000000000;
	tmpnsecs %= 1000000000;

	// Convert to y2k if needed
	if (t1._ttype == T_TSTAMP_32_Y2K)
		tmpsecs -= t1._y2kconv;
	
	timestamp	tmp(t1._ttype, tmpsecs, tmpnsecs);
	return(tmp);
}

/*
 * Add n secs to a timestamp.
 */
const timestamp 
operator+(const timestamp& t1, int64_t n)
{
	int64_t	tmpsecs;

	tmpsecs = t1._secs;
	tmpsecs += n;

	timestamp	tmp(t1._ttype, tmpsecs, t1._nsecs);
	return(tmp);
}

/*
 * Subtract timestamp t2 from t1
 */
const timestamp
operator-(const timestamp& t1, const timestamp& t2)
{
	string	s;
	sarwin::screen	msg;

	int64_t	tmpsecs = 0;
	int64_t	tmpnsecs = 0;
	int64_t	tmp1secs = 0;
	int64_t	tmp1nsecs = 0;
	int64_t	tmp2secs = 0;
	int64_t	tmp2nsecs = 0;

	// Convert to posix epoch if needed
	tmp1secs = (int64_t) t1._secs;
	tmp2secs = (int64_t) t2._secs;
	tmp1nsecs = (int64_t) t1._nsecs;
	tmp2nsecs = (int64_t) t2._nsecs;
	if (t1._ttype == T_TSTAMP_32_Y2K)
		tmp1secs += (int64_t) t1._y2kconv;
	if (t2._ttype == T_TSTAMP_32_Y2K)
		tmp2secs += (int64_t) t2._y2kconv;

	tmpsecs = tmp1secs - tmp2secs;
	tmpnsecs = tmp1nsecs - tmp2nsecs;
	if (tmpnsecs < 0)
	{
		tmpnsecs += 1000000000;
		tmpsecs--;
	}

	// Convert to y2k epoch if needed
	if (t1._ttype == T_TSTAMP_32_Y2K)
		tmpsecs -= t1._y2kconv;
	
	timestamp	tmp(t1._ttype, tmpsecs, tmpnsecs);
	return(tmp);
}

/*
 * Subtract n secs from a timestamp.
 */
const timestamp
operator-(const timestamp& t1, int64_t n)
{
	int64_t	tmpsecs;

	tmpsecs = (int64_t) t1._secs;
	tmpsecs -= n;

	timestamp	tmp(t1._ttype, tmpsecs, t1._nsecs);
	return(tmp);
}

/*
 * Convert timestamp structure to network byte order from host order
 * ready for transmission return pointer to static buffer
 */
char *
timestamp::hton()
{
	static char	b[16];	// Our output buffer is always 16 bytes
	uint32_t	tmp_32;
	uint64_t	tmp_64;
	sarwin::screen	msg;

	// Clean slate
	bzero(&b[0], 16);
	switch(_ttype)
	{
	case T_TSTAMP_UDEF:
		b[15] = (char) T_TSTAMP_UDEF;
		return(b);
	case T_TSTAMP_32:
		b[15] = (char) T_TSTAMP_32;
		tmp_32 = (uint32_t) htonl((uint32_t) _secs);
		memcpy(&b[0], &tmp_32, sizeof(uint32_t));
		tmp_32 = (uint32_t) htonl(0);
		memcpy(&b[4], &tmp_32, sizeof(uint32_t));
		return(b);
	case T_TSTAMP_64:
		b[15] = (char) T_TSTAMP_64;
		tmp_64 = (uint64_t) htonll((uint64_t) _secs);
		memcpy(&b[0], &tmp_64, sizeof(uint64_t));
		tmp_32 = (uint32_t) htonl(0);
		memcpy(&b[8], &tmp_32, sizeof(uint32_t));
		return(b);
	case T_TSTAMP_32_32:
		b[15] = (char) T_TSTAMP_32_32;
		tmp_32 = (uint32_t) htonl((uint32_t) _secs);
		memcpy(&b[0], &tmp_32, sizeof(uint32_t));
		tmp_32 = (uint32_t) htonl((uint32_t) _nsecs);
		memcpy(&b[4], &tmp_32, sizeof(uint32_t));
		return(b);
	case T_TSTAMP_64_32:
		b[15] = (char) T_TSTAMP_64_32;
		tmp_64 = (uint64_t) htonll((uint64_t) _secs);
		memcpy(&b[0], &tmp_64, sizeof(uint64_t));
		tmp_32 = (uint32_t) htonl((uint32_t) _nsecs);
		memcpy(&b[8], &tmp_32, sizeof(uint32_t));
		return(b);
	case T_TSTAMP_32_Y2K:
		b[15] = (char) T_TSTAMP_32_Y2K;
		tmp_32 = (uint32_t) htonl((uint32_t) _secs);
		memcpy(&b[0], &tmp_32, sizeof(uint32_t));
		tmp_32 = (uint32_t) htonl(0);
		memcpy(&b[4], &tmp_32, sizeof(uint32_t));
		return(b);
	default:
		msg.error("timestamp.hton(): Invalid Timestamp type %u", (uint32_t) b[15]);
		b[15] = (char) T_TSTAMP_UDEF;
		return(b);
	}
}

/*
 * Convert timestamp structure to host byte order from network order
 */

char *
timestamp::ntoh(char b[])
{
	static char	newb[16];

	uint32_t	tmp_32;
	uint64_t	tmp_64;
	sarwin::screen	msg;

	_ttype = (t_timestamp) b[15];

	bzero(newb, 16);
	switch(_ttype)
	{
	case T_TSTAMP_UDEF:
		memcpy(newb, b, 16);
		break;
	case T_TSTAMP_32:
		memcpy(&tmp_32, &b[0], sizeof(uint32_t));
		tmp_32 = (uint32_t) ntohl(tmp_32);
		memcpy(&newb[0], &tmp_32, sizeof(uint32_t));
		break;
	case T_TSTAMP_64:
		memcpy(&tmp_64, &b[0], sizeof(uint64_t));
		tmp_64 = (uint64_t) ntohll(tmp_64);
		memcpy(&newb[0], &tmp_64, sizeof(uint64_t));
		break;
	case T_TSTAMP_32_32:
		// secs
		memcpy(&tmp_32, &b[0], sizeof(uint32_t));
		tmp_32 = (uint32_t) ntohl(tmp_32);
		memcpy(&newb[0], &tmp_32, sizeof(uint32_t));
		// nsecs
		memcpy(&tmp_32, &b[4], sizeof(uint32_t));
		tmp_32 = (uint32_t) ntohl(tmp_32);
		memcpy(&newb[4], &tmp_32, sizeof(uint32_t));
		break;
	case T_TSTAMP_64_32:
		// secs
		memcpy(&tmp_64, &b[0], sizeof(uint64_t));
		tmp_64 = (uint64_t) ntohll(tmp_64);
		memcpy(&newb[0], &tmp_64, sizeof(uint64_t));
		// nsecs
		memcpy(&tmp_32, &b[8], sizeof(uint32_t));
		tmp_32 = (uint32_t) ntohl(tmp_32);
		memcpy(&newb[8], &tmp_32, sizeof(uint32_t));
		break;
	case T_TSTAMP_32_Y2K:
		memcpy(&tmp_32, &b[0], sizeof(uint32_t));
		tmp_32 = (uint32_t) ntohl(tmp_32);
		memcpy(&newb[0], &tmp_32, sizeof(uint32_t));
		break;
	default:
		msg.error("timestamp(char b[]):Invalid Timestamp Type %u", (uint_t) _ttype);
		break;
	}
	return newb;
}

}; // Namespace saratoga

/*
 * $Log: timestamp.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.10  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.9  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.8  2013/06/08 07:31:50  chas
 * Direcory entry as Gordon Ramsey would say DONE!! So it can read directory entries and get the file type and info to be ready to go into a metadata or read from to get the info.
 *
 * Revision 1.7  2013/06/07 11:49:14  chas
 * Lots if work done on dirent. Still more work to do. It currently core dumps when you put in an invalid file name. Converting a payload back into a direct isn;t working yet either.
 *
 * Revision 1.6  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.5  2013/04/01 09:57:56  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.4  2013/03/31 02:07:53  chas
 * Timestamp adding a type to handle bundle timestamps which
 * are required in the dirent structure. THese timestamps
 * have an epoch 1 Jan 2000. Not same as posix which is 1970.
 * All the maths is there to handle them as well if you mix
 * and match between the differnt epoch's
 *
 * Revision 1.3  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 * Revision 1.2  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 */
