/*
 * $Id: timestamp.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <iostream>
#include <cstring>
#include <string>
#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"

#ifndef _TIMESTAMP_H
#define _TIMESTAMP_H

#define Y2KCONST	946684800 // Seconds from Unix Epoch to Y2K epoch

using namespace std;

namespace saratoga
{

/*
 * Timestamp Entry Tflag Field Format - 8 bit unsigned char (unsigned char)
 *
 *  01234567
 * +--------+
 * |     TTT|
 * +--------+
 *
 * TTT == Timestamp Type (Bits 5-7)
 */
enum t_timestamp
{
	T_TSTAMP_UDEF =         0x00,
	T_TSTAMP_32 =           0x01,
	T_TSTAMP_64 =           0x02,
	T_TSTAMP_32_32 =        0x03,
	T_TSTAMP_64_32 =        0x04,
	T_TSTAMP_32_Y2K =	0x05
};

class Tflag
{
protected:
	tflag_t	shift;		// Number of bits to shift 
	tflag_t	set;		// The set mask for the bit field covered
public:
	inline Tflag(const tflag_t s, const tflag_t m)	{ shift = s; set = m << s; };
	inline ~Tflag()					{ shift = 0; set = 0; };
	inline tflag_t	fget(tflag_t f)	{ return((f & set) >> shift); };
	inline tflag_t	fset(tflag_t c, tflag_t f) { return(((c) & ~set) | ((f) << shift)); };
};

class Ttimestamp : private Tflag
{
protected:
	static const flag_t	bits = 3;
	static const flag_t	msb = 5;
	static const flag_t	tshift = (8 - bits - msb);
	static const tflag_t	tmask = 0x07; // Last 3 bits hold timestamp type
	enum t_timestamp	ttype;
public:
	// Constructor set the timestamp
	Ttimestamp(enum t_timestamp t) : Tflag(tshift, tmask) { ttype = t; };

	// Given Current Tflag set the bits for this timestamp
	enum t_timestamp	set(enum t_timestamp t ) { ttype = t; return(ttype); };
	
	// What is the timestamp
	enum t_timestamp	get() { return(ttype); };

	// Print out the current timestamp flag
	string	print();
};

class timestamp
{
private:
	// Directory Entry records are in Y2K bundle time standard
	// this is the coversion to/from Posix time
	static const offset_t	_y2kconv = 946684800;
	enum t_timestamp	_ttype;
	int64_t		_secs;
	int64_t		_nsecs;
public:
	/*
	 * We can add,increment,compare,subtract,decrement timestamps 
	 * to/from eachother or int64_t number of seconds to/from a timestamp
	 */
	friend const timestamp		operator+(const timestamp& t1, const timestamp& t2);
	friend const timestamp		operator+(const timestamp& t1, const int64_t);

	friend const timestamp		operator-(const timestamp& t1, const timestamp& t2);
	friend const timestamp		operator-(const timestamp& t1, const int64_t);


	// Constructor set the timestamp secs and nanosecs
	timestamp(enum t_timestamp t, int64_t s, int64_t n) { 
		Ttimestamp	tstamp(t);
		_ttype = tstamp.set(t);
		_secs = s;
		_nsecs = n; };

	// Constructor set the timestamp secs
	//timestamp(enum t_timestamp t, offset_t s) {
	//	Ttimestamp	tstamp(t);
	//	_ttype = tstamp.set(t);
	//	_secs = s;
	//	_nsecs = 0; };

	// Constructor set the timestmap secs given a time_t
	timestamp(enum t_timestamp, time_t);
	// timestamp(enum t_timestamp, time_t*);
	
	// Constructir Create a timestamp with current Zulu time
	timestamp();

	// Constrctor set the timestamp to the current Zulu time
	timestamp(enum t_timestamp t);

	// Constructor given a timestamp buffer decode & set it
	timestamp(char *);

	// Destrucotor	
	~timestamp();

	// A timestamp is ALWAYS 16 octets long
	size_t	length() { return 16; };

	// Convert to posix epoch if required
	uint32_t y2k() { if (this->_ttype != T_TSTAMP_32_Y2K)
			return(0);
		return (_y2kconv);
		};

	// Bump me by a second prefix notation ie ++x
	timestamp&	operator++() {
		++_secs;
		return(*this);
	}

	// Bump me by a second postfix notation ie x++
	const timestamp		operator++(int) {
		timestamp	tmp(*this);
		++(*this);
		return(tmp);
	}

	// Bump me by n seconds
	timestamp&	operator+=(int64_t n) {
		_secs += (offset_t) n;
		return(*this);
	}

	// Add a timestamp to current
	timestamp&	operator+=(timestamp t) {
		timestamp	tmp(*this);

		*this = tmp + t;
		return(*this);
	}

	// Unbump me by a second
	timestamp&	operator--() {
		--_secs;
		return(*this);
	}

	// Unbump me by a second postfix notation ie x--
	const timestamp		operator--(int) {
		timestamp	tmp(*this);
		--(tmp);
		return(tmp);
	}

	// Unbump me by n seconds
	timestamp&	operator-=(int64_t n) {
		_secs -= (offset_t) n;
		return(*this);
	}

	// Subtract a timestamp from current
	timestamp&	operator-=(timestamp t) {
		timestamp	tmp(*this);

		*this = tmp - t;
		return(*this);
	}

	// Equality etc operators comparing timestamps
	bool	operator==(timestamp t2) {
			timestamp	tmp(*this);
			int64_t	t1nsec;
			int64_t	t2nsec;
	
			t1nsec = (tmp._secs * 1000000000) + tmp._nsecs;
			t1nsec -= (tmp.y2k() * 1000000000);
			t2nsec = (t2._secs * 1000000000) + t2._nsecs;
			t2nsec -= (t2.y2k() * 1000000000);

			return(t1nsec == t2nsec);
	}
	bool	operator!=(timestamp t2) {
			timestamp	tmp(*this);
			int64_t	t1nsec;
			int64_t	t2nsec;
				
			t1nsec = (tmp._secs * 1000000000) + tmp._nsecs;
			t1nsec -= (tmp.y2k() * 1000000000);
			t2nsec = (t2._secs * 1000000000) + t2._nsecs;
			t2nsec -= (t2.y2k() * 1000000000);
			return(t1nsec != t2nsec);
	}
	bool	operator<=(timestamp t2) {
			timestamp	tmp(*this);
			int64_t	t1nsec;
			int64_t	t2nsec;
				
			t1nsec = (tmp._secs * 1000000000) + tmp._nsecs;
			t1nsec -= (tmp.y2k() * 1000000000);
			t2nsec = (t2._secs * 1000000000) + t2._nsecs;
			t2nsec -= (t2.y2k() * 1000000000);
			return(t1nsec <= t2nsec);
	}
	bool	operator>=(timestamp t2) {
			timestamp	tmp(*this);
			int64_t	t1nsec;
			int64_t	t2nsec;
				
			t1nsec = (tmp._secs * 1000000000) + tmp._nsecs;
			t1nsec -= (tmp.y2k() * 1000000000);
			t2nsec = (t2._secs * 1000000000) + t2._nsecs;
			t2nsec -= (t2.y2k() * 1000000000);
			return(t1nsec >= t2nsec);
	}
	bool	operator>(timestamp t2) {
			timestamp	tmp(*this);
			int64_t	t1nsec;
			int64_t	t2nsec;
				
			t1nsec = (tmp._secs * 1000000000) + tmp._nsecs;
			t1nsec -= (tmp.y2k() * 1000000000);
			t2nsec = (t2._secs * 1000000000) + t2._nsecs;
			t2nsec -= (t2.y2k() * 1000000000);
			return(t1nsec > t2nsec);
	}
	bool	operator<(timestamp t2) {
			timestamp	tmp(*this);
			uint64_t	t1nsec;
			uint64_t	t2nsec;
				
			t1nsec = (tmp._secs * 1000000000) + tmp._nsecs;
			t1nsec -= (tmp.y2k() * 1000000000);
			t2nsec = (t2._secs * 1000000000) + t2._nsecs;
			t2nsec -= (t2.y2k() * 1000000000);
			return(t1nsec < t2nsec);
	}

	// Given Current Tflag set the bits for this timestamp
	t_timestamp	settype(enum t_timestamp t ) {
		Ttimestamp	tstamp(t);
		_ttype = tstamp.set(t); 
		return(_ttype); };

	// Given Current Tflag set the secs for this timestamp
	offset_t	setsecs(int64_t s) { 
		_secs = s; return(_secs); 
	};
	
	// Given Current Tflag set the secs for this timestamp
	offset_t	setnsecs(int64_t n) { 
		_nsecs = n; return(_nsecs); 
	};
	
	// What is the timestamp type
	enum t_timestamp	ttype() { return(_ttype); };
	// What is the timestamp secs
	offset_t		secs() { return(_secs); };
	// What is the timestamp nanosecs
	offset_t		nsecs() { return(_nsecs); };

	uint64_t		y2kconv() { return(_y2kconv); };

	// Return the time in directory entry format which
	// is number of seconds since the Y2K epoch
	// uint32_t	dirtime() { return (uint32_t) (_secs - this->y2k()); };
	uint32_t	dirtime() { return (uint32_t) (_secs); };

	offset_t	dirtimelength() { return sizeof(uint32_t); };

	// Ascii time string of a timestamp
	string	asctime();

	// Convert timestamp to output buffer to send to peer
	char *			hton();
	char *			ntoh(char *);
	
	// Print out the current timestamp
	string	print();
};

} // namespace saratoga

#endif // _TIMESTAMP_H

/*
 * $Log: timestamp.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.11  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.10  2013/06/08 07:31:50  chas
 * Direcory entry as Gordon Ramsey would say DONE!! So it can read directory entries and get the file type and info to be ready to go into a metadata or read from to get the info.
 *
 * Revision 1.9  2013/06/07 11:49:14  chas
 * Lots if work done on dirent. Still more work to do. It currently core dumps when you put in an invalid file name. Converting a payload back into a direct isn;t working yet either.
 *
 * Revision 1.8  2013/06/05 07:20:09  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.7  2013/06/05 01:44:19  chas
 * Sorted out frame's
 *
 * Revision 1.6  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.5  2013/04/01 09:57:56  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.4  2013/03/31 02:07:53  chas
 * Timestamp adding a type to handle bundle timestamps which
 * are required in the dirent structure. THese timestamps
 * have an epoch 1 Jan 2000. Not same as posix which is 1970.
 * All the maths is there to handle them as well if you mix
 * and match between the differnt epoch's
 *
 * Revision 1.3  2013/03/30 11:42:03  chas
 * Working on directory entry class and also a couple of
 * small changes in flags. The dirent has the most significant
 * bit always det to 1. So I needed to add a dirent::init()
 * flag to set it. Also added support in timestamp for
 * standard year 2000 epoch times. Woops just remembered that what
 * I have done is not correct for times. epoch is 1970 of course.
 * Will sort it
 *
 * Revision 1.2  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 */
